      subroutine svd(g,n,m,ndim,mdim,ut,s,v,wk)
c
c.... subroutine svd forms the singular value decomposition of
c     the matrix G = U*S*Vt where G is an (nxm) matrix; U is an
c     (nxn) matrix of data vectors; V is an (mxm) matrix of 
c     model vectors; and S is a vector of singular values,
c     dimensioned (max(n,m)x1)
c     Notes:
c            link with IMSL routine lsvdf
c            array wk must be dimensioned (2*n)
c
      integer i,j,n,m,ndim,mdim
      dimension g(ndim,mdim),ut(ndim,ndim),v(mdim,mdim)
      dimension wk(*),s(mdim)
c
c.... Create a space to store V
      do 10 j = 1,m
      do 15 i = 1,n
            v(i,j) = g(i,j)
15    continue
10    continue
c
c....  create a space to put U, where  Ut = U-transpose
       do 30 j=1,n
       do 20 i=1,n
       if(i.eq.j) ut(i,j) = 1.0
       if(i.ne.j) ut(i,j) = 0.0
20     continue
30     continue
c
c.... Call IMSL SVD routine to find decompostion = u*s*vt
c
       call lsvdf (v,mdim,n,m,ut,ndim,n,s,wk,ier)
c
       return
       end



      subroutine getp(s,n,m,mdim,tol,p)
c
c.....subroutine getp counts the number of non-zero
c     singular values (singular values less than tol.)
c
      integer i,n,m,mx,mdim,p
      real tol
      real s(mdim)
c
      mx=max(n,m)
c
      do 10 i=2,mx
       if(s(i-1).gt.tol) then
         if(s(i).le.tol) p=i-1
       endif
10    continue
c         
       return
       end




      subroutine natinv(ut,s,v,n,m,ndim,mdim,p,ginv,sm1,sm1upt)
c
c.....subroutine natinv calculates the natural inverse of Lanczos
c     of the matrix G . The routine inputs the singular value
c     decompostion of G = U*S*Vt and calculates the inverse
c     Ginv = Vp*Sp**-1*Upt, where the index p denotes the 
c     p largest singular values used to construct the inverse
c
      integer i,j,n,m,ndim,mdim,p
      dimension ut(ndim,ndim),v(mdim,mdim),s(mdim)
      dimension ginv(mdim,ndim),sm1(mdim),sm1upt(mdim,ndim)
      dimension npar(4)
c
      do 10 i=1,p
      sm1(i) = 1.0/s(i)
10    continue
c         
c..... form sm1upt = Sp**-1 * Upt
       npar(1) = p
       npar(2) = 0
       npar(3) = 0
       npar(4) = n
       call vmulbf(sm1,mdim,ut,ndim,npar,sm1upt,mdim)
c
c......form ginv = Vp*sm1upt
       call vmulff(v,sm1upt,m,p,n,mdim,mdim,ginv,mdim,ier)
c
       return
       end




      subroutine natest(ginv,s,d,n,m,p,ndim,mdim,sm1upt,sgn,estm)
c
c.....subroutine natest calculates a model estimate from the 
c     natural inverse, natest = Ginv * d
c
      integer i,j,n,m,ndim,mdim,p
      dimension ginv(mdim,ndim),sm1upt(mdim,ndim)
      dimension d(ndim),sgn(mdim),estm(mdim),s(mdim)
c
c......form estm = ginv * d
       do 100 i=1,m
        estm(i)=0.0
        do 110 j=1,n
         estm(i) = estm(i) + ginv(i,j)*d(j)
110     continue
100    continue
c
c......form sgn
       do 150 i=1,p
        sgn(i)=0.0
        do 160 j=1,n
         sgn(i) = sgn(i) + sm1upt(i,j)*d(j)
160     continue
        sgn(i) = abs(sgn(i)/s(i))
150    continue
c
       return
       end




      subroutine covari(s,v,m,mdim,p,covm,wk,wkar)
c
c.....subroutine covari calculates covariance matrix of the
c     model estimate using the natural (Lanczos) inverse;
c     where covm = Vp*Sp**-2*Vpt.  Vp is the matrix of
c     model vectors, Sp the p non-zero singular values, and 
c     t indicates transpose. 
c
      integer i,j,m,mdim,p,npar(4),ier
      real v(mdim,mdim),s(mdim),covm(mdim,mdim)
      real wk(*),wkar(mdim,mdim)
c
      do 10 i=1,p
      wk(i) = 1.0/(s(i)*s(i))
10    continue
c         
       npar(1) = m
       npar(2) = p
       npar(3) = 0
       npar(4) = 0
       call vmulfb (v,mdim,wk,mdim,npar,wkar,mdim)
       call vmulfp (wkar,v,m,p,m,mdim,mdim,covm,mdim,ier)
c
       return
       end



      subroutine vnull(v,m,mdim,p,v0)
c
c.....subroutine vnull fills the vector v0 with the last m-p
c     columns of the matrix v
c
      integer i,j,m,mdim,p
      dimension v(mdim,mdim),v0(mdim,mdim)
c
      do 10 j=1,m-p
      do 20 i=1,m
        v0(i,j) = v(i,j+p)
20    continue
10    continue
c         
       return
       end




      subroutine modelcrd(v0,m,mdim,p,uinn,qinn,uapr,qapr,umod,qmod)
c
c.....subroutine modelcrd calculates the "model coordinate"
c     displacement of Segall (in press).  This solution 
c     minimizes the two norm of (umod - uapr) where uapr is
c     an apriori displacement set.  The model coordinate 
c     displacements are given by
c            umod = uinn + v0*(v0)t*uapr
c
c            qmod = qinn + [v0*(v0)t]*qapr*[v0*(v0)t]t
c
c     Note: 
c          dimension mmax must be > m
c
      integer mmax
      parameter (mmax=500)
c
      integer i,j,n,m,p,ier,mdim
      real v0(mdim,mdim),uinn(mdim),qinn(mdim,mdim)
      real uapr(mdim),qapr(mdim,mdim),umod(mdim),qmod(mdim,mdim)
      real a(mmax),b(mmax,mmax),c(mmax,mmax),d(mmax,mmax)
c
      if(m.gt.mmax) then
         print *, 'dimension mmax too small in sub modelcrd'
         stop
      endif
c
c.....form v0*(v0)t = c
      call vmulfp(v0,v0,m,m-p,m,mdim,mdim,c,mmax,ier)
c
c.....form umod = uinn + c * uapr
      call vmulff(c,uapr,m,m,1,mmax,mdim,a,mmax,ier)
c
      do 10 i=1,m
        umod(i) = uinn(i) + a(i)
10    continue
c
c.....form model-coordinate solution covariance matrix
c       qmod = qinn + c * qapr * ct
      call vmulff(c,qapr,m,m,m,mmax,mdim,b,mmax,ier)
      call vmulfp(b,c,m,m,m,mmax,mmax,d,mmax,ier)
c
      do 30 j=1,m
      do 20 i=1,m
        qmod(i,j) = qinn(i,j) + d(i,j)
20    continue
30    continue
c
       return
       end



      subroutine outercrd(v0,m,mdim,p,alpha,uinn,qinn,uout,qout)
c
c.....subroutine outercoord calculates the "outer coordinate"
c     displacement of Prescott (198 ).  This solution 
c     minimizes the components of the displacements in the
c     x1' direction.  Alpha measures the angle between the
c     x1 axis (east) and the x1' axis (measured counter-
c     clockwise in degrees).
c     Note: 
c          dimension mmax must be > m
c          dimensions of wkar must be at least 
c           (m-p)**2 + 3*(m-p)
c
      integer mmax,wkmax
      parameter (mmax=500,wkmax=10000)
c
      integer i,j,n,m,p,ier,mdim,iwkdm,npar(4)
      real alpha,tol
      real v0(mdim,mdim),uinn(mdim),qinn(mdim,mdim)
      real uout(mdim),qout(mdim,mdim)
      real b(mmax,mmax),c(mmax,mmax),binv(mmax,mmax)
      real wkar(wkmax),t(mmax,2),so(mmax)
c
      tol=0.0001
c
      if(m.gt.mmax) then
         print *, 'dimension mmax too small in sub outercrd'
         stop
      endif
      iwkdm = (m-p)**2 + 3*(m-p)
      if(iwkdm.gt.wkmax) then
         print *, 'dimension wkmax too small in sub outercrd'
         stop
      endif
c
c.....form transformation matrix t
      do 10 i=1,m/2
       t(2*(i-1)+1,1) = cosd(alpha)
       t(2*(i-1)+2,1) = 0.0
       t(2*(i-1)+1,2) = sind(alpha)
       t(2*(i-1)+2,2) = 0.0
10    continue
c
c.....form product t*v0 = b
      npar(1)=m
      npar(2)=0
      npar(3)=1
      npar(4)=m-p
      call vmulbf(t,mmax,v0,mdim,npar,b,mmax)
c
c.....calculate natural inverse of b = t*v0
      call lginf(b,mmax,m,m-p,tol,binv,mmax,so,wkar,ier)
c
c.....form v0*binv*t
      call vmulff(v0,binv,m,m-p,m,mdim,mmax,c,mmax,ier)
      npar(1)=m
      npar(2)=m
      npar(3)=0
      npar(4)=1
      call vmulfb(c,mmax,t,mmax,npar,b,mmax)
c
c.....form matrix c = [I - v0*binv*t]
      do 20 j=1,m
      do 30 i=1,m
       if(i.eq.j) c(i,j) = 1.0 - b(i,j)
       if(i.ne.j) c(i,j) =  - b(i,j)
30    continue
20    continue
c
c.....form outer-coordinate solution from uout=c*uinn
      call vmulff(c,uinn,m,m,1,mmax,mdim,uout,mdim,ier)
c
c.....form outer-coordinate solution covariance matrix
c       qout = c * qinn * ct
      call vmulff(c,qinn,m,m,m,mmax,mdim,b,mmax,ier)
      call vmulfp(b,c,m,m,m,mmax,mmax,qout,mdim,ier)
c
       return
       end




      subroutine errlps(b,m,mdim,pa,pb,al)
c
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c
c  this subroutine computes error ellipses from the variance-covariance matrix
c  taken from main12.for
c  program by WH Prescott
c  modified slightly be P Segall 2/22/88
c
c  modified by P Segall 1/10/88 so that the axes pa and pb are:
c
c                   x^2                 y^2
c           _______________     +    ________________   =  1
c            Chi^2_p*(s_x)^2         Chi^2_p*(s_y)^2
c  and
c      s_y^2 and s_x^2 are the prinicpal components of the covariance
c      matrix and x, y are associated components of displacement.
c      Chi^2_p is the Chi-squared value corresponding to probability 
c      p (with two degrees of freedom).  The semi axes of the 
c      p error ellipse are thus:
c    
c                       a or b = sqrt(2.0 * Chi^2_p) * s_x or s_y
c      with
c            Probability       90%       95%        99%
c     sqrt(2.0 * Chi^2_p)     2.146     2.448      3.035
c
c     The routine currently returns 95% error ellipses
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c
      dimension b(mdim,1),pa(1),pb(1),al(1)
      do 2 l=2,m,2
      i=l/2
      arg1 = 2. * b(l,l-1)
      arg2 = b(l-1,l-1) - b(l,l)
      if((arg1.ne.0.0).or.(arg2.ne.0.0))  go to 1
      if(b(l-1,l-1).eq.b(l,l)) then
c.......isotropic covariance matrix
c        pa(i) = 2.0*sqrt(b(l,l))
        pa(i) = 2.448*sqrt(b(l,l))
        pb(i) = pa(i)
        g     = 1.570796
        go to 2
      else
c.......zero covariance matrix
        pa(i) = 0.0
        pb(i) = 0.0
        g     = 1.570796
        go to 2
      endif
    1 continue
      g = 0.5 * atan2(arg1,arg2)
      cg = cos(g)**2
      sg = sin(g)**2
      scg = sin(2.*g)
c
      arg3 = b(l-1,l-1) * cg
     +      +b(l,l-1)   * scg
     +      +b(l,l)     * sg
      arg4 = b(l-1,l-1) * sg
     +      -b(l,l-1)   * scg
     +      +b(l,l)     * cg
c
      pa(i) = 0.0
c      if (arg3.ge.0.0) pa(i) = 2. * sqrt(arg3)
      if (arg3.ge.0.0) pa(i) = 2.448 * sqrt(arg3)
      pb(i) = 0.0
c      if (arg4.ge.0.0) pb(i) = 2. * sqrt(arg4)
      if (arg4.ge.0.0) pb(i) = 2.448 * sqrt(arg4)
      if(pa(i).ge.pb(i)) go to 2
      paa = pa(i)
      pa(i) = pb(i)
      pb(i) = paa
      g = g + 1.5707963
    2 al(i) = 90. -g/0.01745329d0


      return
      end
