      subroutine xyz2llh(xyz, llh)
      implicit none
c
      real*8         xyz(3), llh(3)
c
c     ... Local variables
c
      real*8         lat, lon, h
      real*8         a, f, e2, p, r, num, den, n,
     +               deg2rad, oldh
      integer        iter
c
c     ... Code
c
      deg2rad = datan(1.d0)/45.d0
c
c     ... Transform from x,y,z to lat,lon,h
c
      a = 6378137.d0
      f = 1.d0/298.257223563d0
      e2 = 2.d0*f - f*f
      p = dsqrt(xyz(1)*xyz(1) + xyz(2)*xyz(2))
      r = dsqrt(p*p + xyz(3)*xyz(3))
      lon = datan2(xyz(2),xyz(1))
c
c     ... First iteration on lat and h
c            - assumes h = 0
c
      lat = datan2(xyz(3)/p,(1.d0-e2))
      n = a/sqrt((1.d0 - e2*sin(lat)**2))
      h = p/cos(lat) - n
c
c        ... Iterate until h converges (should be quick since h << n)
c
      oldh = -1.d9
      iter = 0
      num = xyz(3)/p
      do while (abs(h - oldh) .gt. 0.0001)
         iter = iter + 1
         oldh = h
         den = 1.d0 - e2*n/(n+h)
         lat = datan2(num,den)
         n = a/sqrt((1.d0 - e2*sin(lat)**2))
         h = p/cos(lat) - n
      enddo
c      write(stderr,*) 'Converged in ', iter, ' iterations'
c
c     ... Unit conversions
c
      llh(1) = lat/deg2rad
      llh(2) = lon/deg2rad
      llh(3) = h
c
      return
      end
