C   IMSL ROUTINE NAME   - LGINF                                         LIVN0010
C                                                                       LIVN0020
C-----------------------------------------------------------------------LIVN0030
C                                                                       LIVN0040
C   COMPUTER            - VAX/SINGLE                                    LIVN0050
C                                                                       LIVN0060
C   LATEST REVISION     - JUNE 1, 1980                                  LIVN0070
C                                                                       LIVN0080
C   PURPOSE             - GENERALIZED INVERSE OF A REAL MATRIX          LIVN0090
C                                                                       LIVN0100
C   USAGE               - CALL LGINF (A,IA,M,N,TOL,AINV,IAINV,S,WK,IER) LIVN0110
C                                                                       LIVN0120
C   ARGUMENTS    A      - M BY N MATRIX. (INPUT)  A IS DESTROYED.       LIVN0130
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          LIVN0140
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LIVN0150
C                           CALLING PROGRAM. (INPUT)                    LIVN0160
C                           A IS USED BY LGINF AS WORK STORAGE FOR AN   LIVN0170
C                           N BY N MATRIX. THEREFORE, IA MUST BE        LIVN0180
C                           GREATER THAN OR EQUAL TO MAX(M,N).          LIVN0190
C                M      - NUMBER OF ROWS IN A. (INPUT)                  LIVN0200
C                N      - NUMBER OF COLUMNS IN A. (INPUT)               LIVN0210
C                TOL    - TOLERANCE PARAMETER. (INPUT)                  LIVN0220
C                           IF TOL IS LESS THAN OR EQUAL TO ZERO ON     LIVN0230
C                           INPUT, LGINF COMPUTES THE GENERALIZED       LIVN0240
C                           INVERSE OF A.                               LIVN0250
C                           IF TOL IS GREATER THAN ZERO ON INPUT, LGINF LIVN0260
C                           COMPUTES THE GENERALIZED INVERSE OF A MATRIXLIVN0270
C                           CLOSE TO A, BUT HAVING CONDITION NUMBER LESSLIVN0280
C                           THAN 1.0/TOL.                               LIVN0290
C                AINV   - N BY M MATRIX. (OUTPUT)                       LIVN0300
C                           AINV CONTAINS THE GENERALIZED INVERSE OF A  LIVN0310
C                           OR THE GENERALIZED INVERSE OF A MATRIX      LIVN0320
C                           CLOSE TO A. SEE TOL ARGUMENT DESCRIPTION.   LIVN0330
C                IAINV  - ROW DIMENSION OF MATRIX AINV EXACTLY AS       LIVN0340
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LIVN0350
C                           CALLING PROGRAM. (INPUT)                    LIVN0360
C                S      - VECTOR OF LENGTH N. S CONTAINS THE ORDERED    LIVN0370
C                           SINGULAR VALUES OF A.  S(1) .GE. S(2),...,  LIVN0380
C                           .GE. S(N) .GE. 0. (OUTPUT)                  LIVN0390
C                WK     - WORK VECTOR OF LENGTH 2N.                     LIVN0400
C                IER    - ERROR PARAMETER. (OUTPUT)                     LIVN0410
C                         WARNING ERROR                                 LIVN0420
C                           IER=33 INDICATES THAT MATRIX A IS NOT       LIVN0430
C                             FULL RANK OR VERY ILL-CONDITIONED. SMALL  LIVN0440
C                             SINGULAR VALUES MAY NOT BE VERY ACCURATE. LIVN0450
C                           IER=34 INDICATES THAT EITHER N.LE.0 OR      LIVN0460
C                             M.LE.0.                                   LIVN0470
C                         TERMINAL ERROR                                LIVN0480
C                           IER=129 INDICATES THAT CONVERGENCE WAS      LIVN0490
C                             NOT OBTAINED BY LSVDB AND COMPUTATION     LIVN0500
C                             WAS DISCONTINUED.                         LIVN0510
C                                                                       LIVN0520
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LIVN0530
C                       - SINGLE/H36,H48,H60                            LIVN0540
C                                                                       LIVN0550
C   REQD. IMSL ROUTINES - LGING,LSVDB,LSVG1,LSVG2,VHS12,UERSET,UERTST,  LIVN0560
C                           UGETIO                                      LIVN0570
C                                                                       LIVN0580
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LIVN0590
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LIVN0600
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LIVN0610
C                                                                       LIVN0620
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LIVN0630
C                                                                       LIVN0640
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LIVN0650
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LIVN0660
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LIVN0670
C                                                                       LIVN0680
C-----------------------------------------------------------------------LIVN0690
C                                                                       LIVN0700
      SUBROUTINE LGINF  (A,IA,M,N,TOL,AINV,IAINV,S,WK,IER)              LIVN0710
C                                                                       LIVN0720
C                                  SPECIFICATIONS FOR ARGUMENTS         LIVN0730
      INTEGER            IA,IAINV,IER,M,N                               LIVN0740
      REAL               A(IA,N),S(N),TOL,WK(N,2),AINV(IAINV,M)         LIVN0750
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LIVN0760
      INTEGER            I,J,K,L                                        LIVN0770
      REAL               T,TOLL                                         LIVN0780
C                                  FIRST EXECUTABLE STATEMENT           LIVN0790
      IER=0                                                             LIVN0800
      DO 10 I=1,N                                                       LIVN0810
         DO 5 J=1,M                                                     LIVN0820
            AINV(I,J)=0.0                                               LIVN0830
            IF (I.EQ.J) AINV(I,J)=1.0                                   LIVN0840
    5    CONTINUE                                                       LIVN0850
   10 CONTINUE                                                          LIVN0860
      CALL LGING(A,IA,M,N,AINV,IAINV,M,S,WK,IER)                        LIVN0870
      IF (IER.GT.128) GO TO 9000                                        LIVN0880
      L = MIN0(M,N)                                                     LIVN0890
C                                  U**T HAS BEEN PLACED IN AINV         LIVN0900
      TOLL = S(1)*AMAX1(TOL,0.0)                                        LIVN0910
C                                  COMPUTE Q**(+) U**T                  LIVN0920
      DO 30 I=1,L                                                       LIVN0930
         IF (S(I).LE.TOLL) GO TO 20                                     LIVN0940
         DO 15 J=1,M                                                    LIVN0950
   15    AINV(I,J)=AINV(I,J)/S(I)                                       LIVN0960
         GO TO 30                                                       LIVN0970
   20    DO 25 J=1,M                                                    LIVN0980
   25    AINV(I,J)=0.0                                                  LIVN0990
   30 CONTINUE                                                          LIVN1000
C                                  COMPUTE V Q**(+) U**T                LIVN1010
      DO 50 J=1,M                                                       LIVN1020
         DO 40 I=1,N                                                    LIVN1030
            T = 0.0                                                     LIVN1040
            DO 35 K=1,L                                                 LIVN1050
   35       T = T+A(I,K)*AINV(K,J)                                      LIVN1060
            WK(I,1)=T                                                   LIVN1070
   40    CONTINUE                                                       LIVN1080
         DO 45 I=1,N                                                    LIVN1090
   45    AINV(I,J)=WK(I,1)                                              LIVN1100
   50 CONTINUE                                                          LIVN1110
      IF (IER.EQ.0) GO TO 9005                                          LIVN1120
 9000 CONTINUE                                                          LIVN1130
      CALL UERTST (IER,6HLGINF )                                        LIVN1140
 9005 RETURN                                                            LIVN1150
      END                                                               LIVN1160
C   IMSL ROUTINE NAME   - LGING                                         LIVO0010
C                                                                       LIVO0020
C-----------------------------------------------------------------------LIVO0030
C                                                                       LIVO0040
C   COMPUTER            - VAX/SINGLE                                    LIVO0050
C                                                                       LIVO0060
C   LATEST REVISION     - JUNE 1, 1981                                  LIVO0070
C                                                                       LIVO0080
C   PURPOSE             - NUCLEUS USED ONLY BY IMSL ROUTINE LGINF       LIVO0090
C                                                                       LIVO0100
C                                                                       LIVO0110
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LIVO0120
C                       - SINGLE/H36,H48,H60                            LIVO0130
C                                                                       LIVO0140
C   REQD. IMSL ROUTINES - LSVDB,LSVG1,LSVG2,VHS12,UERSET,UERTST,UGETIO  LIVO0150
C                                                                       LIVO0160
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LIVO0170
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LIVO0180
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LIVO0190
C                                                                       LIVO0200
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LIVO0210
C                                                                       LIVO0220
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LIVO0230
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LIVO0240
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LIVO0250
C                                                                       LIVO0260
C-----------------------------------------------------------------------LIVO0270
C                                                                       LIVO0280
      SUBROUTINE LGING (A,IA,M,N,B,IB,NB,S,WK,IER)                      LIVO0290
C                                                                       LIVO0300
C                                  SPECIFICATIONS FOR ARGUMENTS         LIVO0310
      INTEGER            IA,M,N,IB,NB,IER                               LIVO0320
      REAL               A(IA,N),B(IB,1),S(N),WK(N,2)                   LIVO0330
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LIVO0340
      INTEGER            I,J,JP1,K,L,LNM,MM,NN,NNP1,NS,NSP1,NM          LIVO0350
      REAL               ZERO,ONE,T,F,FF                                LIVO0360
      DATA               ZERO /0.0/,ONE /1.0/                           LIVO0370
C                                  FIRST EXECUTABLE STATEMENT           LIVO0380
      IER = 0                                                           LIVO0390
C                                  BEGIN SPECIAL FOR ZERO ROWS AND      LIVO0400
C                                    COLS. PACK THE NONZERO COLS TO THE LIVO0410
C                                    LEFT                               LIVO0420
      NN = N                                                            LIVO0430
      IER = 34                                                          LIVO0440
      IF (NN.LE.0 .OR. M.LE.0) GO TO 9005                               LIVO0450
      IER = 0                                                           LIVO0460
      J = NN                                                            LIVO0470
    5 CONTINUE                                                          LIVO0480
      DO 10 I=1,M                                                       LIVO0490
         IF (A(I,J).NE.ZERO) GO TO 25                                   LIVO0500
   10 CONTINUE                                                          LIVO0510
C                                  COL J IS ZERO. EXCHANGE IT WITH COL  LIVO0520
C                                    NN                                 LIVO0530
      IF (J.EQ.NN) GO TO 20                                             LIVO0540
      DO 15 I=1,M                                                       LIVO0550
   15 A(I,J) = A(I,NN)                                                  LIVO0560
   20 CONTINUE                                                          LIVO0570
      A(1,NN) = J                                                       LIVO0580
      NN = NN-1                                                         LIVO0590
   25 CONTINUE                                                          LIVO0600
      J = J-1                                                           LIVO0610
      IF (J.GE.1) GO TO 5                                               LIVO0620
C                                  IF NN=0 THEN A IS ENTIRELY ZERO AND  LIVO0630
C                                    SVD COMPUTATION CAN BE SKIPPED     LIVO0640
      NS = 0                                                            LIVO0650
      IF (NN.EQ.0) GO TO 135                                            LIVO0660
C                                  PACK NONZERO ROWS TO THE TOP QUIT    LIVO0670
C                                    PACKING IF FIND N NONZERO ROWS     LIVO0680
      DO 30 I=1,N                                                       LIVO0690
   30 WK(I,1) = I                                                       LIVO0700
      I = 1                                                             LIVO0710
      MM = M                                                            LIVO0720
   35 IF (I.GT.N .OR. I.GE.MM) GO TO 70                                 LIVO0730
      IF (A(I,I).NE.ZERO) GO TO 45                                      LIVO0740
      DO 40 J=1,NN                                                      LIVO0750
         IF (A(I,J).NE.ZERO) GO TO 45                                   LIVO0760
   40 CONTINUE                                                          LIVO0770
      GO TO 50                                                          LIVO0780
   45 I = I+1                                                           LIVO0790
      GO TO 35                                                          LIVO0800
C                                  ROW I IS ZERO EXCHANGE ROWS I AND MM LIVO0810
C                                  AND RECORD THE PERMUTATION IN WK(I,1)LIVO0820
   50 WK(I,1) = MM                                                      LIVO0830
      DO 55 J=1,NN                                                      LIVO0840
   55 A(I,J) = A(MM,J)                                                  LIVO0850
      IF (MM.GT.NN) GO TO 65                                            LIVO0860
      DO 60 J=1,NN                                                      LIVO0870
   60 A(MM,J) = ZERO                                                    LIVO0880
   65 CONTINUE                                                          LIVO0890
C                                  EXCHANGE IS FINISHED                 LIVO0900
      MM = MM-1                                                         LIVO0910
      GO TO 35                                                          LIVO0920
C                                                                       LIVO0930
   70 CONTINUE                                                          LIVO0940
C                                  END SPECIAL FOR ZERO ROWS AND        LIVO0950
C                                    COLUMNS                            LIVO0960
C                                  BEGIN SVD ALGORITHM..                LIVO0970
C                                  (1) REDUCE THE MATRIX TO UPPER       LIVO0980
C                                    BIDIAGONAL FORM WITH HOUSEHOLDER   LIVO0990
C                                    TRANSFORMATIONS.                   LIVO1000
C                                    H(N)...H(1)AQ(1)...Q(N-2) =        LIVO1010
C                                    (D**T,0)**T WHERE D IS UPPER       LIVO1020
C                                    BIDIAGONAL.                        LIVO1030
C                                  (2) APPLY H(N)...H(1) TO B. HERE     LIVO1040
C                                    H(N)...H(1)*B REPLACES B IN        LIVO1050
C                                    STORAGE.                           LIVO1060
C                                  (3) THE MATRIX PRODUCT W=            LIVO1070
C                                    Q(1)...Q(N-2) OVERWRITES THE FIRST LIVO1080
C                                    N ROWS OF A IN STORAGE.            LIVO1090
C                                  (4) AN SVD FOR D IS COMPUTED. HERE K LIVO1100
C                                    ROTATIONS RI AND PI ARE COMPUTED   LIVO1110
C                                    SO THAT RK...R1*D*P1**(T)...PK**(T)LIVO1120
C                                    = DIAG(S1,...,SM) TO WORKING       LIVO1130
C                                    ACCURACY. THE SI ARE NONNEGATIVE   LIVO1140
C                                    AND NONINCREASING. HERE RK...R1*B  LIVO1150
C                                    OVERWRITES B IN STORAGE WHILE      LIVO1160
C                                    A*P1**(T)...PK**(T) OVERWRITES A   LIVO1170
C                                    IN STORAGE.                        LIVO1180
C                                  (5) IT FOLLOWS THAT,WITH THE PROPER  LIVO1190
C                                    DEFINITIONS, U**(T)*B OVERWRITES   LIVO1200
C                                    B, WHILE V OVERWRITES THE FIRST N  LIVO1210
C                                    ROW AND COLUMNS OF A.              LIVO1220
      L = MIN0(MM,NN)                                                   LIVO1230
C                                  THE FOLLOWING LOOP REDUCES A TO      LIVO1240
C                                    UPPER BIDIAGONAL AND ALSO APPLIES  LIVO1250
C                                    THE PREMULTIPLYING TRANSFORMATIONS LIVO1260
C                                    TO B.                              LIVO1270
      DO 80 J=1,L                                                       LIVO1280
         IF (J.GE.MM) GO TO 75                                          LIVO1290
         JP1 = MIN0(J+1,NN)                                             LIVO1300
         CALL VHS12(1,J,J+1,MM,A(1,J),1,T,A(1,JP1),1,IA,NN-J)           LIVO1310
         S(J) = T                                                       LIVO1320
   75    IF (J.GE.NN-1) GO TO 80                                        LIVO1330
         CALL VHS12(1,J+1,J+2,NN,A(J,1),IA,WK(J,2),A(J+1,1),IA,1,MM-J)  LIVO1340
   80 CONTINUE                                                          LIVO1350
C                                  CONSTRUCT THE FIRST N ROWS OF THE    LIVO1360
C                                    MATRIX HL*...*H2*H1 = B            LIVO1370
      DO 85 JJ=1,L                                                      LIVO1380
         J = L+1-JJ                                                     LIVO1390
         IF (J.GE.MM) GO TO 85                                          LIVO1400
         T = S(J)                                                       LIVO1410
         CALL VHS12(2,J,J+1,MM,A(1,J),1,T,B,IB,1,N)                     LIVO1420
   85 CONTINUE                                                          LIVO1430
C                                  PERMUTE COLUMNS OF B ACCORDING TO THELIVO1440
C                                  PERMUTATIONS MADE TO THE ROWS OF A   LIVO1450
      LNM = MIN0(N,MM)                                                  LIVO1460
      IF (LNM.LT.1) GO TO 100                                           LIVO1470
      DO 95 K=1,LNM                                                     LIVO1480
         J = WK(K,1)                                                    LIVO1490
         IF (J.EQ.K) GO TO 95                                           LIVO1500
C                                  EXCHANGE COLUMNS J AND K             LIVO1510
         DO 90 I=1,N                                                    LIVO1520
            T = B(I,K)                                                  LIVO1530
            B(I,K) = B(I,J)                                             LIVO1540
            B(I,J) = T                                                  LIVO1550
   90    CONTINUE                                                       LIVO1560
   95 CONTINUE                                                          LIVO1570
  100 CONTINUE                                                          LIVO1580
C                                  COPY THE BIDIAGONAL MATRIX INTO THE  LIVO1590
C                                    ARRAY S FOR LSVDB                  LIVO1600
      IF (L.EQ.1) GO TO 110                                             LIVO1610
      DO 105 J=2,L                                                      LIVO1620
         S(J) = A(J,J)                                                  LIVO1630
         WK(J,1) = A(J-1,J)                                             LIVO1640
  105 CONTINUE                                                          LIVO1650
  110 S(1) = A(1,1)                                                     LIVO1660
C                                                                       LIVO1670
      NS = NN                                                           LIVO1680
      IF (MM.GE.NN) GO TO 115                                           LIVO1690
      NS = MM+1                                                         LIVO1700
      S(NS) = ZERO                                                      LIVO1710
      WK(NS,1) = A(MM,MM+1)                                             LIVO1720
  115 CONTINUE                                                          LIVO1730
C                                  CONSTRUCT THE EXPLICIT NN BY NN      LIVO1740
C                                    PRODUCT MATRIX, W=Q1*Q2*...*QL*I   LIVO1750
C                                    IN THE ARRAY A                     LIVO1760
      DO 130 K=1,NN                                                     LIVO1770
         I = NN+1-K                                                     LIVO1780
         IF (I.GT.MIN0(MM,NN-2)) GO TO 120                              LIVO1790
         CALL VHS12(2,I+1,I+2,NN,A(I,1),IA,WK(I,2),A(1,I+1),1,IA,NN-I)  LIVO1800
  120    DO 125 J=1,NN                                                  LIVO1810
  125    A(I,J) = ZERO                                                  LIVO1820
         A(I,I) = ONE                                                   LIVO1830
  130 CONTINUE                                                          LIVO1840
C                                  COMPUTE THE SVD OF THE BIDIAGONAL    LIVO1850
C                                    MATRIX                             LIVO1860
C                                                                       LIVO1870
      LEVEL = 1                                                         LIVO1880
      CALL UERSET(LEVEL,LEVOLD)                                         LIVO1890
      CALL LSVDB(S(1),WK(1,1),NS,A,IA,NN,B,IB,NB,IER)                   LIVO1900
C                                                                       LIVO1910
      IF (IER.GT.128) GO TO 9005                                        LIVO1920
      CALL UERSET(LEVOLD,LEVOLD)                                        LIVO1930
      IF (IER.NE.33) GO TO 135                                          LIVO1940
      FF = 0.0                                                          LIVO1950
      NM = MIN0(N,M)                                                    LIVO1960
      IF (S(1).NE.ZERO) FF = S(NM)/S(1)                                 LIVO1970
      F = 100.0+FF                                                      LIVO1980
      IF (F.EQ.100.0) GO TO 135                                         LIVO1990
      IER = 0                                                           LIVO2000
  135 CONTINUE                                                          LIVO2010
      IF (NS.GE.NN) GO TO 145                                           LIVO2020
      NSP1 = NS+1                                                       LIVO2030
      DO 140 J=NSP1,NN                                                  LIVO2040
  140 S(J) = ZERO                                                       LIVO2050
  145 CONTINUE                                                          LIVO2060
      IF (NN.EQ.N) GO TO 9005                                           LIVO2070
      NNP1 = NN+1                                                       LIVO2080
C                                  MOVE RECORD OF PERMUTATIONS AND      LIVO2090
C                                    STORE ZEROS                        LIVO2100
      DO 155 J=NNP1,N                                                   LIVO2110
         S(J) = A(1,J)                                                  LIVO2120
         IF (NN.LT.1) GO TO 155                                         LIVO2130
         DO 150 I=1,NN                                                  LIVO2140
  150    A(I,J) = ZERO                                                  LIVO2150
  155 CONTINUE                                                          LIVO2160
C                                  PERMUTE ROWS AND SET ZERO SINGULAR   LIVO2170
C                                    VALUES                             LIVO2180
      DO 165 K=NNP1,N                                                   LIVO2190
         I = S(K)                                                       LIVO2200
         S(K) = ZERO                                                    LIVO2210
         DO 160 J=1,N                                                   LIVO2220
            A(K,J) = A(I,J)                                             LIVO2230
            A(I,J) = ZERO                                               LIVO2240
  160    CONTINUE                                                       LIVO2250
         A(I,K) = ONE                                                   LIVO2260
  165 CONTINUE                                                          LIVO2270
C                                  END SPECIAL FOR ZERO ROWS AND        LIVO2280
C                                    COLUMNS                            LIVO2290
 9000 CONTINUE                                                          LIVO2300
 9005 RETURN                                                            LIVO2310
      END
C   IMSL ROUTINE NAME   - LSVDB                                         LSVB0010
C                                                                       LSVB0020
C-----------------------------------------------------------------------LSVB0030
C                                                                       LSVB0040
C   COMPUTER            - VAX/SINGLE                                    LSVB0050
C                                                                       LSVB0060
C   LATEST REVISION     - JANUARY 1, 1978                               LSVB0070
C                                                                       LSVB0080
C   PURPOSE             - SINGULAR VALUE DECOMPOSITION OF A BIDIAGONAL  LSVB0090
C                           MATRIX.                                     LSVB0100
C                                                                       LSVB0110
C   USAGE               - CALL LSVDB (D,E,N,V,IV,NRV,C,IC,NCC,IER)      LSVB0120
C                                                                       LSVB0130
C   ARGUMENTS    D      - VECTOR OF LENGTH N. (INPUT/OUTPUT)            LSVB0140
C                         ON INPUT, D CONTAINS THE DIAGONAL ELEMENTS    LSVB0150
C                           OF THE BIDIAGONAL MATRIX B. D(I)=B(I,I),    LSVB0160
C                           I=1,...,N.                                  LSVB0170
C                         ON OUTPUT, D CONTAINS THE N (NONNEGATIVE)     LSVB0180
C                           SINGULAR VALUES OF B IN NONINCREASING       LSVB0190
C                           ORDER.                                      LSVB0200
C                E      - VECTOR OF LENGTH N. (INPUT/OUTPUT)            LSVB0210
C                         ON INPUT, E CONTAINS THE SUPERDIAGONAL        LSVB0220
C                           ELEMENTS OF B. E(1) IS ARBITRARY,           LSVB0230
C                           E(I)=B(I-1,I), I=2,...,N.                   LSVB0240
C                         ON OUTPUT, THE CONTENTS OF E ARE MODIFIED     LSVB0250
C                           BY THE SUBROUTINE.                          LSVB0260
C                N      - ORDER OF THE MATRIX B. (INPUT)                LSVB0270
C                V      - NRV BY N MATRIX. (INPUT/OUTPUT)               LSVB0280
C                           IF NRV.LE.0, V IS NOT USED. OTHERWISE,      LSVB0290
C                           V IS REPLACED BY THE NRV BY N PRODUCT       LSVB0300
C                           MATRIX V*VB. SEE REMARKS.                   LSVB0310
C                IV     - ROW DIMENSION OF MATRIX V EXACTLY AS          LSVB0320
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LSVB0330
C                           CALLING PROGRAM. (INPUT)                    LSVB0340
C                NRV    - NUMBER OF ROWS OF V. (INPUT)                  LSVB0350
C                C      - N BY NCC MATRIX. (INPUT/OUTPUT)               LSVB0360
C                           IF NCC.LE.0 C IS NOT USED. OTHERWISE, C     LSVB0370
C                           IS REPLACED BY THE N BY NCC PRODUCT         LSVB0380
C                           MATRIX UB**(T) * C. SEE REMARKS.            LSVB0390
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          LSVB0400
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LSVB0410
C                           CALLING PROGRAM. (INPUT)                    LSVB0420
C                NCC    - NUMBER OF COLUMNS IN C. (INPUT)               LSVB0430
C                IER    - ERROR PARAMETER. (INPUT)                      LSVB0440
C                         WARNING ERROR                                 LSVB0450
C                           IER=33 INDICATES THAT MATRIX B IS NOT FULL  LSVB0460
C                             RANK OR VERY ILL-CONDITIONED. SMALL       LSVB0470
C                             SINGULAR VALUES MAY NOT BE VERY ACCURATE. LSVB0480
C                         TERMINAL ERROR                                LSVB0490
C                           IER=129 INDICATES THAT CONVERGENCE WAS      LSVB0500
C                             NOT ATTAINED AFTER 10*N QR SWEEPS.        LSVB0510
C                             (CONVERGENCE USUALLY OCCURS IN ABOUT      LSVB0520
C                             2*N SWEEPS).                              LSVB0530
C                                                                       LSVB0540
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LSVB0550
C                       - SINGLE/H36,H48,H60                            LSVB0560
C                                                                       LSVB0570
C   REQD. IMSL ROUTINES - LSVG1,LSVG2,VHS12,UERTST,UGETIO               LSVB0580
C                                                                       LSVB0590
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LSVB0600
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LSVB0610
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LSVB0620
C                                                                       LSVB0630
C   REMARKS      LSVDB COMPUTES THE SINGULAR VALUE DECOMPOSITION OF     LSVB0640
C                AN N BY N BIDIAGONAL MATRIX                            LSVB0650
C                     B = UB * S * VB**(T)    WHERE                     LSVB0660
C                UB AND VB ARE N BY N ORTHOGONAL MATRICES AND           LSVB0670
C                S IS DIAGONAL.                                         LSVB0680
C                IF ARGUMENTS V AND C ARE N BY N IDENTITY MATRICES,     LSVB0690
C                ON EXIT THEY ARE REPLACED BY VB AND UB**T,             LSVB0700
C                RESPECTIVELY.                                          LSVB0710
C                                                                       LSVB0720
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LSVB0730
C                                                                       LSVB0740
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LSVB0750
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LSVB0760
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LSVB0770
C                                                                       LSVB0780
C-----------------------------------------------------------------------LSVB0790
C                                                                       LSVB0800
      SUBROUTINE LSVDB (D,E,N,V,IV,NRV,C,IC,NCC,IER)                    LSVB0810
C                                                                       LSVB0820
C                                  SPECIFICATIONS FOR ARGUMENTS         LSVB0830
      INTEGER            N,IV,NRV,IC,NCC,IER                            LSVB0840
      REAL               D(N),E(N),V(IV,1),C(IC,1)                      LSVB0850
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LSVB0860
      INTEGER            I,II,J,K,KK,L,LL,LP1,NQRS,N10                  LSVB0870
      LOGICAL            WNTV,HAVERS,FAIL                               LSVB0880
      REAL               DNORM,ZERO,ONE,TWO,CS,F,G,H,SN,T,X,Y,Z         LSVB0890
      DATA               ZERO/0.0/,ONE/1.0/,TWO/2.0/                    LSVB0900
C                                  FIRST EXECUTABLE STATEMENT           LSVB0910
      IER = 0                                                           LSVB0920
      IF (N.LE.0) GO TO 9005                                            LSVB0930
      N10 = 10*N                                                        LSVB0940
      WNTV = NRV.GT.0                                                   LSVB0950
      HAVERS = NCC.GT.0                                                 LSVB0960
      FAIL = .FALSE.                                                    LSVB0970
      NQRS = 0                                                          LSVB0980
      E(1) = ZERO                                                       LSVB0990
      DNORM = ZERO                                                      LSVB1000
      DO 5 J=1,N                                                        LSVB1010
    5 DNORM = AMAX1(ABS(D(J))+ABS(E(J)),DNORM)                          LSVB1020
      DO 100 KK=1,N                                                     LSVB1030
         K = N+1-KK                                                     LSVB1040
C                                  TEST FOR SPLITTING OR RANK           LSVB1050
C                                    DEFICIENCIES FIRST MAKE TEST FOR   LSVB1060
C                                    LAST DIAGONAL TERM, D(K), BEING    LSVB1070
C                                    SMALL.                             LSVB1080
   10    IF (K.EQ.1) GO TO 25                                           LSVB1090
         T = DNORM+D(K)                                                 LSVB1100
         IF (T.NE.DNORM) GO TO 25                                       LSVB1110
C                                                                       LSVB1120
C                                  SINCE D(K) IS SMALL WE WILL MAKE A   LSVB1130
C                                    SPECIAL PASS TO TRANSFORM E(K) TO  LSVB1140
C                                    ZERO.                              LSVB1150
         CS = ZERO                                                      LSVB1160
         SN = -ONE                                                      LSVB1170
         DO 20 II=2,K                                                   LSVB1180
            I = K+1-II                                                  LSVB1190
            F = -SN*E(I+1)                                              LSVB1200
            E(I+1) = CS*E(I+1)                                          LSVB1210
            T = D(I)                                                    LSVB1220
            CALL LSVG1 (T,F,CS,SN,D(I))                                 LSVB1230
C                                  TRANSFORMATION CONSTRUCTED TO ZERO   LSVB1240
C                                    POSITION (I,K).                    LSVB1250
            IF (.NOT.WNTV) GO TO 20                                     LSVB1260
            DO 15 J=1,NRV                                               LSVB1270
   15       CALL LSVG2 (CS,SN,V(J,I),V(J,K))                            LSVB1280
C                                                                       LSVB1290
C                                  ACCUMULATE RT. TRANSFORMATIONS IN V. LSVB1300
   20    CONTINUE                                                       LSVB1310
C                                  THE MATRIX IS NOW BIDIAGONAL, AND OF LSVB1320
C                                    LOWER ORDER SINCE E(K) .EQ. ZERO   LSVB1330
   25    DO 30 LL=1,K                                                   LSVB1340
            L = K+1-LL                                                  LSVB1350
            T = DNORM+E(L)                                              LSVB1360
            IF (T.EQ.DNORM) GO TO 50                                    LSVB1370
            T = DNORM+D(L-1)                                            LSVB1380
            IF (T.EQ.DNORM) GO TO 35                                    LSVB1390
   30    CONTINUE                                                       LSVB1400
C                                  THIS LOOP CANT COMPLETE SINCE E(1) = LSVB1410
C                                    ZERO.                              LSVB1420
         GO TO 50                                                       LSVB1430
C                                  CANCELLATION OF E(L), L.GT.1.        LSVB1440
   35    CS = ZERO                                                      LSVB1450
         SN = -ONE                                                      LSVB1460
         DO 45 I=L,K                                                    LSVB1470
            F = -SN*E(I)                                                LSVB1480
            E(I) = CS*E(I)                                              LSVB1490
            T = DNORM+F                                                 LSVB1500
            IF (T.EQ.DNORM) GO TO 50                                    LSVB1510
            T = D(I)                                                    LSVB1520
            CALL LSVG1 (T,F,CS,SN,D(I))                                 LSVB1530
            IF (.NOT.HAVERS) GO TO 45                                   LSVB1540
            DO 40 J=1,NCC                                               LSVB1550
   40       CALL LSVG2 (CS,SN,C(I,J),C(L-1,J))                          LSVB1560
   45    CONTINUE                                                       LSVB1570
C                                  TEST FOR CONVERGENCE                 LSVB1580
   50    Z = D(K)                                                       LSVB1590
         IF (L.EQ.K) GO TO 85                                           LSVB1600
C                                  SHIFT FROM BOTTOM 2 BY 2 MINOR OF    LSVB1610
C                                    B**(T)*B.                          LSVB1620
         X = D(L)                                                       LSVB1630
         Y = D(K-1)                                                     LSVB1640
         G = E(K-1)                                                     LSVB1650
         H = E(K)                                                       LSVB1660
         F = ((Y-Z)*(Y+Z)+(G-H)*(G+H))/(TWO*H*Y)                        LSVB1670
         G = SQRT(ONE+F**2)                                             LSVB1680
         IF (F.LT.ZERO) GO TO 55                                        LSVB1690
         T = F+G                                                        LSVB1700
         GO TO 60                                                       LSVB1710
   55    T = F-G                                                        LSVB1720
   60    F = ((X-Z)*(X+Z)+H*(Y/T-H))/X                                  LSVB1730
C                                  NEXT QR SWEEP                        LSVB1740
         CS = ONE                                                       LSVB1750
         SN = ONE                                                       LSVB1760
         LP1 = L+1                                                      LSVB1770
         DO 80 I=LP1,K                                                  LSVB1780
            G = E(I)                                                    LSVB1790
            Y = D(I)                                                    LSVB1800
            H = SN*G                                                    LSVB1810
            G = CS*G                                                    LSVB1820
            CALL LSVG1 (F,H,CS,SN,E(I-1))                               LSVB1830
            F = X*CS+G*SN                                               LSVB1840
            G = -X*SN+G*CS                                              LSVB1850
            H = Y*SN                                                    LSVB1860
            Y = Y*CS                                                    LSVB1870
            IF (.NOT.WNTV) GO TO 70                                     LSVB1880
C                                  ACCUMULATE ROTATIONS (FROM THE       LSVB1890
C                                    RIGHT) IN V                        LSVB1900
            DO 65 J=1,NRV                                               LSVB1910
   65       CALL LSVG2 (CS,SN,V(J,I-1),V(J,I))                          LSVB1920
   70       CALL LSVG1 (F,H,CS,SN,D(I-1))                               LSVB1930
            F = CS*G+SN*Y                                               LSVB1940
            X = -SN*G+CS*Y                                              LSVB1950
            IF (.NOT.HAVERS) GO TO 80                                   LSVB1960
            DO 75 J=1,NCC                                               LSVB1970
   75       CALL LSVG2 (CS,SN,C(I-1,J),C(I,J))                          LSVB1980
C                                                                       LSVB1990
C                                  APPLY ROTATIONS FROM THE LEFT TO     LSVB2000
C                                    RIGHT HAND SIDES IN C              LSVB2010
   80    CONTINUE                                                       LSVB2020
         E(L) = ZERO                                                    LSVB2030
         E(K) = F                                                       LSVB2040
         D(K) = X                                                       LSVB2050
         NQRS = NQRS+1                                                  LSVB2060
         IF (NQRS.LE.N10) GO TO 10                                      LSVB2070
C                                  RETURN TO TEST FOR SPLITTING.        LSVB2080
         FAIL = .TRUE.                                                  LSVB2090
C                                  CUTOFF FOR CONVERGENCE FAILURE. NQRS LSVB2100
C                                    WILL BE 2*N USUALLY.               LSVB2110
   85    IF (Z.GE.ZERO) GO TO 95                                        LSVB2120
         D(K) = -Z                                                      LSVB2130
         IF (.NOT.WNTV) GO TO 95                                        LSVB2140
         DO 90 J=1,NRV                                                  LSVB2150
   90    V(J,K) = -V(J,K)                                               LSVB2160
   95    CONTINUE                                                       LSVB2170
C                                  CONVERGENCE. D(K) IS MADE            LSVB2180
C                                    NONNEGATIVE                        LSVB2190
  100 CONTINUE                                                          LSVB2200
      IF (N.EQ.1) GO TO 140                                             LSVB2210
      DO 105 I=2,N                                                      LSVB2220
         IF (D(I).GT.D(I-1)) GO TO 110                                  LSVB2230
  105 CONTINUE                                                          LSVB2240
      GO TO 140                                                         LSVB2250
C                                  EVERY SINGULAR VALUE IS IN ORDER     LSVB2260
  110 DO 135 I=2,N                                                      LSVB2270
         T = D(I-1)                                                     LSVB2280
         K = I-1                                                        LSVB2290
         DO 115 J=I,N                                                   LSVB2300
            IF (T.GE.D(J)) GO TO 115                                    LSVB2310
            T = D(J)                                                    LSVB2320
            K = J                                                       LSVB2330
  115    CONTINUE                                                       LSVB2340
         IF (K.EQ.I-1) GO TO 135                                        LSVB2350
         D(K) = D(I-1)                                                  LSVB2360
         D(I-1) = T                                                     LSVB2370
         IF (.NOT.HAVERS) GO TO 125                                     LSVB2380
         DO 120 J=1,NCC                                                 LSVB2390
            T = C(I-1,J)                                                LSVB2400
            C(I-1,J) = C(K,J)                                           LSVB2410
  120    C(K,J) = T                                                     LSVB2420
  125    IF (.NOT.WNTV) GO TO 135                                       LSVB2430
         DO 130 J=1,NRV                                                 LSVB2440
            T = V(J,I-1)                                                LSVB2450
            V(J,I-1) = V(J,K)                                           LSVB2460
  130    V(J,K) = T                                                     LSVB2470
  135 CONTINUE                                                          LSVB2480
C                                  END OF ORDERING ALGORITHM.           LSVB2490
  140 IER = 129                                                         LSVB2500
      IF (FAIL) GO TO 9000                                              LSVB2510
C                                  CHECK FOR POSSIBLE RANK DEFICIENCY   LSVB2520
      IER = 33                                                          LSVB2530
      T = 0.0                                                           LSVB2540
      IF (D(1).NE.ZERO) T=D(N)/D(1)                                     LSVB2550
      F=100.0+T                                                         LSVB2560
      IF (F.EQ.100.0) GO TO 9000                                        LSVB2570
      IER = 0                                                           LSVB2580
      GO TO 9005                                                        LSVB2590
 9000 CONTINUE                                                          LSVB2600
      CALL UERTST (IER,6HLSVDB )                                        LSVB2610
 9005 RETURN                                                            LSVB2620
      END                                                               LSVB2630
C   IMSL ROUTINE NAME   - LSVDF                                         LSVF0010
C                                                                       LSVF0020
C-----------------------------------------------------------------------LSVF0030
C                                                                       LSVF0040
C   COMPUTER            - VAX/SINGLE                                    LSVF0050
C                                                                       LSVF0060
C   LATEST REVISION     - JUNE 1, 1980                                  LSVF0070
C                                                                       LSVF0080
C   PURPOSE             - SINGULAR VALUE DECOMPOSITION OF A REAL        LSVF0090
C                           MATRIX                                      LSVF0100
C                                                                       LSVF0110
C   USAGE               - CALL LSVDF (A,IA,M,N,B,IB,NB,S,WK,IER)        LSVF0120
C                                                                       LSVF0130
C   ARGUMENTS    A      - REAL M BY N MATRIX. (INPUT/OUTPUT)            LSVF0140
C                         ON INPUT, A CONTAINS THE MATRIX TO BE         LSVF0150
C                           DECOMPOSED.                                 LSVF0160
C                         ON OUTPUT, A CONTAINS THE N BY N MATRIX V     LSVF0170
C                           IN ITS FIRST N ROWS. SEE REMARKS. EITHER    LSVF0180
C                           M.GE.N OR M.LT.N IS PERMITTED.              LSVF0190
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          LSVF0200
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LSVF0210
C                           CALLING PROGRAM. (INPUT)                    LSVF0220
C                           A IS USED BY LSVDF AS WORK STORAGE FOR AN   LSVF0230
C                           N BY N MATRIX. THEREFORE, IA MUST BE        LSVF0240
C                           GREATER THAN OR EQUAL TO MAX(M,N).          LSVF0250
C                M      - NUMBER OF ROWS IN A. (INPUT)                  LSVF0260
C                N      - NUMBER OF COLUMNS IN A. (INPUT)               LSVF0270
C                B      - M BY NB MATRIX. (INPUT/OUTPUT)                LSVF0280
C                           B IS NOT USED IF NB.LE.0. OTHERWISE, B IS   LSVF0290
C                           REPLACED BY THE MATRIX PRODUCT U**(T) * B.  LSVF0300
C                           SEE REMARKS.                                LSVF0310
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          LSVF0320
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE LSVF0330
C                           CALLING PROGRAM. (INPUT)                    LSVF0340
C                NB     - NUMBER OF COLUMNS IN B. (INPUT)               LSVF0350
C                           IF NB.LE.0, B IS NOT USED.                  LSVF0360
C                S      - VECTOR OF LENGTH N. (OUTPUT)                  LSVF0370
C                         ON OUTPUT, S CONTAINS THE ORDERED SINGULAR    LSVF0380
C                           VALUES OF A.  S(1) .GE. S(2),...,           LSVF0390
C                           .GE. S(N) .GE. 0.                           LSVF0400
C                WK     - WORK VECTOR OF LENGTH 2N.                     LSVF0410
C                IER    - ERROR PARAMETER. (OUTPUT)                     LSVF0420
C                         WARNING ERROR                                 LSVF0430
C                           IER=33 INDICATES THAT MATRIX A IS NOT       LSVF0440
C                             FULL RANK OR VERY ILL-CONDITIONED. SMALL  LSVF0450
C                             SINGULAR VALUES MAY NOT BE VERY ACCURATE. LSVF0460
C                           IER=34 INDICATES THAT EITHER N.LE.0 OR      LSVF0470
C                             M.LE.0.                                   LSVF0480
C                         TERMINAL ERROR                                LSVF0490
C                           IER=129 INDICATES THAT CONVERGENCE WAS      LSVF0500
C                             NOT OBTAINED BY LSVDB AND COMPUTATION     LSVF0510
C                             WAS DISCONTINUED.                         LSVF0520
C                                                                       LSVF0530
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LSVF0540
C                       - SINGLE/H36,H48,H60                            LSVF0550
C                                                                       LSVF0560
C   REQD. IMSL ROUTINES - LSVDB,LSVG1,LSVG2,VHS12,UERSET,UERTST,UGETIO  LSVF0570
C                                                                       LSVF0580
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LSVF0590
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LSVF0600
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LSVF0610
C                                                                       LSVF0620
C   REMARKS  1.  LSVDF COMPUTES THE SINGULAR VALUE DECOMPOSITION OF     LSVF0630
C                A REAL M BY N MATRIX                                   LSVF0640
C                     A = U * Q * V**(T)  WHERE                         LSVF0650
C                U IS AN M BY M ORTHOGONAL MATRIX,                      LSVF0660
C                V IS AN N BY N ORTHOGONAL MATRIX, AND                  LSVF0670
C                Q IS AN M BY N MATRIX WITH ALL ELEMENTS ZERO EXCEPT    LSVF0680
C                     Q(I,I) = S(I) I=1,...,MIN(M,N).                   LSVF0690
C                V IS RETURNED IN THE FIRST N ROWS OF A.                LSVF0700
C                U IS OBTAINED BY SETTING B TO THE M BY M IDENTITY      LSVF0710
C                MATRIX, ON INPUT, AND SETTING NB=M. ON OUTPUT, B IS    LSVF0720
C                REPLACED BY U**(T).                                    LSVF0730
C            2.  THE NOTATION U**(T) AND V**(T) REPRESENTS U            LSVF0740
C                TRANSPOSE AND V TRANSPOSE, RESPECTIVELY. Q**(+)        LSVF0750
C                DENOTES THE GENERALIZED INVERSE OF Q.                  LSVF0760
C            3.  LSVDF IS USEFUL IN ANALYZING AND SOLVING THE LEAST     LSVF0770
C                SQUARES PROBLEM A*X.APPR.B (WHERE .APPR. MEANS         LSVF0780
C                APPROXIMATELY EQUALS). IN THIS CASE B IS A VECTOR OF   LSVF0790
C                LENGTH M AND LSVDF IS CALLED WITH IB=M, NB=1. THE      LSVF0800
C                SOLUTION IS X=V*Q**(+)*U**(T)*B. U**(T)*B REPLACES     LSVF0810
C                B ON OUTPUT. THE SOLUTION X IS OBTAINED AS FOLLOWS...  LSVF0820
C                (THE USER MAY WISH TO SET SMALL SINGULAR VALUES SUCH ASLSVF0830
C                S(I) TO ZERO JUST PRIOR TO COMPUTING X. SEE REFERENCE  LSVF0840
C                FOR DETAILS.)                                          LSVF0850
C                                                                       LSVF0860
C                C                  COMPUTE Q**(+) * U**(T) * B         LSVF0870
C                      L=MIN0(M,N)                                      LSVF0880
C                      DO 10 I=1,L                                      LSVF0890
C                         T=0.0                                         LSVF0900
C                         IF (S(I).NE.0.0) T=B(I)/S(I)                  LSVF0910
C                         B(I)=T                                        LSVF0920
C                   10 CONTINUE                                         LSVF0930
C                C                  COMPUTE V * Q**(+) * U**(T) * B     LSVF0940
C                   15 DO 25 I=1,N                                      LSVF0950
C                         X(I)=0.0                                      LSVF0960
C                         DO 20 J=1,L                                   LSVF0970
C                   20    X(I)=X(I)+A(I,J)*B(J)                         LSVF0980
C                   25 CONTINUE                                         LSVF0990
C                IF B IS SET TO THE J-TH COLUMN OF THE M BY M IDENTITY  LSVF1000
C                MATRIX ON INPUT, X IS THE J=TH COLUMN OF A**(+) (THE   LSVF1010
C                GENERALIZED INVERSE OF A).                             LSVF1020
C            4.  THE USER SHOULD BE AWARE OF SEVERAL PRACTICAL ASPECTS  LSVF1030
C                OF THE SINGULAR VALUE ANALYSIS. ONE OF THESE IS THE    LSVF1040
C                EFFECT OF THE UNCERTAINTY OF THE DATA AND THE NEED FOR LSVF1050
C                SCALING. SEE THE LAWSON-HANSON REFERENCE PAGES 180-198 LSVF1060
C                FOR DETAILS.                                           LSVF1070
C                                                                       LSVF1080
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LSVF1090
C                                                                       LSVF1100
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LSVF1110
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LSVF1120
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LSVF1130
C                                                                       LSVF1140
C-----------------------------------------------------------------------LSVF1150
C                                                                       LSVF1160
      SUBROUTINE LSVDF  (A,IA,M,N,B,IB,NB,S,WK,IER)                     LSVF1170
C                                                                       LSVF1180
C                                  SPECIFICATIONS FOR ARGUMENTS         LSVF1190
      INTEGER            IA,M,N,IB,NB,IER                               LSVF1200
      REAL               A(IA,N),B(IB,1),S(N),WK(N,2)                   LSVF1210
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LSVF1220
      INTEGER            I,J,JP1,K,L,MM,NN,NNP1,NS,NSP1                 LSVF1230
      REAL               ZERO,ONE,T                                     LSVF1240
      DATA               ZERO/0.0/,ONE/1.0/                             LSVF1250
C                                  FIRST EXECUTABLE STATEMENT           LSVF1260
      IER = 0                                                           LSVF1270
C                                  BEGIN SPECIAL FOR ZERO ROWS AND      LSVF1280
C                                    COLS. PACK THE NONZERO COLS TO THE LSVF1290
C                                    LEFT                               LSVF1300
      NN = N                                                            LSVF1310
      IER = 34                                                          LSVF1320
      IF (NN.LE.0.OR.M.LE.0) GO TO 9000                                 LSVF1330
      IER = 0                                                           LSVF1340
      J = NN                                                            LSVF1350
    5 CONTINUE                                                          LSVF1360
      DO 10 I=1,M                                                       LSVF1370
         IF (A(I,J).NE.ZERO) GO TO 25                                   LSVF1380
   10 CONTINUE                                                          LSVF1390
C                                  COL J IS ZERO. EXCHANGE IT WITH COL  LSVF1400
C                                    N                                  LSVF1410
      IF (J.EQ.NN) GO TO 20                                             LSVF1420
      DO 15 I=1,M                                                       LSVF1430
   15 A(I,J) = A(I,NN)                                                  LSVF1440
   20 CONTINUE                                                          LSVF1450
      A(1,NN) = J                                                       LSVF1460
      NN = NN-1                                                         LSVF1470
   25 CONTINUE                                                          LSVF1480
      J = J-1                                                           LSVF1490
      IF (J.GE.1) GO TO 5                                               LSVF1500
C                                  IF N=0 THEN A IS ENTIRELY ZERO AND   LSVF1510
C                                    SVD COMPUTATION CAN BE SKIPPED     LSVF1520
      NS = 0                                                            LSVF1530
      IF (NN.EQ.0) GO TO 120                                            LSVF1540
C                                  PACK NONZERO ROWS TO THE TOP QUIT    LSVF1550
C                                    PACKING IF FIND N NONZERO ROWS     LSVF1560
      I = 1                                                             LSVF1570
      MM = M                                                            LSVF1580
   30 IF (I.GT.N.OR.I.GE.MM) GO TO 75                                   LSVF1590
      IF (A(I,I).NE.ZERO) GO TO 40                                      LSVF1600
      DO 35 J=1,NN                                                      LSVF1610
         IF (A(I,J).NE.ZERO) GO TO 40                                   LSVF1620
   35 CONTINUE                                                          LSVF1630
      GO TO 45                                                          LSVF1640
   40 I = I+1                                                           LSVF1650
      GO TO 30                                                          LSVF1660
C                                  ROW I IS ZERO EXCHANGE ROWS I AND M  LSVF1670
   45 IF (NB.LE.0) GO TO 55                                             LSVF1680
      DO 50 J=1,NB                                                      LSVF1690
         T = B(I,J)                                                     LSVF1700
         B(I,J) = B(MM,J)                                               LSVF1710
         B(MM,J) = T                                                    LSVF1720
   50 CONTINUE                                                          LSVF1730
   55 DO 60 J=1,NN                                                      LSVF1740
   60 A(I,J) = A(MM,J)                                                  LSVF1750
      IF (MM.GT.NN) GO TO 70                                            LSVF1760
      DO 65 J=1,NN                                                      LSVF1770
   65 A(MM,J) = ZERO                                                    LSVF1780
   70 CONTINUE                                                          LSVF1790
C                                  EXCHANGE IS FINISHED                 LSVF1800
      MM = MM-1                                                         LSVF1810
      GO TO 30                                                          LSVF1820
C                                                                       LSVF1830
   75 CONTINUE                                                          LSVF1840
C                                  END SPECIAL FOR ZERO ROWS AND        LSVF1850
C                                    COLUMNS                            LSVF1860
C                                  BEGIN SVD ALGORITHM..                LSVF1870
C                                  (1) REDUCE THE MATRIX TO UPPER       LSVF1880
C                                    BIDIAGONAL FORM WITH HOUSEHOLDER   LSVF1890
C                                    TRANSFORMATIONS.                   LSVF1900
C                                    H(N)...H(1)AQ(1)...Q(N-2) =        LSVF1910
C                                    (D**T,0)**T WHERE D IS UPPER       LSVF1920
C                                    BIDIAGONAL.                        LSVF1930
C                                  (2) APPLY H(N)...H(1) TO B. HERE     LSVF1940
C                                    H(N)...H(1)*B REPLACES B IN        LSVF1950
C                                    STORAGE.                           LSVF1960
C                                  (3) THE MATRIX PRODUCT W=            LSVF1970
C                                    Q(1)...Q(N-2) OVERWRITES THE FIRST LSVF1980
C                                    N ROWS OF A IN STORAGE.            LSVF1990
C                                  (4) AN SVD FOR D IS COMPUTED. HERE K LSVF2000
C                                    ROTATIONS RI AND PI ARE COMPUTED   LSVF2010
C                                    SO THAT RK...R1*D*P1**(T)...PK**(T)LSVF2020
C                                    = DIAG(S1,...,SM) TO WORKING       LSVF2030
C                                    ACCURACY. THE SI ARE NONNEGATIVE   LSVF2040
C                                    AND NONINCREASING. HERE RK...R1*B  LSVF2050
C                                    OVERWRITES B IN STORAGE WHILE      LSVF2060
C                                    A*P1**(T)...PK**(T) OVERWRITES A   LSVF2070
C                                    IN STORAGE.                        LSVF2080
C                                  (5) IT FOLLOWS THAT,WITH THE PROPER  LSVF2090
C                                    DEFINITIONS, U**(T)*B OVERWRITES   LSVF2100
C                                    B, WHILE V OVERWRITES THE FIRST N  LSVF2110
C                                    ROW AND COLUMNS OF A.              LSVF2120
      L = MIN0(MM,NN)                                                   LSVF2130
C                                  THE FOLLOWING LOOP REDUCES A TO      LSVF2140
C                                    UPPER BIDIAGONAL AND ALSO APPLIES  LSVF2150
C                                    THE PREMULTIPLYING TRANSFORMATIONS LSVF2160
C                                    TO B.                              LSVF2170
      DO 85 J=1,L                                                       LSVF2180
         IF (J.GE.MM) GO TO 80                                          LSVF2190
         JP1 = MIN0(J+1,NN)                                             LSVF2200
         CALL VHS12 (1,J,J+1,MM,A(1,J),1,T,A(1,JP1),1,IA,NN-J)          LSVF2210
         CALL VHS12 (2,J,J+1,MM,A(1,J),1,T,B,1,IB,NB)                   LSVF2220
   80    IF (J.GE.NN-1) GO TO 85                                        LSVF2230
         CALL VHS12 (1,J+1,J+2,NN,A(J,1),IA,WK(J,2),A(J+1,1),IA,1,MM-J) LSVF2240
   85 CONTINUE                                                          LSVF2250
C                                  COPY THE BIDIAGONAL MATRIX INTO THE  LSVF2260
C                                    ARRAY S FOR LSVDB                  LSVF2270
      IF (L.EQ.1) GO TO 95                                              LSVF2280
      DO 90 J=2,L                                                       LSVF2290
         S(J) = A(J,J)                                                  LSVF2300
         WK(J,1) = A(J-1,J)                                             LSVF2310
   90 CONTINUE                                                          LSVF2320
   95 S(1) = A(1,1)                                                     LSVF2330
C                                                                       LSVF2340
      NS = NN                                                           LSVF2350
      IF (MM.GE.NN) GO TO 100                                           LSVF2360
      NS = MM+1                                                         LSVF2370
      S(NS) = ZERO                                                      LSVF2380
      WK(NS,1) = A(MM,MM+1)                                             LSVF2390
  100 CONTINUE                                                          LSVF2400
C                                  CONSTRUCT THE EXPLICIT N BY N        LSVF2410
C                                    PRODUCT MATRIX, W=Q1*Q2*...*QL*I   LSVF2420
C                                    IN THE ARRAY A                     LSVF2430
      DO 115 K=1,NN                                                     LSVF2440
         I = NN+1-K                                                     LSVF2450
         IF (I.GT.MIN0(MM,NN-2)) GO TO 105                              LSVF2460
         CALL VHS12 (2,I+1,I+2,NN,A(I,1),IA,WK(I,2),A(1,I+1),1,IA,NN-I) LSVF2470
  105    DO 110 J=1,NN                                                  LSVF2480
  110    A(I,J) = ZERO                                                  LSVF2490
         A(I,I) = ONE                                                   LSVF2500
  115 CONTINUE                                                          LSVF2510
C                                  COMPUTE THE SVD OF THE BIDIAGONAL    LSVF2520
C                                    MATRIX                             LSVF2530
C                                                                       LSVF2540
      LEVEL=1                                                           LSVF2550
      CALL UERSET(LEVEL,LEVOLD)                                         LSVF2560
      CALL LSVDB (S(1),WK(1,1),NS,A,IA,NN,B,IB,NB,IER)                  LSVF2570
C                                  TEST FOR IER=33                      LSVF2580
C                                                                       LSVF2590
      IF (IER.GT.128) GO TO 9000                                        LSVF2600
      CALL UERSET(LEVOLD,LEVOLD)                                        LSVF2610
      IF (IER.NE.33) GO TO 120                                          LSVF2620
      T=0.0                                                             LSVF2630
      NM=MIN0(M,N)                                                      LSVF2640
      IF (S(1).NE.ZERO) T=S(NM)/S(1)                                    LSVF2650
      F=100.0+T                                                         LSVF2660
      IF (F.EQ.100.0) GO TO 120                                         LSVF2670
      IER=0                                                             LSVF2680
  120 CONTINUE                                                          LSVF2690
      IF (NS.GE.NN) GO TO 130                                           LSVF2700
      NSP1 = NS+1                                                       LSVF2710
      DO 125 J=NSP1,NN                                                  LSVF2720
  125 S(J) = ZERO                                                       LSVF2730
  130 CONTINUE                                                          LSVF2740
      IF (NN.EQ.N) GO TO 155                                            LSVF2750
      NNP1 = NN+1                                                       LSVF2760
C                                  MOVE RECORD OF PERMUTATIONS AND      LSVF2770
C                                    STORE ZEROS                        LSVF2780
      DO 140 J=NNP1,N                                                   LSVF2790
         S(J) = A(1,J)                                                  LSVF2800
         IF (NN.LT.1) GO TO 140                                         LSVF2810
         DO 135 I=1,NN                                                  LSVF2820
  135    A(I,J) = ZERO                                                  LSVF2830
  140 CONTINUE                                                          LSVF2840
C                                  PERMUTE ROWS AND SET ZERO SINGULAR   LSVF2850
C                                    VALUES                             LSVF2860
      DO 150 K=NNP1,N                                                   LSVF2870
         I = S(K)                                                       LSVF2880
         S(K) = ZERO                                                    LSVF2890
         DO 145 J=1,N                                                   LSVF2900
            A(K,J) = A(I,J)                                             LSVF2910
  145    A(I,J) = ZERO                                                  LSVF2920
         A(I,K) = ONE                                                   LSVF2930
  150 CONTINUE                                                          LSVF2940
C                                  END SPECIAL FOR ZERO ROWS AND        LSVF2950
C                                    COLUMNS                            LSVF2960
  155 IF (IER.EQ.0) GO TO 9005                                          LSVF2970
 9000 CONTINUE                                                          LSVF2980
      CALL UERTST (IER,6HLSVDF )                                        LSVF2990
 9005 RETURN                                                            LSVF3000
      END                                                               LSVF3010
C   IMSL ROUTINE NAME   - LSVG1                                         LSVG0010
C                                                                       LSVG0020
C-----------------------------------------------------------------------LSVG0030
C                                                                       LSVG0040
C   COMPUTER            - VAX/SINGLE                                    LSVG0050
C                                                                       LSVG0060
C   LATEST REVISION     - JANUARY 1, 1978                               LSVG0070
C                                                                       LSVG0080
C   PURPOSE             - NUCLEUS CALLED ONLY BY IMSL ROUTINE LSVDB     LSVG0090
C                                                                       LSVG0100
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LSVG0110
C                       - SINGLE/H36,H48,H60                            LSVG0120
C                                                                       LSVG0130
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 LSVG0140
C                                                                       LSVG0150
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LSVG0160
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LSVG0170
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LSVG0180
C                                                                       LSVG0190
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LSVG0200
C                                                                       LSVG0210
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LSVG0220
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LSVG0230
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LSVG0240
C                                                                       LSVG0250
C-----------------------------------------------------------------------LSVG0260
C                                                                       LSVG0270
      SUBROUTINE LSVG1  (A,B,COS,SIN,SIG)                               LSVG0280
C                                                                       LSVG0290
C                                  SPECIFICATIONS FOR ARGUMENTS         LSVG0300
      REAL               A,B,COS,SIN,SIG                                LSVG0310
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LSVG0320
      REAL               AA,BB                                          LSVG0330
C                                  FIRST EXECUTABLE STATEMENT           LSVG0340
      IF (ABS(A).LE.ABS(B)) GO TO 5                                     LSVG0350
      AA = ABS(A+A)                                                     LSVG0360
      SIG = AA*SQRT(0.25+(B/AA)**2)                                     LSVG0370
      COS = A/SIG                                                       LSVG0380
      SIN = B/SIG                                                       LSVG0390
      RETURN                                                            LSVG0400
    5 IF (B.EQ.0.0) GO TO 10                                            LSVG0410
      BB = ABS(B+B)                                                     LSVG0420
      SIG = BB*SQRT(0.25+(A/BB)**2)                                     LSVG0430
      COS = A/SIG                                                       LSVG0440
      SIN = B/SIG                                                       LSVG0450
      RETURN                                                            LSVG0460
   10 SIG = 0.0                                                         LSVG0470
      COS = 0.0                                                         LSVG0480
      SIN = 1.0                                                         LSVG0490
      RETURN                                                            LSVG0500
      END                                                               LSVG0510
C   IMSL ROUTINE NAME   - LSVG2                                         LSVH0010
C                                                                       LSVH0020
C-----------------------------------------------------------------------LSVH0030
C                                                                       LSVH0040
C   COMPUTER            - VAX/SINGLE                                    LSVH0050
C                                                                       LSVH0060
C   LATEST REVISION     - JANUARY 1, 1978                               LSVH0070
C                                                                       LSVH0080
C   PURPOSE             - NUCLEUS CALLED ONLY BY IMSL ROUTINE LSVDB     LSVH0090
C                                                                       LSVH0100
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         LSVH0110
C                       - SINGLE/H36,H48,H60                            LSVH0120
C                                                                       LSVH0130
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 LSVH0140
C                                                                       LSVH0150
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           LSVH0160
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      LSVH0170
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  LSVH0180
C                                                                       LSVH0190
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       LSVH0200
C                                                                       LSVH0210
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN LSVH0220
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    LSVH0230
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        LSVH0240
C                                                                       LSVH0250
C-----------------------------------------------------------------------LSVH0260
C                                                                       LSVH0270
      SUBROUTINE LSVG2  (COS,SIN,X,Y)                                   LSVH0280
C                                                                       LSVH0290
C                                  SPECIFICATIONS FOR ARGUMENTS         LSVH0300
      REAL               COS,SIN,X,Y                                    LSVH0310
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   LSVH0320
      REAL               XR                                             LSVH0330
C                                  FIRST EXECUTABLE STATEMENT           LSVH0340
      XR=COS*X+SIN*Y                                                    LSVH0350
      Y=-SIN*X+COS*Y                                                    LSVH0360
      X=XR                                                              LSVH0370
      RETURN                                                            LSVH0380
      END                                                               LSVH0390
C   IMSL ROUTINE NAME   - UERSET                                        UERS0010
C                                                                       UERS0020
C-----------------------------------------------------------------------UERS0030
C                                                                       UERS0040
C   COMPUTER            - VAX/SINGLE                                    UERS0050
C                                                                       UERS0060
C   LATEST REVISION     - JANUARY 1, 1978                               UERS0070
C                                                                       UERS0080
C   PURPOSE             - SET MESSAGE LEVEL FOR IMSL ROUTINE UERTST     UERS0090
C                                                                       UERS0100
C   USAGE               - CALL UERSET (LEVEL,LEVOLD)                    UERS0110
C                                                                       UERS0120
C   ARGUMENTS    LEVEL  - NEW VALUE FOR MESSAGE LEVEL. (INPUT)          UERS0130
C                           OUTPUT FROM IMSL ROUTINE UERTST IS          UERS0140
C                           CONTROLLED SELECTIVELY AS FOLLOWS,          UERS0150
C                             LEVEL = 4 CAUSES ALL MESSAGES TO BE       UERS0160
C                                       PRINTED,                        UERS0170
C                             LEVEL = 3 MESSAGES ARE PRINTED IF IER IS  UERS0180
C                                       GREATER THAN 32,                UERS0190
C                             LEVEL = 2 MESSAGES ARE PRINTED IF IER IS  UERS0200
C                                       GREATER THAN 64,                UERS0210
C                             LEVEL = 1 MESSAGES ARE PRINTED IF IER IS  UERS0220
C                                       GREATER THAN 128,               UERS0230
C                             LEVEL = 0 ALL MESSAGE PRINTING IS         UERS0240
C                                       SUPPRESSED.                     UERS0250
C                LEVOLD - PREVIOUS MESSAGE LEVEL. (OUTPUT)              UERS0260
C                                                                       UERS0270
C   PRECISION/HARDWARE  - SINGLE/ALL                                    UERS0280
C                                                                       UERS0290
C   REQD. IMSL ROUTINES - UERTST,UGETIO                                 UERS0300
C                                                                       UERS0310
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           UERS0320
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      UERS0330
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  UERS0340
C                                                                       UERS0350
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       UERS0360
C                                                                       UERS0370
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN UERS0380
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    UERS0390
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        UERS0400
C                                                                       UERS0410
C-----------------------------------------------------------------------UERS0420
C                                                                       UERS0430
      SUBROUTINE UERSET (LEVEL,LEVOLD)                                  UERS0440
C                                  SPECIFICATIONS FOR ARGUMENTS         UERS0450
      INTEGER            LEVEL,LEVOLD                                   UERS0460
C                                  FIRST EXECUTABLE STATEMENT           UERS0470
      LEVOLD = LEVEL                                                    UERS0480
      CALL UERTST (LEVOLD,6HUERSET)                                     UERS0490
      RETURN                                                            UERS0500
      END                                                               UERS0510
C   IMSL ROUTINE NAME   - UERTST                                        UERT0010
C                                                                       UERT0020
C-----------------------------------------------------------------------UERT0030
C                                                                       UERT0040
C   COMPUTER            - VAX/SINGLE                                    UERT0050
C                                                                       UERT0060
C   LATEST REVISION     - JUNE 1, 1982                                  UERT0070
C                                                                       UERT0080
C   PURPOSE             - PRINT A MESSAGE REFLECTING AN ERROR CONDITION UERT0090
C                                                                       UERT0100
C   USAGE               - CALL UERTST (IER,NAME)                        UERT0110
C                                                                       UERT0120
C   ARGUMENTS    IER    - ERROR PARAMETER. (INPUT)                      UERT0130
C                           IER = I+J WHERE                             UERT0140
C                             I = 128 IMPLIES TERMINAL ERROR MESSAGE,   UERT0150
C                             I =  64 IMPLIES WARNING WITH FIX MESSAGE, UERT0160
C                             I =  32 IMPLIES WARNING MESSAGE.          UERT0170
C                             J = ERROR CODE RELEVANT TO CALLING        UERT0180
C                                 ROUTINE.                              UERT0190
C                NAME   - A CHARACTER STRING OF LENGTH SIX PROVIDING    UERT0200
C                           THE NAME OF THE CALLING ROUTINE. (INPUT)    UERT0210
C                                                                       UERT0220
C   PRECISION/HARDWARE  - SINGLE/ALL                                    UERT0230
C                                                                       UERT0240
C   REQD. IMSL ROUTINES - UGETIO,USPKD                                  UERT0250
C                                                                       UERT0260
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           UERT0270
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      UERT0280
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  UERT0290
C                                                                       UERT0300
C   REMARKS      THE ERROR MESSAGE PRODUCED BY UERTST IS WRITTEN        UERT0310
C                TO THE STANDARD OUTPUT UNIT. THE OUTPUT UNIT           UERT0320
C                NUMBER CAN BE DETERMINED BY CALLING UGETIO AS          UERT0330
C                FOLLOWS..   CALL UGETIO(1,NIN,NOUT).                   UERT0340
C                THE OUTPUT UNIT NUMBER CAN BE CHANGED BY CALLING       UERT0350
C                UGETIO AS FOLLOWS..                                    UERT0360
C                                NIN = 0                                UERT0370
C                                NOUT = NEW OUTPUT UNIT NUMBER          UERT0380
C                                CALL UGETIO(3,NIN,NOUT)                UERT0390
C                SEE THE UGETIO DOCUMENT FOR MORE DETAILS.              UERT0400
C                                                                       UERT0410
C   COPYRIGHT           - 1982 BY IMSL, INC. ALL RIGHTS RESERVED.       UERT0420
C                                                                       UERT0430
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN UERT0440
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    UERT0450
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        UERT0460
C                                                                       UERT0470
C-----------------------------------------------------------------------UERT0480
C                                                                       UERT0490
      SUBROUTINE UERTST (IER,NAME)                                      UERT0500
C                                  SPECIFICATIONS FOR ARGUMENTS         UERT0510
      INTEGER            IER                                            UERT0520
      INTEGER            NAME(1)                                        UERT0530
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   UERT0540
      INTEGER            I,IEQ,IEQDF,IOUNIT,LEVEL,LEVOLD,NAMEQ(6),      UERT0550
     *                   NAMSET(6),NAMUPK(6),NIN,NMTB                   UERT0560
      DATA               NAMSET/1HU,1HE,1HR,1HS,1HE,1HT/                UERT0570
      DATA               NAMEQ/6*1H /                                   UERT0580
      DATA               LEVEL/4/,IEQDF/0/,IEQ/1H=/                     UERT0590
C                                  UNPACK NAME INTO NAMUPK              UERT0600
C                                  FIRST EXECUTABLE STATEMENT           UERT0610
      CALL USPKD (NAME,6,NAMUPK,NMTB)                                   UERT0620
C                                  GET OUTPUT UNIT NUMBER               UERT0630
      CALL UGETIO(1,NIN,IOUNIT)                                         UERT0640
C                                  CHECK IER                            UERT0650
      IF (IER.GT.999) GO TO 25                                          UERT0660
      IF (IER.LT.-32) GO TO 55                                          UERT0670
      IF (IER.LE.128) GO TO 5                                           UERT0680
      IF (LEVEL.LT.1) GO TO 30                                          UERT0690
C                                  PRINT TERMINAL MESSAGE               UERT0700
      IF (IEQDF.EQ.1) WRITE(IOUNIT,35) IER,NAMEQ,IEQ,NAMUPK             UERT0710
      IF (IEQDF.EQ.0) WRITE(IOUNIT,35) IER,NAMUPK                       UERT0720
      GO TO 30                                                          UERT0730
    5 IF (IER.LE.64) GO TO 10                                           UERT0740
      IF (LEVEL.LT.2) GO TO 30                                          UERT0750
C                                  PRINT WARNING WITH FIX MESSAGE       UERT0760
      IF (IEQDF.EQ.1) WRITE(IOUNIT,40) IER,NAMEQ,IEQ,NAMUPK             UERT0770
      IF (IEQDF.EQ.0) WRITE(IOUNIT,40) IER,NAMUPK                       UERT0780
      GO TO 30                                                          UERT0790
   10 IF (IER.LE.32) GO TO 15                                           UERT0800
C                                  PRINT WARNING MESSAGE                UERT0810
      IF (LEVEL.LT.3) GO TO 30                                          UERT0820
      IF (IEQDF.EQ.1) WRITE(IOUNIT,45) IER,NAMEQ,IEQ,NAMUPK             UERT0830
      IF (IEQDF.EQ.0) WRITE(IOUNIT,45) IER,NAMUPK                       UERT0840
      GO TO 30                                                          UERT0850
   15 CONTINUE                                                          UERT0860
C                                  CHECK FOR UERSET CALL                UERT0870
      DO 20 I=1,6                                                       UERT0880
         IF (NAMUPK(I).NE.NAMSET(I)) GO TO 25                           UERT0890
   20 CONTINUE                                                          UERT0900
      LEVOLD = LEVEL                                                    UERT0910
      LEVEL = IER                                                       UERT0920
      IER = LEVOLD                                                      UERT0930
      IF (LEVEL.LT.0) LEVEL = 4                                         UERT0940
      IF (LEVEL.GT.4) LEVEL = 4                                         UERT0950
      GO TO 30                                                          UERT0960
   25 CONTINUE                                                          UERT0970
      IF (LEVEL.LT.4) GO TO 30                                          UERT0980
C                                  PRINT NON-DEFINED MESSAGE            UERT0990
      IF (IEQDF.EQ.1) WRITE(IOUNIT,50) IER,NAMEQ,IEQ,NAMUPK             UERT1000
      IF (IEQDF.EQ.0) WRITE(IOUNIT,50) IER,NAMUPK                       UERT1010
   30 IEQDF = 0                                                         UERT1020
      RETURN                                                            UERT1030
   35 FORMAT(19H *** TERMINAL ERROR,10X,7H(IER = ,I3,                   UERT1040
     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        UERT1050
   40 FORMAT(27H *** WARNING WITH FIX ERROR,2X,7H(IER = ,I3,            UERT1060
     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        UERT1070
   45 FORMAT(18H *** WARNING ERROR,11X,7H(IER = ,I3,                    UERT1080
     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        UERT1090
   50 FORMAT(20H *** UNDEFINED ERROR,9X,7H(IER = ,I5,                   UERT1100
     1       20H) FROM IMSL ROUTINE ,6A1,A1,6A1)                        UERT1110
C                                                                       UERT1120
C                                  SAVE P FOR P = R CASE                UERT1130
C                                    P IS THE PAGE NAMUPK               UERT1140
C                                    R IS THE ROUTINE NAMUPK            UERT1150
   55 IEQDF = 1                                                         UERT1160
      DO 60 I=1,6                                                       UERT1170
   60 NAMEQ(I) = NAMUPK(I)                                              UERT1180
   65 RETURN                                                            UERT1190
      END                                                               UERT1200
C   IMSL ROUTINE NAME   - UGETIO                                        UGET0010
C                                                                       UGET0020
C-----------------------------------------------------------------------UGET0030
C                                                                       UGET0040
C   COMPUTER            - VAX/SINGLE                                    UGET0050
C                                                                       UGET0060
C   LATEST REVISION     - JUNE 1, 1981                                  UGET0070
C                                                                       UGET0080
C   PURPOSE             - TO RETRIEVE CURRENT VALUES AND TO SET NEW     UGET0090
C                           VALUES FOR INPUT AND OUTPUT UNIT            UGET0100
C                           IDENTIFIERS.                                UGET0110
C                                                                       UGET0120
C   USAGE               - CALL UGETIO(IOPT,NIN,NOUT)                    UGET0130
C                                                                       UGET0140
C   ARGUMENTS    IOPT   - OPTION PARAMETER. (INPUT)                     UGET0150
C                           IF IOPT=1, THE CURRENT INPUT AND OUTPUT     UGET0160
C                           UNIT IDENTIFIER VALUES ARE RETURNED IN NIN  UGET0170
C                           AND NOUT, RESPECTIVELY.                     UGET0180
C                           IF IOPT=2, THE INTERNAL VALUE OF NIN IS     UGET0190
C                           RESET FOR SUBSEQUENT USE.                   UGET0200
C                           IF IOPT=3, THE INTERNAL VALUE OF NOUT IS    UGET0210
C                           RESET FOR SUBSEQUENT USE.                   UGET0220
C                NIN    - INPUT UNIT IDENTIFIER.                        UGET0230
C                           OUTPUT IF IOPT=1, INPUT IF IOPT=2.          UGET0240
C                NOUT   - OUTPUT UNIT IDENTIFIER.                       UGET0250
C                           OUTPUT IF IOPT=1, INPUT IF IOPT=3.          UGET0260
C                                                                       UGET0270
C   PRECISION/HARDWARE  - SINGLE/ALL                                    UGET0280
C                                                                       UGET0290
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 UGET0300
C                                                                       UGET0310
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           UGET0320
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      UGET0330
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  UGET0340
C                                                                       UGET0350
C   REMARKS      EACH IMSL ROUTINE THAT PERFORMS INPUT AND/OR OUTPUT    UGET0360
C                OPERATIONS CALLS UGETIO TO OBTAIN THE CURRENT UNIT     UGET0370
C                IDENTIFIER VALUES. IF UGETIO IS CALLED WITH IOPT=2 OR  UGET0380
C                IOPT=3, NEW UNIT IDENTIFIER VALUES ARE ESTABLISHED.    UGET0390
C                SUBSEQUENT INPUT/OUTPUT IS PERFORMED ON THE NEW UNITS. UGET0400
C                                                                       UGET0410
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       UGET0420
C                                                                       UGET0430
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN UGET0440
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    UGET0450
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        UGET0460
C                                                                       UGET0470
C-----------------------------------------------------------------------UGET0480
C                                                                       UGET0490
      SUBROUTINE UGETIO(IOPT,NIN,NOUT)                                  UGET0500
C                                  SPECIFICATIONS FOR ARGUMENTS         UGET0510
      INTEGER            IOPT,NIN,NOUT                                  UGET0520
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   UGET0530
      INTEGER            NIND,NOUTD                                     UGET0540
      DATA               NIND/1/,NOUTD/2/                               UGET0550
C                                  FIRST EXECUTABLE STATEMENT           UGET0560
      IF (IOPT.EQ.3) GO TO 10                                           UGET0570
      IF (IOPT.EQ.2) GO TO 5                                            UGET0580
      IF (IOPT.NE.1) GO TO 9005                                         UGET0590
      NIN = NIND                                                        UGET0600
      NOUT = NOUTD                                                      UGET0610
      GO TO 9005                                                        UGET0620
    5 NIND = NIN                                                        UGET0630
      GO TO 9005                                                        UGET0640
   10 NOUTD = NOUT                                                      UGET0650
 9005 RETURN                                                            UGET0660
      END                                                               UGET0670
C   IMSL ROUTINE NAME   - USPKD                                         USPK0010
C                                                                       USPK0020
C-----------------------------------------------------------------------USPK0030
C                                                                       USPK0040
C   COMPUTER            - VAX/SINGLE                                    USPK0050
C                                                                       USPK0060
C   LATEST REVISION     - JUNE 1, 1982                                  USPK0070
C                                                                       USPK0080
C   PURPOSE             - NUCLEUS CALLED BY IMSL ROUTINES THAT HAVE     USPK0090
C                           CHARACTER STRING ARGUMENTS                  USPK0100
C                                                                       USPK0110
C   USAGE               - CALL USPKD  (PACKED,NCHARS,UNPAKD,NCHMTB)     USPK0120
C                                                                       USPK0130
C   ARGUMENTS    PACKED - CHARACTER STRING TO BE UNPACKED.(INPUT)       USPK0140
C                NCHARS - LENGTH OF PACKED. (INPUT)  SEE REMARKS.       USPK0150
C                UNPAKD - INTEGER ARRAY TO RECEIVE THE UNPACKED         USPK0160
C                         REPRESENTATION OF THE STRING. (OUTPUT)        USPK0170
C                NCHMTB - NCHARS MINUS TRAILING BLANKS. (OUTPUT)        USPK0180
C                                                                       USPK0190
C   PRECISION/HARDWARE  - SINGLE/ALL                                    USPK0200
C                                                                       USPK0210
C   REQD. IMSL ROUTINES - NONE                                          USPK0220
C                                                                       USPK0230
C   REMARKS  1.  USPKD UNPACKS A CHARACTER STRING INTO AN INTEGER ARRAY USPK0240
C                IN (A1) FORMAT.                                        USPK0250
C            2.  UP TO 129 CHARACTERS MAY BE USED.  ANY IN EXCESS OF    USPK0260
C                THAT ARE IGNORED.                                      USPK0270
C                                                                       USPK0280
C   COPYRIGHT           - 1982 BY IMSL, INC. ALL RIGHTS RESERVED.       USPK0290
C                                                                       USPK0300
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN USPK0310
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    USPK0320
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        USPK0330
C                                                                       USPK0340
C-----------------------------------------------------------------------USPK0350
      SUBROUTINE USPKD  (PACKED,NCHARS,UNPAKD,NCHMTB)                   USPK0360
C                                  SPECIFICATIONS FOR ARGUMENTS         USPK0370
      INTEGER            NC,NCHARS,NCHMTB                               USPK0380
C                                                                       USPK0390
      INTEGER            UNPAKD(1),IBLANK                               USPK0400
      INTEGER            PACKED(1)                                      USPK0410
      DATA               IBLANK /1H /                                   USPK0420
C                                  INITIALIZE NCHMTB                    USPK0430
      NCHMTB = 0                                                        USPK0440
C                                  RETURN IF NCHARS IS LE ZERO          USPK0450
      IF(NCHARS.LE.0) RETURN                                            USPK0460
C                                  SET NC=NUMBER OF CHARS TO BE DECODED USPK0470
      NC = MIN0 (129,NCHARS)                                            USPK0480
      DECODE (NC,150,PACKED) (UNPAKD(I),I=1,NC)                         USPK0490
  150 FORMAT (129A1)                                                    USPK0500
C                                  CHECK UNPAKD ARRAY AND SET NCHMTB    USPK0510
C                                  BASED ON TRAILING BLANKS FOUND       USPK0520
      DO 200 N = 1,NC                                                   USPK0530
         NN = NC - N + 1                                                USPK0540
         IF(UNPAKD(NN) .NE. IBLANK) GO TO 210                           USPK0550
  200 CONTINUE                                                          USPK0560
  210 NCHMTB = NN                                                       USPK0570
      RETURN                                                            USPK0580
      END                                                               USPK0590
C   IMSL ROUTINE NAME   - VHS12                                         VHS70010
C                                                                       VHS70020
C-----------------------------------------------------------------------VHS70030
C                                                                       VHS70040
C   COMPUTER            - VAX/SINGLE                                    VHS70050
C                                                                       VHS70060
C   LATEST REVISION     - JANUARY 1, 1978                               VHS70070
C                                                                       VHS70080
C   PURPOSE             - REAL HOUSEHOLDER TRANSFORMATION -             VHS70090
C                           COMPUTATION AND APPLICATIONS.               VHS70100
C                                                                       VHS70110
C   USAGE               - CALL VHS12 (MODE,LP,L1,M,U,INCU,UP,C,INCC,    VHS70120
C                           ICV,NCV)                                    VHS70130
C                                                                       VHS70140
C   ARGUMENTS    MODE   - OPTION PARAMETER. (INPUT)                     VHS70150
C                         IF MODE=1, THE SUBROUTINE COMPUTES A          VHS70160
C                           HOUSEHOLDER TRANSFORMATION AND IF NCV.GT.0, VHS70170
C                           MULTIPLIES IT BY THE SET OF NCV VECTORS     VHS70180
C                           (EACH OF LENGTH M) STORED IN C. FOR A       VHS70190
C                           GIVEN VECTOR V OF LENGTH M AND TWO INTEGER  VHS70200
C                           INDICES LP AND L1 THAT SATISFY              VHS70210
C                           1 .LE. LP .LT. L1 .LE. M, THE SUBROUTINE    VHS70220
C                           DEFINES AN M BY M HOUSEHOLDER               VHS70230
C                           TRANSFORMATION Q WHICH SATIFIES QV=W WHERE  VHS70240
C                           W(I)=V(I) FOR I.LT.LP                       VHS70250
C                           W(LP)=-SIG*SQRT(V(LP)**2+V(L1)**2+...       VHS70260
C                             +V(M)**2)                                 VHS70270
C                             SIG=1  IF V(LP).GE.0                      VHS70280
C                             SIG=-1 IF V(LP).LT.0                      VHS70290
C                           W(I)=V(I) FOR LP.LT.I.LT.L1                 VHS70300
C                           W(I)=0    FOR I.GE.L1.                      VHS70310
C                         IF MODE=2, THE SUBROUTINE ASSUMES THAT A      VHS70320
C                           HOUSEHOLDER TRANSFORMATION HAS ALREADY      VHS70330
C                           BEEN DEFINED BY A PREVIOUS CALL WITH        VHS70340
C                           MODE=1, AND IF NCV.GT.0, MULTIPLIES IT BY   VHS70350
C                           THE SET OF NCV VECTORS (EACH OF LENGTH      VHS70360
C                           M) STORED IN C.                             VHS70370
C                LP     - PARAMETERS THAT DEFINE THE DESIRED            VHS70380
C                L1         HOUSEHOLDER TRANSFORMATION. (INPUT)         VHS70390
C                M          IF THE CONDITION 1.LE.LP.LT.L1.LE.M IS      VHS70400
C                           NOT SATISFIED, THE SUBROUTINE RETURNS TO    VHS70410
C                           THE CALLING PROGRAM WITHOUT PERFORMING      VHS70420
C                           ANY COMPUTATIONS.                           VHS70430
C                U      - VECTOR OF M ELEMENTS. (INPUT, AND OUTPUT IF   VHS70440
C                           MODE=1)                                     VHS70450
C                           THE STORAGE INCREMENT BETWEEN ELEMENTS      VHS70460
C                           OF U IS INCU. (I.E., U(1+(J-1)*INCU),       VHS70470
C                           J=1,...,M). IF MODE=1, THE ARRAY V IS       VHS70480
C                           DEFINED AS V(J)=U(1+(J-1)*INCU),            VHS70490
C                           J=1,...,M.                                  VHS70500
C                         ON OUTPUT, U(1+(LP-1)*INCU) IS SET TO         VHS70510
C                           W(LP) (AS DEFINED ABOVE IN THE DESCRIPTION  VHS70520
C                           OF MODE=1).                                 VHS70530
C                INCU   - INCREMENT BETWEEN ELEMENTS OF U. (INPUT)      VHS70540
C                UP     - SCALAR SET TO V(LP)-W(LP) TO DEFINE THE       VHS70550
C                           HOUSEHOLDER TRANSFORMATION Q. (INPUT IF     VHS70560
C                           MODE=2, OUTPUT IF MODE=1)                   VHS70570
C                C      - VECTOR OF NCV*M ELEMENTS. (INPUT/OUTPUT)      VHS70580
C                           IF NCV.LE.0, C IS NOT USED.                 VHS70590
C                           IF NCV.GT.0, C CONTAINS NCV VECTORS         VHS70600
C                           OF LENGTH M WITH INCREMENT INCC BETWEEN     VHS70610
C                           ELEMENTS OF VECTORS AND INCREMENT ICV       VHS70620
C                           BETWEEN VECTORS. ELEMENT I OF VECTOR J IS   VHS70630
C                           DEFINED AS C(1+(I-1)*INCC+(J-1)*ICV),       VHS70640
C                           I=1,...,M AND J=1,...,NCV.                  VHS70650
C                         ON OUTPUT, C CONTAINS THE SET OF NCV          VHS70660
C                           VECTORS RESULTING FROM MULTIPLYING          VHS70670
C                           THE GIVEN VECTORS BY Q.                     VHS70680
C                INCC   - INCREMENT BETWEEN ELEMENTS OF VECTORS         VHS70690
C                           IN C. (INPUT)                               VHS70700
C                ICV    - INCREMENT BETWEEN VECTORS IN C. (INPUT)       VHS70710
C                NCV    - NUMBER OF VECTORS STORED IN C. (INPUT)        VHS70720
C                                                                       VHS70730
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VHS70740
C                       - SINGLE/H36,H48,H60                            VHS70750
C                                                                       VHS70760
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VHS70770
C                                                                       VHS70780
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VHS70790
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VHS70800
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VHS70810
C                                                                       VHS70820
C   REMARKS  1.  IF U IS A SINGLE SUBSCRIPTED ARRAY OR THE J-TH COLUMN  VHS70830
C                OF A MATRIX, THEN INCU=1. IF U IS THE I-TH ROW OF A    VHS70840
C                MATRIX THEN INCU IS THE ROW DIMENSION OF THE MATRIX    VHS70850
C                EXACTLY AS SPECIFIED IN THE CALLING PROGRAM.           VHS70860
C            2.  IF C IS A DOUBLE SUBSCRIPTED MATRIX AND THE VECTORS    VHS70870
C                ARE THE FIRST NCV COLUMNS OF C, THEN INCC=1 AND ICV    VHS70880
C                IS THE ROW DIMENSION OF C EXACTLY AS SPECIFIED IN      VHS70890
C                THE CALLING PROGRAM. IN THIS CASE C IS REPLACED        VHS70900
C                BY QC. IF THE VECTORS ARE SUCCESSIVE ROWS OF C         VHS70910
C                THEN INCC IS THE ROW DIMENSION OF C EXACTLY AS         VHS70920
C                SPECIFIED IN THE CALLING PROGRAM AND ICV=1. IN THIS    VHS70930
C                CASE C IS REPLACED BY CQ.                              VHS70940
C                                                                       VHS70950
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VHS70960
C                                                                       VHS70970
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VHS70980
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VHS70990
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VHS71000
C                                                                       VHS71010
C-----------------------------------------------------------------------VHS71020
C                                                                       VHS71030
      SUBROUTINE VHS12  (MODE,LP,L1,M,U,INCU,UP,C,INCC,ICV,NCV)         VHS71040
C                                                                       VHS71050
C                                  SPECIFICATIONS FOR ARGUMENTS         VHS71060
      INTEGER            MODE,LP,L1,M,INCU,INCC,ICV,NCV                 VHS71070
      REAL               U(1),UP,C(1)                                   VHS71080
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   VHS71090
      INTEGER            IJ,ILP,IL1,IM,INCR,I2,I3,I4,J                  VHS71100
      DOUBLE PRECISION   SM,B                                           VHS71110
      REAL               ONE,CL,CLINV,SM1                               VHS71120
C                                  FIRST EXECUTABLE STATEMENT           VHS71130
      ONE = 1.                                                          VHS71140
C                                                                       VHS71150
      IF (0.GE.LP.OR.LP.GE.L1.OR.L1.GT.M) GO TO 9005                    VHS71160
      ILP = (LP-1)*INCU+1                                               VHS71170
      IL1 = (L1-1)*INCU+1                                               VHS71180
      IM = (M-1)*INCU+1                                                 VHS71190
      CL = ABS(U(ILP))                                                  VHS71200
      IF (MODE.EQ.2) GO TO 15                                           VHS71210
C                                  CONSTRUCT THE TRANSFORMATION.        VHS71220
      DO 5 IJ=IL1,IM,INCU                                               VHS71230
    5 CL = AMAX1(ABS(U(IJ)),CL)                                         VHS71240
      IF (CL.LE.0.0) GO TO 9005                                         VHS71250
      CLINV = ONE/CL                                                    VHS71260
      SM = (DBLE(U(ILP))*CLINV)**2                                      VHS71270
      DO 10 IJ=IL1,IM,INCU                                              VHS71280
   10 SM = SM+(DBLE(U(IJ))*CLINV)**2                                    VHS71290
C                                  CONVERT DBLE. PREC. SM TO SNGL.      VHS71300
C                                    PREC. SM1                          VHS71310
      SM1 = SM                                                          VHS71320
      CL = CL*SQRT(SM1)                                                 VHS71330
      IF (U(ILP).GT.0.0) CL = -CL                                       VHS71340
      UP = U(ILP)-CL                                                    VHS71350
      U(ILP) = CL                                                       VHS71360
      GO TO 20                                                          VHS71370
C                                  APPLY THE TRANSFORMATION             VHS71380
C                                    I+U*(U**T)/B TO C.                 VHS71390
   15 IF (CL.LE.0.0) GO TO 9005                                         VHS71400
   20 IF (NCV.LE.0) GO TO 9005                                          VHS71410
      B = DBLE(UP)*U(ILP)                                               VHS71420
C                                  B MUST BE NONPOSITIVE HERE. IF B =   VHS71430
C                                    0., RETURN.                        VHS71440
      IF (B.GE.0.0) GO TO 9005                                          VHS71450
      B = ONE/B                                                         VHS71460
      I2 = 1-ICV+INCC*(LP-1)                                            VHS71470
      INCR = INCC*(L1-LP)                                               VHS71480
      DO 35 J=1,NCV                                                     VHS71490
         I2 = I2+ICV                                                    VHS71500
         I3 = I2+INCR                                                   VHS71510
         I4 = I3                                                        VHS71520
         SM = C(I2)*DBLE(UP)                                            VHS71530
         DO 25 IJ=IL1,IM,INCU                                           VHS71540
            SM = SM+C(I3)*DBLE(U(IJ))                                   VHS71550
            I3 = I3+INCC                                                VHS71560
   25    CONTINUE                                                       VHS71570
         IF (SM.EQ.0.0) GO TO 35                                        VHS71580
         SM = SM*B                                                      VHS71590
         C(I2) = C(I2)+SM*DBLE(UP)                                      VHS71600
         DO 30 IJ=IL1,IM,INCU                                           VHS71610
            C(I4) = C(I4)+SM*DBLE(U(IJ))                                VHS71620
            I4 = I4+INCC                                                VHS71630
   30    CONTINUE                                                       VHS71640
   35 CONTINUE                                                          VHS71650
 9005 RETURN                                                            VHS71660
      END                                                               VHS71670
C   IMSL ROUTINE NAME   - VMULBB                                        VMBB0010
C                                                                       VMBB0020
C-----------------------------------------------------------------------VMBB0030
C                                                                       VMBB0040
C   COMPUTER            - VAX/SINGLE                                    VMBB0050
C                                                                       VMBB0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMBB0070
C                                                                       VMBB0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND STORAGE MODE)     VMBB0090
C                                                                       VMBB0100
C   USAGE               - CALL VMULBB (A,IA,B,IB,N,C,IC)                VMBB0110
C                                                                       VMBB0120
C   ARGUMENTS    A      - N(1) BY N(1) BAND MATRIX WITH N(2) LOWER      VMBB0130
C                           CODIAGINALS AND N(3) UPPER CODIAGONALS.     VMBB0140
C                           A IS STORED IN BAND STORAGE MODE AND        VMBB0150
C                           THEREFORE HAS DIMENSION N(1) BY             VMBB0160
C                           (N(2)+N(3)+1). (INPUT)                      VMBB0170
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMBB0180
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBB0190
C                           CALLING PROGRAM. (INPUT)                    VMBB0200
C                B      - N(1) BY N(1) BAND MATRIX WITH N(4) LOWER      VMBB0210
C                           CODIAGONALS AND N(5) UPPER CODIAGONALS. B   VMBB0220
C                           IS STORED IN BAND STORAGE MODE AND          VMBB0230
C                           THEREFORE HAS DIMENSION N(1) BY             VMBB0240
C                           (N(4)+N(5)+1). (INPUT)                      VMBB0250
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMBB0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBB0270
C                           CALLING PROGRAM. (INPUT)                    VMBB0280
C                N      - INPUT VECTOR OF LENGTH 5 CONTAINING THE       VMBB0290
C                           FOLLOWING:                                  VMBB0300
C                           N(1) CONTAINS THE NUMBER OF ROWS AND        VMBB0310
C                             COLUMNS IN A,B,AND C.                     VMBB0320
C                           N(2) CONTAINS THE NUMBER OF LOWER           VMBB0330
C                             CODIAGONALS IN MATRIX A.                  VMBB0340
C                           N(3) CONTAINS THE NUMBER OF UPPER           VMBB0350
C                             CODIAGONALS IN A.                         VMBB0360
C                           N(4) CONTAINS THE NUMBER OF LOWER           VMBB0370
C                             CODIAGONALS IN B.                         VMBB0380
C                           N(5) CONTAINS THE NUMBER OF UPPER           VMBB0390
C                             CODIAGONALS IN B.                         VMBB0400
C                C      - OUTPUT MATRIX CONTAINING THE PRODUCT C=A*B.   VMBB0410
C                           C IS A BAND MATRIX WITH M LOWER CODIAGONALS,VMBB0420
C                           WHERE M=MIN(N(1)-1,N(2)+N(4)) AND L UPPER   VMBB0430
C                           CODIAGONALS, WHERE L=MIN(N(1)-1,N(3)+N(5)). VMBB0440
C                           C IS STORED IN BAND STORAGE MODE AND        VMBB0450
C                           THEREFORE HAS DIMENSION N(1) BY (L+M+1).    VMBB0460
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMBB0470
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBB0480
C                           CALLING PROGRAM. (INPUT)                    VMBB0490
C                                                                       VMBB0500
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMBB0510
C                       - SINGLE/H36,H48,H60                            VMBB0520
C                                                                       VMBB0530
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMBB0540
C                                                                       VMBB0550
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMBB0560
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMBB0570
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMBB0580
C                                                                       VMBB0590
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMBB0600
C                                                                       VMBB0610
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMBB0620
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMBB0630
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMBB0640
C                                                                       VMBB0650
C-----------------------------------------------------------------------VMBB0660
C                                                                       VMBB0670
      SUBROUTINE VMULBB (A,IA,B,IB,N,C,IC)                              VMBB0680
C                                                                       VMBB0690
      INTEGER            N(5)                                           VMBB0700
      REAL               A(IA,1),B(IB,1),C(IC,1),ZERO                   VMBB0710
      DOUBLE PRECISION   SUM                                            VMBB0720
      DATA               ZERO/0.0/                                      VMBB0730
C                                  FIRST EXECUTABLE STATEMENT           VMBB0740
      NN = N(1)                                                         VMBB0750
      NLA = N(2)                                                        VMBB0760
      NLAP2 = NLA+2                                                     VMBB0770
      NUA = N(3)                                                        VMBB0780
      NLB = N(4)                                                        VMBB0790
      NUB = N(5)                                                        VMBB0800
      NCA = NLA+NUA+1                                                   VMBB0810
      NCB = NUB+NLB+1                                                   VMBB0820
      NMNUC = NN-NUA                                                    VMBB0830
      NLC = MIN0(NLB+NLA,NN-1)                                          VMBB0840
      NLCP1 =NLC+1                                                      VMBB0850
      NUC = MIN0(NUB+NUA,NN-1)                                          VMBB0860
      MM = NUC+NLC+1                                                    VMBB0870
      N1 = NN+1                                                         VMBB0880
C                                  ZERO OUT CORNERS OF THE              VMBB0890
C                                  RESULTING MATRIX                     VMBB0900
      IF (NLC .LE. 0) GO TO 15                                          VMBB0910
      NX = NLC                                                          VMBB0920
      DO 10 I = 1,NLC                                                   VMBB0930
         DO 5 J = 1,NX                                                  VMBB0940
            C(I,J) = ZERO                                               VMBB0950
    5    CONTINUE                                                       VMBB0960
         NX = NX-1                                                      VMBB0970
   10 CONTINUE                                                          VMBB0980
   15 IF (NUC .LE. 0) GO TO 30                                          VMBB0990
      NX = NLC+2                                                        VMBB1000
      DO 25 I = 1,NUC                                                   VMBB1010
         DO 20 J = NX,MM                                                VMBB1020
            C(N1-I,J) = ZERO                                            VMBB1030
   20    CONTINUE                                                       VMBB1040
         NX = NX+1                                                      VMBB1050
   25 CONTINUE                                                          VMBB1060
   30 NCB = NUB+NLB+1                                                   VMBB1070
      JBBEG = NLC                                                       VMBB1080
      JCBEG = NLC+1                                                     VMBB1090
      DO 50 IRA = 1,NN                                                  VMBB1100
         DO 45 JB = 1,NN                                                VMBB1110
            JABEG = NLAP2-IRA                                           VMBB1120
            IRB = 1                                                     VMBB1130
            JBBEG = NLB+JB                                              VMBB1140
            KL = MAX0(IDIM(JBBEG,NCB),IDIM(1,JABEG))                    VMBB1150
            JABEG = JABEG+KL                                            VMBB1160
            JBBEG = JBBEG-KL                                            VMBB1170
            IRB = IRB+KL                                                VMBB1180
            IF (JABEG .GT. NCA) GO TO 45                                VMBB1190
            IF (IRB .LT. 1) GO TO 45                                    VMBB1200
            IF (JBBEG .LT. 1) GO TO 45                                  VMBB1210
            SUM = 0.0D0                                                 VMBB1220
            DO 35 L = 1,NN                                              VMBB1230
               SUM = SUM+DBLE(A(IRA,JABEG))*DBLE(B(IRB,JBBEG))          VMBB1240
               JABEG = JABEG+1                                          VMBB1250
               IRB = IRB+1                                              VMBB1260
               JBBEG = JBBEG-1                                          VMBB1270
               IF (JABEG .GT. NCA) GO TO 40                             VMBB1280
               IF (JBBEG .LT. 1) GO TO 40                               VMBB1290
               IF (IRB .GT. NN) GO TO 40                                VMBB1300
   35       CONTINUE                                                    VMBB1310
   40       C(IRA,JCBEG) = SUM                                          VMBB1320
            JCBEG = JCBEG+1                                             VMBB1330
   45    CONTINUE                                                       VMBB1340
         JCBEG = MAX0(NLCP1-IRA,1)                                      VMBB1350
         IF (IRA .GT. NMNUC) NCA = NCA-1                                VMBB1360
   50 CONTINUE                                                          VMBB1370
      RETURN                                                            VMBB1380
      END                                                               VMBB1390
C   IMSL ROUTINE NAME   - VMULBF                                        VMBF0010
C                                                                       VMBF0020
C-----------------------------------------------------------------------VMBF0030
C                                                                       VMBF0040
C   COMPUTER            - VAX/SINGLE                                    VMBF0050
C                                                                       VMBF0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMBF0070
C                                                                       VMBF0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND BY FULL MATRICES) VMBF0090
C                                                                       VMBF0100
C   USAGE               - CALL VMULBF (A,IA,B,IB,N,C,IC)                VMBF0110
C                                                                       VMBF0120
C   ARGUMENTS    A      - N(1) BY N(1) BAND MATRIX WITH N(2) LOWER      VMBF0130
C                           CODIAGONALS AND N(3) UPPER CODIAGONALS. A   VMBF0140
C                           IS STORED IN BAND STORAGE MODE AND          VMBF0150
C                           THEREFORE HAS DIMENSION N(1) BY             VMBF0160
C                           (N(2)+N(3)+1). (INPUT)                      VMBF0170
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMBF0180
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBF0190
C                           CALLING PROGRAM. (INPUT)                    VMBF0200
C                B      - N(1) BY N(4) MATRIX IN FULL STORAGE MODE.     VMBF0210
C                           (INPUT)                                     VMBF0220
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMBF0230
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBF0240
C                           CALLING PROGRAM. (INPUT)                    VMBF0250
C                N      - INPUT VECTOR OF LENGTH 4 CONTAINING THE       VMBF0260
C                           FOLLOWING INFORMATION:                      VMBF0270
C                           N(1) CONTAINS THE NUMBER OF ROWS IN A,B,    VMBF0280
C                             AND C AND THE NUMBER OF COLUMNS IN A.     VMBF0290
C                           N(2) CONTAINS THE NUMBER OF LOWER           VMBF0300
C                             CODIAGONALS IN A.                         VMBF0310
C                           N(3) CONTAINS THE NUMBER OF UPPER           VMBF0320
C                             CODIAGONALS IN A.                         VMBF0330
C                           N(4) CONTAINS THE NUMBER OF COLUMNS IN B    VMBF0340
C                             AND C.                                    VMBF0350
C                C      - N(1) BY N(4) MATRIX CONTAINING THE PRODUCT    VMBF0360
C                           C = A*B. C IS STORED IN FULL STORAGE MODE.  VMBF0370
C                           (OUTPUT)                                    VMBF0380
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMBF0390
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBF0400
C                           CALLING PROGRAM. (INPUT)                    VMBF0410
C                                                                       VMBF0420
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMBF0430
C                       - SINGLE/H36,H48,H60                            VMBF0440
C                                                                       VMBF0450
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMBF0460
C                                                                       VMBF0470
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMBF0480
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMBF0490
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMBF0500
C                                                                       VMBF0510
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMBF0520
C                                                                       VMBF0530
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMBF0540
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMBF0550
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMBF0560
C                                                                       VMBF0570
C-----------------------------------------------------------------------VMBF0580
C                                                                       VMBF0590
      SUBROUTINE VMULBF (A,IA,B,IB,N,C,IC)                              VMBF0600
C                                                                       VMBF0610
      INTEGER            N(4)                                           VMBF0620
      REAL               A(IA,1),B(IB,1),C(IC,1)                        VMBF0630
      DOUBLE PRECISION   SUM                                            VMBF0640
C                                  FIRST EXECUTABLE STATEMENT           VMBF0650
      NN = N(1)                                                         VMBF0660
      NLC = N(2)                                                        VMBF0670
      NUC = N(3)                                                        VMBF0680
      NCC = NLC+NUC+1                                                   VMBF0690
      M = N(4)                                                          VMBF0700
      NLCP1 = NLC+1                                                     VMBF0710
      NLCP2 = NLC+2                                                     VMBF0720
      NMNUC = NN-NUC                                                    VMBF0730
      DO 15 JC = 1,M                                                    VMBF0740
         NC = NCC                                                       VMBF0750
         KK = 1                                                         VMBF0760
         DO 10 I = 1,NN                                                 VMBF0770
            L = NLCP2-I                                                 VMBF0780
            IR = MAX0(L,1)                                              VMBF0790
            IF (L .LE. 0) KK = KK+1                                     VMBF0800
            K = KK                                                      VMBF0810
            SUM = 0.0D0                                                 VMBF0820
            DO 5 J = IR,NC                                              VMBF0830
               SUM = SUM+DBLE(A(I,J))*DBLE(B(K,JC))                     VMBF0840
               K = K+1                                                  VMBF0850
    5       CONTINUE                                                    VMBF0860
            C(I,JC) = SUM                                               VMBF0870
            IF (I .GE. NMNUC) NC = NC-1                                 VMBF0880
   10    CONTINUE                                                       VMBF0890
   15 CONTINUE                                                          VMBF0900
      RETURN                                                            VMBF0910
      END                                                               VMBF0920
C   IMSL ROUTINE NAME   - VMULBS                                        VMBS0010
C                                                                       VMBS0020
C-----------------------------------------------------------------------VMBS0030
C                                                                       VMBS0040
C   COMPUTER            - VAX/SINGLE                                    VMBS0050
C                                                                       VMBS0060
C   LATEST REVISION     - JUNE 1, 1980                                  VMBS0070
C                                                                       VMBS0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND BY SYMMETRIC      VMBS0090
C                           MATRICES)                                   VMBS0100
C                                                                       VMBS0110
C   USAGE               - CALL VMULBS (A,IA,B,N,C,IC)                   VMBS0120
C                                                                       VMBS0130
C   ARGUMENTS    A      - N(1) BY N(1) BAND MATRIX WITH N(2) LOWER      VMBS0140
C                           CODIAGONALS AND N(3) UPPER CODIAGONALS. A   VMBS0150
C                           IS STORED IN BAND MODE AND THEREFORE HAS    VMBS0160
C                           DIMENSION N(1) BY (N(2)+N(3)+1). (INPUT)    VMBS0170
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMBS0180
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBS0190
C                           CALLING PROGRAM. (INPUT)                    VMBS0200
C                B      - N(1) BY N(1) SYMMETRIC MATRIX. B IS STORED    VMBS0210
C                           IN SYMMETRIC STORAGE MODE AND THEREFORE     VMBS0220
C                           HAS DIMENSION N(1)*(N(1)+1)/2. (INPUT)      VMBS0230
C                N      - INPUT VECTOR OF LENGTH 3 CONTAINING THE       VMBS0240
C                           FOLLOWING MATRIX INFORMATION -              VMBS0250
C                         N(1) CONTAINS THE NUMBER OF ROWS AND COLUMNS  VMBS0260
C                           IN MATRICES A,B,AND C.                      VMBS0270
C                         N(2) CONTAINS THE NUMBER OF LOWER CODIAGONALS VMBS0280
C                           IN MATRIX A.                                VMBS0290
C                         N(3) CONTAINS THE NUMBER OF UPPER CODIAGONALS VMBS0300
C                           IN MATRIX A.                                VMBS0310
C                C      - N(1) BY N(1) MATRIX CONTAINING THE PRODUCT    VMBS0320
C                           C = A*B. C IS STORED IN FULL STORAGE MODE.  VMBS0330
C                           (OUTPUT)                                    VMBS0340
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMBS0350
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMBS0360
C                           CALLING PROGRAM. (INPUT)                    VMBS0370
C                                                                       VMBS0380
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMBS0390
C                       - SINGLE/H36,H48,H60                            VMBS0400
C                                                                       VMBS0410
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMBS0420
C                                                                       VMBS0430
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMBS0440
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMBS0450
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMBS0460
C                                                                       VMBS0470
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMBS0480
C                                                                       VMBS0490
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMBS0500
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMBS0510
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMBS0520
C                                                                       VMBS0530
C-----------------------------------------------------------------------VMBS0540
C                                                                       VMBS0550
      SUBROUTINE VMULBS (A,IA,B,N,C,IC)                                 VMBS0560
C                                                                       VMBS0570
C                                  SPECIFICATIONS FOR ARGUMENTS         VMBS0580
      INTEGER            N(3),IA,IC                                     VMBS0590
      REAL               A(IA,1),B(1),C(IC,1)                           VMBS0600
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   VMBS0610
      DOUBLE PRECISION   SUM                                            VMBS0620
C                                  FIRST EXECUTABLE STATEMENT           VMBS0630
      NN = N(1)                                                         VMBS0640
      KBEG = 1+N(2)                                                     VMBS0650
      KEND = KBEG+N(3)                                                  VMBS0660
      JCNT = 0                                                          VMBS0670
      N1MN3 = NN-N(3)                                                   VMBS0680
      DO 30 I = 1,NN                                                    VMBS0690
         IBEG1 = 1                                                      VMBS0700
         DO 25 J = 1,NN                                                 VMBS0710
            IBEG = IBEG1                                                VMBS0720
            ICNT = 0                                                    VMBS0730
            SUM = 0.0D0                                                 VMBS0740
            IF(I .LE. N(2)+1) GO TO 15                                  VMBS0750
            IBEG = IBEG+MIN0(JCNT,J-1)                                  VMBS0760
            IF(JCNT .LT. J) GO TO 10                                    VMBS0770
            DO 5 KKK = J,JCNT                                           VMBS0780
               IBEG = IBEG+KKK                                          VMBS0790
    5       CONTINUE                                                    VMBS0800
   10       ICNT = JCNT                                                 VMBS0810
   15       DO 20 K = KBEG,KEND                                         VMBS0820
               ICNTO = ICNT                                             VMBS0830
               ICNT = ICNT+1                                            VMBS0840
               SUM = SUM+DBLE(A(I,K))*DBLE(B(IBEG))                     VMBS0850
               IBEG = IBEG+1                                            VMBS0860
               IF(ICNT .GE. J) IBEG = IBEG+ICNTO                        VMBS0870
   20       CONTINUE                                                    VMBS0880
            C(I,J) = SUM                                                VMBS0890
            IBEG1 = IBEG1+J                                             VMBS0900
   25    CONTINUE                                                       VMBS0910
         IF(I .GT. N(2)) JCNT = JCNT+1                                  VMBS0920
         IF(I .GE. N1MN3) KEND = KEND-1                                 VMBS0930
         KBEG = MAX0(KBEG-1,1)                                          VMBS0940
   30 CONTINUE                                                          VMBS0950
      RETURN                                                            VMBS0960
      END                                                               VMBS0970
C   IMSL ROUTINE NAME   - VMULFB                                        VMFB0010
C                                                                       VMFB0020
C-----------------------------------------------------------------------VMFB0030
C                                                                       VMFB0040
C   COMPUTER            - VAX/SINGLE                                    VMFB0050
C                                                                       VMFB0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMFB0070
C                                                                       VMFB0080
C   PURPOSE             - MATRIX MULTIPLICATION (FULL BY BAND MATRICES) VMFB0090
C                                                                       VMFB0100
C   USAGE               - CALL VMULFB (A,IA,B,IB,N,C,IC)                VMFB0110
C                                                                       VMFB0120
C   ARGUMENTS    A      - N(1) BY N(2) FULL MATRIX STORED IN FULL       VMFB0130
C                           STORAGE MODE. (INPUT)                       VMFB0140
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMFB0150
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFB0160
C                           CALLING PROGRAM. (INPUT)                    VMFB0170
C                B      - N(2) BY N(2) BAND MATRIX WITH N(3) LOWER      VMFB0180
C                           CODIAGONALS AND N(4) UPPER CODIAGONALS. B   VMFB0190
C                           IS STORED IN BAND STORAGE MODE AND          VMFB0200
C                           THEREFORE HAS DIMENSION N(2) BY             VMFB0210
C                           (N(3)+N(4)+1). (INPUT)                      VMFB0220
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMFB0230
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFB0240
C                           CALLING PROGRAM. (INPUT)                    VMFB0250
C                N      - INPUT VECTOR OF LENGTH 4 CONTAINING THE       VMFB0260
C                           FOLLOWING INFORMATION:                      VMFB0270
C                         N(1) CONTAINS THE NUMBER OF ROWS IN MATRICES  VMFB0280
C                           A AND C.                                    VMFB0290
C                         N(2) CONTAINS THE NUMBER OF COLUMNS IN MATRIX VMFB0300
C                           A,B AND C AND THE NUMBER OF ROWS IN MATRIX  VMFB0310
C                           B.                                          VMFB0320
C                         N(3) CONTAINS THE NUMBER OF LOWER CODIAGONALS VMFB0330
C                           IN MATRIX B.                                VMFB0340
C                         N(4) CONTAINS THE NUMBER OF UPPER CODIAGONALS VMFB0350
C                           IN MATRIX B.                                VMFB0360
C                C      - N(1) BY N(2) MATRIX CONTAINING THE PRODUCT    VMFB0370
C                           C = A*B. C IS STORED IN FULL STORAGE MODE.  VMFB0380
C                           (OUTPUT)                                    VMFB0390
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMFB0400
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFB0410
C                           CALLING PROGRAM. (INPUT)                    VMFB0420
C                                                                       VMFB0430
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFB0440
C                       - SINGLE/H36,H48,H60                            VMFB0450
C                                                                       VMFB0460
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMFB0470
C                                                                       VMFB0480
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFB0490
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFB0500
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFB0510
C                                                                       VMFB0520
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFB0530
C                                                                       VMFB0540
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFB0550
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFB0560
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFB0570
C                                                                       VMFB0580
C-----------------------------------------------------------------------VMFB0590
C                                                                       VMFB0600
      SUBROUTINE VMULFB (A,IA,B,IB,N,C,IC)                              VMFB0610
C                                                                       VMFB0620
      INTEGER            N(4)                                           VMFB0630
      REAL               A(IA,1),B(IB,1),C(IC,1)                        VMFB0640
      DOUBLE PRECISION   SUM                                            VMFB0650
C                                  FIRST EXECUTABLE STATEMENT           VMFB0660
      NRA = N(1)                                                        VMFB0670
      NCA = N(2)                                                        VMFB0680
      NLC = N(3)                                                        VMFB0690
      NUC = N(4)                                                        VMFB0700
      NLCP1 = NLC+1                                                     VMFB0710
      NC = NLCP1+NUC                                                    VMFB0720
      NCA1 = NCA-NLC                                                    VMFB0730
      DO 25 I = 1,NRA                                                   VMFB0740
         LCA = 1                                                        VMFB0750
         JCB1 = NLCP1                                                   VMFB0760
         IRB1 = 1                                                       VMFB0770
         ICNT = 0                                                       VMFB0780
         DO 20 J = 1,NCA                                                VMFB0790
            IRB = IRB1                                                  VMFB0800
            JCB = JCB1                                                  VMFB0810
            LC = LCA                                                    VMFB0820
            SUM = 0.0D0                                                 VMFB0830
            L = MIN0(JCB,NC)                                            VMFB0840
            IF (J .LE. NCA1) GO TO 5                                    VMFB0850
            ICNT = ICNT+1                                               VMFB0860
            L = L-ICNT                                                  VMFB0870
    5       DO 10 K = 1,L                                               VMFB0880
               SUM = SUM+DBLE(A(I,LC))*DBLE(B(IRB,JCB))                 VMFB0890
               JCB = JCB-1                                              VMFB0900
               IRB = IRB+1                                              VMFB0910
               LC = LC+1                                                VMFB0920
   10       CONTINUE                                                    VMFB0930
            C(I,J) = SUM                                                VMFB0940
            IF (JCB1 .LT. NC) GO TO 15                                  VMFB0950
            IRB1 = IRB1+1                                               VMFB0960
            LCA = LCA+1                                                 VMFB0970
            GO TO 20                                                    VMFB0980
   15       JCB1 = JCB1+1                                               VMFB0990
   20    CONTINUE                                                       VMFB1000
   25 CONTINUE                                                          VMFB1010
      RETURN                                                            VMFB1020
      END                                                               VMFB1030
C   IMSL ROUTINE NAME   - VMULFF                                        VMFF0010
C                                                                       VMFF0020
C-----------------------------------------------------------------------VMFF0030
C                                                                       VMFF0040
C   COMPUTER            - VAX/SINGLE                                    VMFF0050
C                                                                       VMFF0060
C   LATEST REVISION     - NOVEMBER 1, 1979                              VMFF0070
C                                                                       VMFF0080
C   PURPOSE             - MATRIX MULTIPLICATION (FULL STORAGE MODE)     VMFF0090
C                                                                       VMFF0100
C   USAGE               - CALL VMULFF (A,B,L,M,N,IA,IB,C,IC,IER)        VMFF0110
C                                                                       VMFF0120
C   ARGUMENTS    A      - L BY M MATRIX STORED IN FULL STORAGE MODE.    VMFF0130
C                           (INPUT)                                     VMFF0140
C                B      - M BY N MATRIX STORED IN FULL STORAGE MODE.    VMFF0150
C                           (INPUT)                                     VMFF0160
C                L      - NUMBER OF ROWS IN A. (INPUT)                  VMFF0170
C                M      - NUMBER OF COLUMNS IN A (SAME AS NUMBER OF     VMFF0180
C                           ROWS IN B). (INPUT)                         VMFF0190
C                N      - NUMBER OF COLUMNS IN B. (INPUT)               VMFF0200
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMFF0210
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFF0220
C                           CALLING PROGRAM. (INPUT)                    VMFF0230
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMFF0240
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFF0250
C                           CALLING PROGRAM. (INPUT)                    VMFF0260
C                C      - L BY N MATRIX CONTAINING THE PRODUCT          VMFF0270
C                           C = A*B. (OUTPUT)                           VMFF0280
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMFF0290
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFF0300
C                           CALLING PROGRAM. (INPUT)                    VMFF0310
C                IER    - ERROR PARAMETER. (OUTPUT)                     VMFF0320
C                         TERMINAL ERROR                                VMFF0330
C                           IER=129 INDICATES A,B,OR C WAS DIMENSIONED  VMFF0340
C                             INCORRECTLY.                              VMFF0350
C                                                                       VMFF0360
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFF0370
C                       - SINGLE/H36,H48,H60                            VMFF0380
C                                                                       VMFF0390
C   REQD. IMSL ROUTINES - UERTST,UGETIO                                 VMFF0400
C                                                                       VMFF0410
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFF0420
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFF0430
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFF0440
C                                                                       VMFF0450
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFF0460
C                                                                       VMFF0470
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFF0480
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFF0490
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFF0500
C                                                                       VMFF0510
C-----------------------------------------------------------------------VMFF0520
C                                                                       VMFF0530
      SUBROUTINE VMULFF (A,B,L,M,N,IA,IB,C,IC,IER)                      VMFF0540
C                                                                       VMFF0550
C                                  SPECIFICATIONS FOR ARGUMENTS         VMFF0560
      INTEGER            L,M,N,IA,IB,IC,IER                             VMFF0570
      REAL               A(IA,M),B(IB,N),C(IC,N)                        VMFF0580
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   VMFF0590
      DOUBLE PRECISION   TEMP                                           VMFF0600
C                                  FIRST EXECUTABLE STATEMENT           VMFF0610
      IF (IA .GE. L .AND. IB .GE. M .AND. IC .GE. L) GO TO 5            VMFF0620
C                                  TERMINAL ERROR                       VMFF0630
      IER=129                                                           VMFF0640
      GO TO 9000                                                        VMFF0650
C                                  ROW INDICATOR                        VMFF0660
    5 IER = 0                                                           VMFF0670
      DO 15 I=1,L                                                       VMFF0680
C                                  COLUMN INDICATOR                     VMFF0690
         DO 15 J=1,N                                                    VMFF0700
            TEMP=0.0D0                                                  VMFF0710
C                                  VECTOR DOT PRODUCT                   VMFF0720
            DO 10 K=1,M                                                 VMFF0730
               TEMP=DBLE(A(I,K))*DBLE(B(K,J))+TEMP                      VMFF0740
   10       CONTINUE                                                    VMFF0750
            C(I,J)=TEMP                                                 VMFF0760
   15 CONTINUE                                                          VMFF0770
      GO TO 9005                                                        VMFF0780
 9000 CONTINUE                                                          VMFF0790
      CALL UERTST (IER,6HVMULFF)                                        VMFF0800
 9005 RETURN                                                            VMFF0810
      END                                                               VMFF0820
C   IMSL ROUTINE NAME   - VMULFM                                        VMFM0010
C                                                                       VMFM0020
C-----------------------------------------------------------------------VMFM0030
C                                                                       VMFM0040
C   COMPUTER            - VAX/SINGLE                                    VMFM0050
C                                                                       VMFM0060
C   LATEST REVISION     - JUNE 1, 1980                                  VMFM0070
C                                                                       VMFM0080
C   PURPOSE             - MATRIX MULTIPLICATION OF THE TRANSPOSE OF     VMFM0090
C                           MATRIX A BY MATRIX B (FULL STORAGE MODE)    VMFM0100
C                                                                       VMFM0110
C   USAGE               - CALL VMULFM (A,B,L,M,N,IA,IB,C,IC,IER)        VMFM0120
C                                                                       VMFM0130
C   ARGUMENTS    A      - L BY M MATRIX STORED IN FULL STORAGE MODE.    VMFM0140
C                           (INPUT)                                     VMFM0150
C                B      - L BY N MATRIX STORED IN FULL STORAGE MODE.    VMFM0160
C                           (INPUT)                                     VMFM0170
C                L      - NUMBER OF ROWS IN A AND B. (INPUT)            VMFM0180
C                M      - NUMBER OF COLUMNS IN MATRIX A AND NUMBER      VMFM0190
C                           OF ROWS IN MATRIX C. (INPUT)                VMFM0200
C                N      - NUMBER OF COLUMNS IN B AND C. (INPUT)         VMFM0210
C                IA     - ROW DIMENSION OF A EXACTLY AS SPECIFIED IN    VMFM0220
C                           IN THE DIMENSION STATEMENT IN THE CALLING   VMFM0230
C                           PROGRAM. (INPUT)                            VMFM0240
C                IB     - ROW DIMENSION OF B EXACTLY AS SPECIFIED IN    VMFM0250
C                           THE DIMENSION STATEMENT OF THE CALLING      VMFM0260
C                           PROGRAM. (INPUT)                            VMFM0270
C                C      - M BY N MATRIX CONTAINING THE PRODUCT          VMFM0280
C                           C = (A-TRANSPOSE) * B. (OUTPUT)             VMFM0290
C                IC     - ROW DIMENSION OF C EXACTLY AS SPECIFIED IN    VMFM0300
C                           THE DIMENSION STATEMENT IN THE CALLING      VMFM0310
C                           PROGRAM. (INPUT)                            VMFM0320
C                IER    - ERROR PARAMETER. (OUTPUT)                     VMFM0330
C                         IER=0 IMPLIES NO ERROR.                       VMFM0340
C                         TERMINAL ERROR                                VMFM0350
C                           IER=129 INDICATES A,B, OR C WAS DIMENSIONED VMFM0360
C                             INCORRECTLY.                              VMFM0370
C                                                                       VMFM0380
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFM0390
C                       - SINGLE/H36,H48,H60                            VMFM0400
C                                                                       VMFM0410
C   REQD. IMSL ROUTINES - UERTST,UGETIO                                 VMFM0420
C                                                                       VMFM0430
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFM0440
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFM0450
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFM0460
C                                                                       VMFM0470
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFM0480
C                                                                       VMFM0490
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFM0500
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFM0510
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFM0520
C                                                                       VMFM0530
C-----------------------------------------------------------------------VMFM0540
C                                                                       VMFM0550
      SUBROUTINE VMULFM (A,B,L,M,N,IA,IB,C,IC,IER)                      VMFM0560
C                                                                       VMFM0570
C                                  SPECIFICATIONS FOR ARGUMENTS         VMFM0580
      INTEGER            L,M,N,IA,IB,IC,IER                             VMFM0590
      REAL               A(IA,1),B(IB,1),C(IC,1)                        VMFM0600
C                                  SPECIFICATIONS FOR LOCAL VARIABLES   VMFM0610
      DOUBLE PRECISION   TEMP                                           VMFM0620
C                                  FIRST EXECUTABLE STATEMENT           VMFM0630
      IF (IA.GE.L .AND. IB.GE.L .AND. IC.GE.M) GO TO 5                  VMFM0640
C                                  TERMINAL ERROR                       VMFM0650
      IER = 129                                                         VMFM0660
      GO TO 9000                                                        VMFM0670
C                                  ROW INDICATOR                        VMFM0680
    5 IER = 0                                                           VMFM0690
      DO 20 I = 1,M                                                     VMFM0700
C                                  COLUMN INDICATOR                     VMFM0710
         DO 15 J = 1,N                                                  VMFM0720
            TEMP = 0.0D0                                                VMFM0730
C                                  VECTOR DOT PRODUCT                   VMFM0740
            DO 10 K = 1,L                                               VMFM0750
               TEMP = TEMP + DBLE(A(K,I))*DBLE(B(K,J))                  VMFM0760
   10       CONTINUE                                                    VMFM0770
            C(I,J) = TEMP                                               VMFM0780
   15    CONTINUE                                                       VMFM0790
   20 CONTINUE                                                          VMFM0800
      GO TO 9005                                                        VMFM0810
 9000 CONTINUE                                                          VMFM0820
      CALL UERTST (IER,6HVMULFM)                                        VMFM0830
 9005 RETURN                                                            VMFM0840
      END                                                               VMFM0850
C   IMSL ROUTINE NAME   - VMULFP                                        VMFP0010
C                                                                       VMFP0020
C-----------------------------------------------------------------------VMFP0030
C                                                                       VMFP0040
C   COMPUTER            - VAX/SINGLE                                    VMFP0050
C                                                                       VMFP0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMFP0070
C                                                                       VMFP0080
C   PURPOSE             - MATRIX MULTIPLICATION OF MATRIX A BY THE      VMFP0090
C                           TRANSPOSE OF MATRIX B (FULL STORAGE MODE)   VMFP0100
C                                                                       VMFP0110
C   USAGE               - CALL VMULFP (A,B,L,M,N,IA,IB,C,IC,IER)        VMFP0120
C                                                                       VMFP0130
C   ARGUMENTS    A      - L BY M MATRIX STORED IN FULL STORAGE MODE.    VMFP0140
C                           (INPUT)                                     VMFP0150
C                B      - N BY M MATRIX STORED IN FULL STORAGE MODE.    VMFP0160
C                           (INPUT)                                     VMFP0170
C                L      - NUMBER OF ROWS IN A AND C. (INPUT)            VMFP0180
C                M      - NUMBER OF COLUMNS IN A AND B. (INPUT)         VMFP0190
C                N      - NUMBER OF ROWS IN MATRIX B AND NUMBER OF      VMFP0200
C                           COLUMNS IN MATRIX C. (INPUT)                VMFP0210
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMFP0220
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFP0230
C                           CALLING PROGRAM. (INPUT)                    VMFP0240
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMFP0250
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFP0260
C                           CALLING PROGRAM. (INPUT)                    VMFP0270
C                C      - L BY N MATRIX CONTAINING THE PRODUCT          VMFP0280
C                           C = A*B-TRANSPOSE. (OUTPUT)                 VMFP0290
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMFP0300
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFP0310
C                           CALLING PROGRAM. (INPUT)                    VMFP0320
C                IER    - ERROR PARAMETER.                              VMFP0330
C                         TERMINAL ERROR                                VMFP0340
C                           IER=129 INDICATES A,B,OR C WAS DIMENSIONED  VMFP0350
C                             INCORRECTLY.                              VMFP0360
C                                                                       VMFP0370
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFP0380
C                       - SINGLE/H36,H48,H60                            VMFP0390
C                                                                       VMFP0400
C   REQD. IMSL ROUTINES - UERTST,UGETIO                                 VMFP0410
C                                                                       VMFP0420
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFP0430
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFP0440
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFP0450
C                                                                       VMFP0460
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFP0470
C                                                                       VMFP0480
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFP0490
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFP0500
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFP0510
C                                                                       VMFP0520
C-----------------------------------------------------------------------VMFP0530
C                                                                       VMFP0540
      SUBROUTINE VMULFP (A,B,L,M,N,IA,IB,C,IC,IER)                      VMFP0550
C                                                                       VMFP0560
      REAL               A(IA,M),B(IB,M),C(IC,N)                        VMFP0570
      DOUBLE PRECISION   TEMP                                           VMFP0580
C                                  FIRST EXECUTABLE STATEMENT           VMFP0590
      IF (IA.GE.L .AND. IB.GE.N .AND. IC.GE.L) GO TO 5                  VMFP0600
C                                  TERMINAL ERROR                       VMFP0610
      IER = 129                                                         VMFP0620
      GO TO 9000                                                        VMFP0630
C                                  ROW INDICATOR                        VMFP0640
    5 IER = 0                                                           VMFP0650
      DO 20 I = 1,L                                                     VMFP0660
C                                  COLUMN INDICATOR                     VMFP0670
         DO 15 J = 1,N                                                  VMFP0680
            TEMP = 0.0D0                                                VMFP0690
C                                  VECTOR DOT PRODUCT                   VMFP0700
            DO 10 K = 1,M                                               VMFP0710
               TEMP = TEMP + DBLE(A(I,K))*DBLE(B(J,K))                  VMFP0720
   10       CONTINUE                                                    VMFP0730
            C(I,J) = TEMP                                               VMFP0740
   15    CONTINUE                                                       VMFP0750
   20 CONTINUE                                                          VMFP0760
      GO TO 9005                                                        VMFP0770
 9000 CONTINUE                                                          VMFP0780
      CALL UERTST (IER,6HVMULFP)                                        VMFP0790
 9005 RETURN                                                            VMFP0800
      END                                                               VMFP0810
C   IMSL ROUTINE NAME   - VMULFQ                                        VMFQ0010
C                                                                       VMFQ0020
C-----------------------------------------------------------------------VMFQ0030
C                                                                       VMFQ0040
C   COMPUTER            - VAX/SINGLE                                    VMFQ0050
C                                                                       VMFQ0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMFQ0070
C                                                                       VMFQ0080
C   PURPOSE             - MATRIX MULTIPLICATION (FULL BY BAND SYMMETRIC VMFQ0090
C                           MATRICES)                                   VMFQ0100
C                                                                       VMFQ0110
C   USAGE               - CALL VMULFQ (A,M,N,IA,B,NC,IB,C,IC)           VMFQ0120
C                                                                       VMFQ0130
C   ARGUMENTS    A      - M BY N MATRIX IN FULL STORAGE MODE. (INPUT)   VMFQ0140
C                M      - NUMBER OF ROWS IN A AND C.  (INPUT)           VMFQ0150
C                N      - NUMBER OF COLUMNS IN A AND C AND ORDER OF B.  VMFQ0160
C                           (INPUT)                                     VMFQ0170
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMFQ0180
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFQ0190
C                           CALLING PROGRAM. (INPUT)                    VMFQ0200
C                B      - N BY N BAND SYMMETRIC MATRIX.  B IS STORED IN VMFQ0210
C                           BAND SYMMETRIC STORAGE MODE AND THEREFORE   VMFQ0220
C                           HAS DIMENSION N BY (NC + 1). (INPUT)        VMFQ0230
C                NC     - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMFQ0240
C                           MATRIX B. (INPUT)                           VMFQ0250
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMFQ0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFQ0270
C                           CALLING PROGRAM. (INPUT)                    VMFQ0280
C                C      - M BY N MATRIX CONTAINING THE PRODUCT C = A*B. VMFQ0290
C                           C IS STORED IN FULL STORAGE MODE.  (OUTPUT) VMFQ0300
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMFQ0310
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFQ0320
C                           CALLING PROGRAM. (INPUT)                    VMFQ0330
C                                                                       VMFQ0340
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFQ0350
C                       - SINGLE/H36,H48,H60                            VMFQ0360
C                                                                       VMFQ0370
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMFQ0380
C                                                                       VMFQ0390
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFQ0400
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFQ0410
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFQ0420
C                                                                       VMFQ0430
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFQ0440
C                                                                       VMFQ0450
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFQ0460
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFQ0470
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFQ0480
C                                                                       VMFQ0490
C-----------------------------------------------------------------------VMFQ0500
C                                                                       VMFQ0510
      SUBROUTINE VMULFQ (A,M,N,IA,B,NC,IB,C,IC)                         VMFQ0520
C                                                                       VMFQ0530
      REAL               A(IA,N),B(IB,1),C(IC,N)                        VMFQ0540
      DOUBLE PRECISION   SUM                                            VMFQ0550
C                                  FIRST EXECUTABLE STATEMENT           VMFQ0560
      NCP1 = NC + 1                                                     VMFQ0570
      NCP2 = NC + 2                                                     VMFQ0580
      DO 35 ICC = 1,N                                                   VMFQ0590
         DO 30 IRA = 1,M                                                VMFQ0600
            SUM = 0.0D0                                                 VMFQ0610
C                                  SET INDICES OF MATRIX B              VMFQ0620
            IRB = ICC                                                   VMFQ0630
            JCB = MAX0(NCP2-ICC,1)                                      VMFQ0640
            LEDZRO = NCP2                                               VMFQ0650
            K = 0                                                       VMFQ0660
            DO 20 JCA = 1,N                                             VMFQ0670
C                                  CHECK FOR ENDING ZERO IN COLUMN OF   VMFQ0680
C                                  MATRIX B AND, IF FOUND, STORE THE    VMFQ0690
C                                  ACCUMULATED PRODUCT IN MATRIX C      VMFQ0700
               IF (JCB .EQ. 0) GO TO 25                                 VMFQ0710
C                                  CHECK FOR LEADING ZERO IN COLUMN OF  VMFQ0720
C                                  MATRIX B AND, IF FOUND, INCREMENT    VMFQ0730
C                                  COLUMN INDEX OF MATRIX A             VMFQ0740
               IF (ICC .LT. LEDZRO) GO TO 5                             VMFQ0750
               LEDZRO = LEDZRO + 1                                      VMFQ0760
               GO TO 20                                                 VMFQ0770
    5          SUM = SUM + DBLE(A(IRA,JCA)) * DBLE(B(IRB,JCB))          VMFQ0780
C                                                                       VMFQ0790
C                                  CHECK FOR INCREMENTING OR DECRE-     VMFQ0800
C                                  MENTING COLUMN INDEX OF MATRIX B     VMFQ0810
               IF (K .NE. 0) GO TO 15                                   VMFQ0820
               IF (JCB .GE. NCP1) GO TO 10                              VMFQ0830
               JCB = JCB + 1                                            VMFQ0840
               GO TO 20                                                 VMFQ0850
   10          K = 1                                                    VMFQ0860
   15          JCB = JCB - 1                                            VMFQ0870
               IRB = IRB + 1                                            VMFQ0880
   20       CONTINUE                                                    VMFQ0890
   25       C(IRA,ICC) = SUM                                            VMFQ0900
   30    CONTINUE                                                       VMFQ0910
   35 CONTINUE                                                          VMFQ0920
      RETURN                                                            VMFQ0930
      END                                                               VMFQ0940
C   IMSL ROUTINE NAME   - VMULFS                                        VMFS0010
C                                                                       VMFS0020
C-----------------------------------------------------------------------VMFS0030
C                                                                       VMFS0040
C   COMPUTER            - VAX/SINGLE                                    VMFS0050
C                                                                       VMFS0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMFS0070
C                                                                       VMFS0080
C   PURPOSE             - MATRIX MULTIPLICATION (FULL BY SYMMETRIC      VMFS0090
C                           MATRICES)                                   VMFS0100
C                                                                       VMFS0110
C   USAGE               - CALL VMULFS (A,B,L,M,IA,C,IC)                 VMFS0120
C                                                                       VMFS0130
C   ARGUMENTS    A      - L BY M MATRIX STORED IN FULL STORAGE MODE.    VMFS0140
C                           (INPUT)                                     VMFS0150
C                B      - M BY M SYMMETRIC MATRIX STORED IN SYMMETRIC   VMFS0160
C                           MODE. (INPUT)                               VMFS0170
C                L      - NUMBER OF ROWS IN A. (INPUT)                  VMFS0180
C                M      - NUMBER OF COLUMNS IN A (SAME AS NUMBER OF     VMFS0190
C                           ROWS IN B). (INPUT)                         VMFS0200
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMFS0210
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFS0220
C                           CALLING PROGRAM. (INPUT)                    VMFS0230
C                C      - L BY M MATRIX CONTAINING THE PRODUCT C = A*B. VMFS0240
C                           (OUTPUT)                                    VMFS0250
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMFS0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMFS0270
C                           CALLING PROGRAM. (INPUT)                    VMFS0280
C                                                                       VMFS0290
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMFS0300
C                       - SINGLE/H36,H48,H60                            VMFS0310
C                                                                       VMFS0320
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMFS0330
C                                                                       VMFS0340
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMFS0350
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMFS0360
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMFS0370
C                                                                       VMFS0380
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMFS0390
C                                                                       VMFS0400
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMFS0410
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMFS0420
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMFS0430
C                                                                       VMFS0440
C-----------------------------------------------------------------------VMFS0450
C                                                                       VMFS0460
      SUBROUTINE VMULFS (A,B,L,M,IA,C,IC)                               VMFS0470
C                                                                       VMFS0480
      REAL               A(IA,M),B(1),C(IC,M)                           VMFS0490
      DOUBLE PRECISION   TEMP                                           VMFS0500
C                                  FIRST EXECUTABLE STATEMENT           VMFS0510
         DO 15 I=1,L                                                    VMFS0520
         LI=1                                                           VMFS0530
            DO 15 J=1,M                                                 VMFS0540
            LS=LI                                                       VMFS0550
            TEMP=0.0D0                                                  VMFS0560
               DO 10 K=1,M                                              VMFS0570
               TEMP=TEMP+DBLE(A(I,K))*DBLE(B(LS))                       VMFS0580
               IF (K .GE. J) GO TO 5                                    VMFS0590
               LS=LS+1                                                  VMFS0600
               GO TO 10                                                 VMFS0610
    5          LS=LS+K                                                  VMFS0620
   10          CONTINUE                                                 VMFS0630
            C(I,J)=TEMP                                                 VMFS0640
   15       LI=LI+J                                                     VMFS0650
      RETURN                                                            VMFS0660
      END                                                               VMFS0670
C   IMSL ROUTINE NAME   - VMULQB                                        VMQB0010
C                                                                       VMQB0020
C-----------------------------------------------------------------------VMQB0030
C                                                                       VMQB0040
C   COMPUTER            - VAX/SINGLE                                    VMQB0050
C                                                                       VMQB0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMQB0070
C                                                                       VMQB0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND SYMMETRIC BY BAND VMQB0090
C                           MATRICES)                                   VMQB0100
C                                                                       VMQB0110
C   USAGE               - CALL VMULQB (A,IA,B,IB,N,C,IC)                VMQB0120
C                                                                       VMQB0130
C   ARGUMENTS    A      - N(1) BY N(1) BAND SYMMETRIC MATRIX WITH N(2)  VMQB0140
C                           UPPER OR LOWER CODIAGONALS. A IS STORED IN  VMQB0150
C                           BAND SYMMETRIC STORAGE MODE AND THEREFORE   VMQB0160
C                           HAS DIMENSION N(1) BY (N(2)+1). (INPUT)     VMQB0170
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMQB0180
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQB0190
C                           CALLING PROGRAM. (INPUT)                    VMQB0200
C                B      - N(1) BY N(1) BAND MATRIX WITH N(3) LOWER      VMQB0210
C                           CODIAGONALS AND N(4) UPPER CODIAGONALS. B   VMQB0220
C                           IS STORED IN BAND STORAGE MODE AND          VMQB0230
C                           THEREFORE HAS DIMENSION N(1) BY             VMQB0240
C                           (N(3)+N(4)+1). (INPUT)                      VMQB0250
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMQB0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQB0270
C                           CALLING PROGRAM. (INPUT)                    VMQB0280
C                N      - INPUT VECTOR OF LENGTH 4 CONTAINING THE       VMQB0290
C                           FOLLOWING INFORMATION:                      VMQB0300
C                         N(1) CONTAINS THE NUMBER OF ROWS AND COLUMNS  VMQB0310
C                           IN MATRICES A,B,AND C.                      VMQB0320
C                         N(2) CONTAINS THE NUMBER OF UPPER OR LOWER    VMQB0330
C                           CODIAGONALS IN MATRIX A.                    VMQB0340
C                         N(3) CONTAINS THE NUMBER OF LOWER CODIAGONALS VMQB0350
C                           IN MATRIX B.                                VMQB0360
C                         N(4) CONTAINS THE NUMBER OF UPPER CODIAGONALS VMQB0370
C                           IN MATRIX B.                                VMQB0380
C                C      - OUTPUT MATRIX CONTAINING THE PRODUCT C = A*B. VMQB0390
C                           C IS A BAND MATRIX WITH M LOWER CODIAGONALS,VMQB0400
C                           WHERE M = MIN(N(1)-1,N(2)+N(3)) AND L       VMQB0410
C                           UPPER CODIAGONALS, WHERE L = MIN(N(1)-1,    VMQB0420
C                           N(2)+N(4)). C IS STORED IN BAND STORAGE     VMQB0430
C                           MODE AND THEREFORE HAS DIMENSION N(1) BY    VMQB0440
C                           (L+M+1). (OUTPUT)                           VMQB0450
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMQB0460
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQB0470
C                           CALLING PROGRAM. (INPUT)                    VMQB0480
C                                                                       VMQB0490
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMQB0500
C                       - SINGLE/H36,H48,H60                            VMQB0510
C                                                                       VMQB0520
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMQB0530
C                                                                       VMQB0540
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMQB0550
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMQB0560
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMQB0570
C                                                                       VMQB0580
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMQB0590
C                                                                       VMQB0600
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMQB0610
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMQB0620
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMQB0630
C                                                                       VMQB0640
C-----------------------------------------------------------------------VMQB0650
C                                                                       VMQB0660
      SUBROUTINE VMULQB (A,IA,B,IB,N,C,IC)                              VMQB0670
C                                                                       VMQB0680
      INTEGER            N(4)                                           VMQB0690
      REAL               A(IA,1),B(IB,1),C(IC,1),ZERO                   VMQB0700
      DOUBLE PRECISION   SUM                                            VMQB0710
      EQUIVALENCE        (NLC,M),(NUC,L)                                VMQB0720
      DATA               ZERO/0.0/                                      VMQB0730
C                                  FIRST EXECUTABLE STATEMENT           VMQB0740
      NN = N(1)                                                         VMQB0750
      NCA = N(2)                                                        VMQB0760
      NLB = N(3)                                                        VMQB0770
      NUB = N(4)                                                        VMQB0780
      NCP1 = NCA+1                                                      VMQB0790
      NCP2 = NCA+2                                                      VMQB0800
      NCB = NLB + 1 + NUB                                               VMQB0810
      M = MIN0(NCA+NLB,NN-1)                                            VMQB0820
      L = MIN0(NCA+NUB,NN-1)                                            VMQB0830
      NCC = M + L + 1                                                   VMQB0840
      NLCP1 = NLC + 1                                                   VMQB0850
      NUBP1 = NUB + 1                                                   VMQB0860
C                                  COMPUTE 1 ROW OF MATRIX C            VMQB0870
      DO 75 IRC = 1,NN                                                  VMQB0880
C                                  INITIALIZE C-COL AND B-VECTOR        VMQB0890
         JCC = 1                                                        VMQB0900
         IVB = 1                                                        VMQB0910
         LDZROC = NLCP1 - IRC                                           VMQB0920
         IF (LDZROC) 15,20,5                                            VMQB0930
C                                  INSERT LEADING ZEROES IN ROW OF C    VMQB0940
C                                  (AS COMPRESSED) AND INCREMENT        VMQB0950
C                                    COL. INDEX                         VMQB0960
    5    DO 10 JCC = 1,LDZROC                                           VMQB0970
            C(IRC,JCC) = ZERO                                           VMQB0980
   10    CONTINUE                                                       VMQB0990
         JCC = LDZROC + 1                                               VMQB1000
         GO TO 20                                                       VMQB1010
C                                  INCREMENT B-VECTOR FOR COMPRESSION   VMQB1020
C                                    OF C                               VMQB1030
   15    IVB = 1 - LDZROC                                               VMQB1040
C                                  COMPUTE NO. OF LEADING ZEROES IN     VMQB1050
C                                    NEXT A-VECTOR                      VMQB1060
   20    LDZROA = MAX0(IRC-NCP1,0)                                      VMQB1070
         NCP2X = NCP2 - IRC                                             VMQB1080
C                                  COMPUTE 1 ELEMENT OF C, USING 1 ROW  VMQB1090
C                                    OF A AND 1 COL OF B                VMQB1100
         DO 65 NVB = IVB,NN                                             VMQB1110
C                                  INITIALIZE C-ELEMENT SUM             VMQB1120
            SUM = 0.0D0                                                 VMQB1130
C                                  SET INDICES FOR FIRST NON-ZERO       VMQB1140
C                                    ELEMENT OF A-VECTOR                VMQB1150
            IRA = IRC                                                   VMQB1160
            JCA = MAX0(NCP2X,1)                                         VMQB1170
C                                  COMPUTE NO. OF LEADING ZEROES IN     VMQB1180
C                                    NEXT B-VECTOR                      VMQB1190
            LDZROB = MAX0(NVB-NUBP1,0)                                  VMQB1200
C                                  SET INDICES FOR FIRST NON-ZERO       VMQB1210
C                                    ELEMENT OF NEXT B-VECTOR           VMQB1220
            IRB = LDZROB + 1                                            VMQB1230
            JCB = MIN0(NVB+NLB,NCB)                                     VMQB1240
C                                  ADJUST A OR B INDICES FOR FIRST SET  VMQB1250
C                                    OF FACTORS (IF UNEQUAL NO. OF      VMQB1260
C                                    LEADING ZEROES)                    VMQB1270
            NLZBA = LDZROB-LDZROA                                       VMQB1280
            IF (NLZBA) 40,45,25                                         VMQB1290
   25       DO 35 INCA = 1,NLZBA                                        VMQB1300
               IF ((IRA.NE.IRC).OR.(JCA.GE.NCP1)) GO TO 30              VMQB1310
               JCA = JCA + 1                                            VMQB1320
               GO TO 35                                                 VMQB1330
   30          JCA = JCA - 1                                            VMQB1340
               IRA = IRA + 1                                            VMQB1350
   35       CONTINUE                                                    VMQB1360
            GO TO 45                                                    VMQB1370
   40       JCB = JCB+NLZBA                                             VMQB1380
            IRB = IRB-NLZBA                                             VMQB1390
C                                  MULTIPLY 2 ELEMENTS (IF END OF       VMQB1400
C                                    VECTORS NOT EXCEEDED) AND          VMQB1410
C                                    ACCUMULATE INTO SUM                VMQB1420
   45       IF ((JCA.LT.1).OR.(IRA.GT.NN).OR.(JCB.LT.1).OR.(IRB.GT.NN)) VMQB1430
     *        GO TO 60                                                  VMQB1440
            SUM = SUM + DBLE(A(IRA,JCA)) * DBLE(B(IRB,JCB))             VMQB1450
C                                  INCREMENT A AND B INDICES            VMQB1460
            IF ((IRA.NE.IRC).OR.(JCA.GE.NCP1)) GO TO 50                 VMQB1470
            JCA = JCA + 1                                               VMQB1480
            GO TO 55                                                    VMQB1490
   50       JCA = JCA - 1                                               VMQB1500
            IRA = IRA + 1                                               VMQB1510
   55       JCB = JCB - 1                                               VMQB1520
            IRB = IRB + 1                                               VMQB1530
            GO TO 45                                                    VMQB1540
C                                  STORE C-ELEMENT                      VMQB1550
   60       C(IRC,JCC) = SUM                                            VMQB1560
C                                  INCREASE C-COL INDEX                 VMQB1570
            JCC = JCC + 1                                               VMQB1580
         IF (JCC .GT. NCC) GO TO 75                                     VMQB1590
   65    CONTINUE                                                       VMQB1600
C                                  INSERT TRAILING ZEROES IN C-ROW IF   VMQB1610
C                                    REQUIRED                           VMQB1620
         DO 70 ICC = JCC,NCC                                            VMQB1630
            C(IRC,ICC) = ZERO                                           VMQB1640
   70    CONTINUE                                                       VMQB1650
   75 CONTINUE                                                          VMQB1660
      RETURN                                                            VMQB1670
      END                                                               VMQB1680
C   IMSL ROUTINE NAME   - VMULQF                                        VMQF0010
C                                                                       VMQF0020
C-----------------------------------------------------------------------VMQF0030
C                                                                       VMQF0040
C   COMPUTER            - VAX/SINGLE                                    VMQF0050
C                                                                       VMQF0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMQF0070
C                                                                       VMQF0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND SYMMETRIC BY      VMQF0090
C                           FULL MATRICES)                              VMQF0100
C                                                                       VMQF0110
C   USAGE               - CALL VMULQF (A,M,NC,IA,B,N,IB,C,IC)           VMQF0120
C                                                                       VMQF0130
C   ARGUMENTS    A      - M BY M BAND SYMMETRIC MATRIX.  A IS STORED IN VMQF0140
C                           BAND SYMMETRIC STORAGE MODE AND THERE-      VMQF0150
C                           FORE HAS DIMENSION M BY (NC + 1). (INPUT)   VMQF0160
C                M      - ORDER OF MATRIX A AND NUMBER OF ROWS IN       VMQF0170
C                           MATRICES B AND C.  (INPUT)                  VMQF0180
C                NC     - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMQF0190
C                           MATRIX A. (INPUT)                           VMQF0200
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMQF0210
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQF0220
C                           CALLING PROGRAM. (INPUT)                    VMQF0230
C                B      - M BY N MATRIX IN FULL STORAGE MODE. (INPUT)   VMQF0240
C                N      - NUMBER OF COLUMNS IN B AND C.  (INPUT)        VMQF0250
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMQF0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQF0270
C                           CALLING PROGRAM. (INPUT)                    VMQF0280
C                C      - M BY N MATRIX CONTAINING THE PRODUCT C = A*B. VMQF0290
C                           C IS STORED IN FULL STORAGE MODE.  (OUTPUT) VMQF0300
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMQF0310
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQF0320
C                           CALLING PROGRAM. (INPUT)                    VMQF0330
C                                                                       VMQF0340
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMQF0350
C                       - SINGLE/H36,H48,H60                            VMQF0360
C                                                                       VMQF0370
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMQF0380
C                                                                       VMQF0390
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMQF0400
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMQF0410
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMQF0420
C                                                                       VMQF0430
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMQF0440
C                                                                       VMQF0450
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMQF0460
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMQF0470
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMQF0480
C                                                                       VMQF0490
C-----------------------------------------------------------------------VMQF0500
C                                                                       VMQF0510
      SUBROUTINE VMULQF (A,M,NC,IA,B,N,IB,C,IC)                         VMQF0520
C                                                                       VMQF0530
      REAL               A(IA,1),B(IB,1),C(IC,1)                        VMQF0540
      DOUBLE PRECISION   SUM                                            VMQF0550
C                                  FIRST EXECUTABLE STATEMENT           VMQF0560
      NCP1 = NC + 1                                                     VMQF0570
      NCP2 = NC + 2                                                     VMQF0580
      DO 35 IRC = 1,M                                                   VMQF0590
         DO 30 JCB = 1,N                                                VMQF0600
            SUM = 0.0D0                                                 VMQF0610
C                                  SET INDICES OF MATRIX A              VMQF0620
            IRA = IRC                                                   VMQF0630
            JCA = MAX0(NCP2-IRC,1)                                      VMQF0640
            LEDZRO = NCP2                                               VMQF0650
            K = 0                                                       VMQF0660
            DO 20 IRB = 1,M                                             VMQF0670
C                                  CHECK FOR ENDING ZERO IN ROW OF      VMQF0680
C                                  MATRIX A AND, IF FOUND, STORE THE    VMQF0690
C                                  ACCUMULATED PRODUCT IN MATRIX C      VMQF0700
               IF ( JCA .EQ. 0) GO TO 25                                VMQF0710
C                                  CHECK FOR LEADING ZERO IN ROW OF     VMQF0720
C                                  MATRIX A AND, IF FOUND, INCREMENT    VMQF0730
C                                  ROW INDEX OF MATRIX B                VMQF0740
               IF (IRC .LT. LEDZRO) GO TO 5                             VMQF0750
               LEDZRO = LEDZRO + 1                                      VMQF0760
               GO TO 20                                                 VMQF0770
    5          SUM = SUM + DBLE(A(IRA,JCA)) * DBLE(B(IRB,JCB))          VMQF0780
C                                                                       VMQF0790
C                                  CHECK FOR INCREMENTING OR DECREMENTINVMQF0800
C                                  COLUMN INDEX OF MATRIX A             VMQF0810
               IF (K .NE. 0) GO TO 15                                   VMQF0820
               IF (JCA .GE. NCP1) GO TO 10                              VMQF0830
               JCA = JCA + 1                                            VMQF0840
               GO TO 20                                                 VMQF0850
   10          K = 1                                                    VMQF0860
   15          JCA = JCA - 1                                            VMQF0870
               IRA = IRA + 1                                            VMQF0880
   20       CONTINUE                                                    VMQF0890
   25       C(IRC,JCB) = SUM                                            VMQF0900
   30    CONTINUE                                                       VMQF0910
   35 CONTINUE                                                          VMQF0920
      RETURN                                                            VMQF0930
      END                                                               VMQF0940
C   IMSL ROUTINE NAME   - VMULQQ                                        VMQQ0010
C                                                                       VMQQ0020
C-----------------------------------------------------------------------VMQQ0030
C                                                                       VMQQ0040
C   COMPUTER            - VAX/SINGLE                                    VMQQ0050
C                                                                       VMQQ0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMQQ0070
C                                                                       VMQQ0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND SYMMETRIC STORAGE VMQQ0090
C                           MODE)                                       VMQQ0100
C                                                                       VMQQ0110
C   USAGE               - CALL VMULQQ (A,N,NCA,IA,B,NCB,IB,C,IC)        VMQQ0120
C                                                                       VMQQ0130
C   ARGUMENTS    A      - N BY N BAND SYMMETRIC MATRIX STORED IN BAND   VMQQ0140
C                           SYMMETRIC STORAGE MODE.  MATRIX A IS        VMQQ0150
C                           DIMENSIONED N BY (NCA + 1).  (INPUT)        VMQQ0160
C                N      - ORDER OF MATRICES A, B, AND C.  (INPUT)       VMQQ0170
C                NCA    - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMQQ0180
C                           MATRIX A.  (INPUT)                          VMQQ0190
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMQQ0200
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQQ0210
C                           CALLING PROGRAM. (INPUT)                    VMQQ0220
C                B      - N BY N BAND SYMMETRIC MATRIX STORED IN BAND   VMQQ0230
C                           SYMMETRIC STORAGE MODE.  MATRIX B IS        VMQQ0240
C                           DIMENSIONED N BY (NCB + 1).  (INPUT)        VMQQ0250
C                NCB    - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMQQ0260
C                           MATRIX B.  (INPUT)                          VMQQ0270
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMQQ0280
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQQ0290
C                           CALLING PROGRAM. (INPUT)                    VMQQ0300
C                C      - OUTPUT MATRIX CONTAINING THE PRODUCT C = A*B. VMQQ0310
C                           C IS A BAND MATRIX WITH M UPPER AND M LOWER VMQQ0320
C                           CO-DIAGONALS, WHERE M = MIN(N-1,NCA+NCB).   VMQQ0330
C                C      - OUTPUT MATRIX CONTAINING THE PRODUCT C = A*B. VMQQ0310
C                           C IS A BAND MATRIX WITH M UPPER AND M LOWER VMQQ0320
C                           CO-DIAGONALS, WHERE M = MIN(N-1,NCA+NCB).   VMQQ0330
C                           C IS STORED IN BAND STORAGE MODE AND        VMQQ0340
C                           THEREFORE HAS DIMENSION N BY                VMQQ0350
C                           MIN(2*N-1,2*(NCA+NCB)+1).                   VMQQ0360
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMQQ0370
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQQ0380
C                           CALLING PROGRAM. (INPUT)                    VMQQ0390
C                                                                       VMQQ0400
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMQQ0410
C                       - SINGLE/H36,H48,H60                            VMQQ0420
C                                                                       VMQQ0430
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMQQ0440
C                                                                       VMQQ0450
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMQQ0460
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMQQ0470
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMQQ0480
C                                                                       VMQQ0490
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMQQ0500
C                                                                       VMQQ0510
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMQQ0520
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMQQ0530
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMQQ0540
C                                                                       VMQQ0550
C-----------------------------------------------------------------------VMQQ0560
C                                                                       VMQQ0570
      SUBROUTINE VMULQQ (A,N,NCA,IA,B,NCB,IB,C,IC)                      VMQQ0580
C                                                                       VMQQ0590
      REAL               A(IA,1),B(IB,1),C(IC,1)                        VMQQ0600
      DOUBLE PRECISION   SUM                                            VMQQ0610
C                                  FIRST EXECUTABLE STATEMENT           VMQQ0620
      NCAP1 = NCA + 1                                                   VMQQ0630
      NCAP2 = NCA + 2                                                   VMQQ0640
      NCBP1 = NCB + 1                                                   VMQQ0650
      NCBP2 = NCB + 2                                                   VMQQ0660
      NM1 = N+N-1                                                       VMQQ0670
      NCC = 2*(NCA+NCB) +1                                              VMQQ0680
      NCC = MIN0(NCC,NM1)                                               VMQQ0690
      ND = (NCC+3)/2                                                    VMQQ0700
      DO 65 I = 1,N                                                     VMQQ0710
         DO 2 K1 = 1,NCC                                                VMQQ0720
            C(I,K1) = 0.                                                VMQQ0730
    2    CONTINUE                                                       VMQQ0740
         NDMI = ND-I                                                    VMQQ0750
         M = MAX0(1,NDMI)                                               VMQQ0760
         DO 60 J = 1,N                                                  VMQQ0770
            JPTA = 0                                                    VMQQ0780
            JPTB = 0                                                    VMQQ0790
            LEDZRA = NCAP2                                              VMQQ0800
            LEDZRB = NCBP2                                              VMQQ0810
C                                  SET INDICES OF A AND B               VMQQ0820
            IRA = I                                                     VMQQ0830
            IRB = J                                                     VMQQ0840
            JCA = MAX0(NCAP2-I,1)                                       VMQQ0850
            JCB = MAX0(NCBP2-J,1)                                       VMQQ0860
            SUM = 0.D0                                                  VMQQ0870
C                                  MULTIPLY ACROSS ROW OF A AND DOWN    VMQQ0880
C                                  COLUMN OF B                          VMQQ0890
            DO 50 K = 1,N                                               VMQQ0900
               NB0 = 0                                                  VMQQ0910
C                                  TEST FOR BEGINNING AND ENDING ZEROES VMQQ0920
C                                  IN ROW OF A AND COLUMN OF B          VMQQ0930
               IF (JCA .EQ. 0) GO TO 52                                 VMQQ0940
               IF (I .LT. LEDZRA) GO TO 5                               VMQQ0950
               LEDZRA = LEDZRA + 1                                      VMQQ0960
               IF (J .LT. LEDZRB) GO TO 35                              VMQQ0970
               LEDZRB = LEDZRB + 1                                      VMQQ0980
               GO TO 50                                                 VMQQ0990
    5          IF (JCB .EQ. 0) GO TO 52                                 VMQQ1000
               IF (J .LT. LEDZRB) GO TO 10                              VMQQ1010
               LEDZRB = LEDZRB + 1                                      VMQQ1020
               NB0 = 1                                                  VMQQ1030
               GO TO 15                                                 VMQQ1040
   10          SUM = SUM + DBLE(A(IRA,JCA)) * DBLE(B(IRB,JCB))          VMQQ1050
C                                  INCREMENT OR DECREMENT INDICES OF A  VMQQ1060
   15          IF (JPTA .NE. 0) GO TO 20                                VMQQ1070
               IF (JCA .LT. NCAP1) GO TO 25                             VMQQ1080
               JPTA = 1                                                 VMQQ1090
   20          JCA = JCA - 1                                            VMQQ1100
               IRA = IRA + 1                                            VMQQ1110
               GO TO 30                                                 VMQQ1120
   25          JCA = JCA + 1                                            VMQQ1130
   30          IF (NB0 .EQ. 1) GO TO 50                                 VMQQ1140
C                                  INCREMENT OR DECREMENT INDICES OF B  VMQQ1150
   35          IF (JPTB .NE. 0) GO TO 40                                VMQQ1160
               IF (JCB .LT. NCBP1) GO TO 45                             VMQQ1170
               JPTB = 1                                                 VMQQ1180
   40          JCB = JCB - 1                                            VMQQ1190
               JCB = MAX0(JCB,0)                                        VMQQ1200
               IRB = IRB + 1                                            VMQQ1210
               GO TO 50                                                 VMQQ1220
   45          JCB = JCB + 1                                            VMQQ1230
   50       CONTINUE                                                    VMQQ1240
   52       IF (NDMI .GE. 1) GO TO 55                                   VMQQ1250
            NDMI = NDMI + 1                                             VMQQ1260
            GO TO 60                                                    VMQQ1270
   55       C(I,M) = SUM                                                VMQQ1280
            M = M + 1                                                   VMQQ1290
            IF (M .GT. NCC) GO TO 65                                    VMQQ1300
   60    CONTINUE                                                       VMQQ1310
   65 CONTINUE                                                          VMQQ1320
      RETURN                                                            VMQQ1330
      END                                                               VMQQ1340
C   IMSL ROUTINE NAME   - VMULQS                                        VMQS0010
C                                                                       VMQS0020
C-----------------------------------------------------------------------VMQS0030
C                                                                       VMQS0040
C   COMPUTER            - VAX/SINGLE                                    VMQS0050
C                                                                       VMQS0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMQS0070
C                                                                       VMQS0080
C   PURPOSE             - MATRIX MULTIPLICATION (BAND SYMMETRIC BY      VMQS0090
C                           SYMMETRIC MATRICES)                         VMQS0100
C                                                                       VMQS0110
C   USAGE               - CALL VMULQS (A,N,NC,IA,B,C,IC)                VMQS0120
C                                                                       VMQS0130
C   ARGUMENTS    A      - N BY N BAND SYMMETRIC MATRIX STORED IN BAND   VMQS0140
C                           SYMMETRIC STORAGE MODE.  MATRIX A IS        VMQS0150
C                           DIMENSIONED N BY (NC + 1).  (INPUT)         VMQS0160
C                N      - ORDER OF MATRICES A,B, AND C.  (INPUT)        VMQS0170
C                NC     - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMQS0180
C                           MATRIX A.  (INPUT)                          VMQS0190
C                IA     - ROW DIMENSION OF MATRIX A EXACTLY AS          VMQS0200
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQS0210
C                           CALLING PROGRAM. (INPUT)                    VMQS0220
C                B      - INPUT VECTOR OF LENGTH N(N+1)/2 CONTAINING AN VMQS0230
C                           N BY N SYMMETRIC MATRIX STORED IN SYMMETRIC VMQS0240
C                           STORAGE MODE.                               VMQS0250
C                C      - N X N MATRIX CONTAINING THE PRODUCT C = A*B   VMQS0260
C                           STORED IN FULL STORAGE MODE.  (OUTPUT)      VMQS0270
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMQS0280
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMQS0290
C                           CALLING PROGRAM. (INPUT)                    VMQS0300
C                                                                       VMQS0310
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMQS0320
C                       - SINGLE/H36,H48,H60                            VMQS0330
C                                                                       VMQS0340
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMQS0350
C                                                                       VMQS0360
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMQS0370
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMQS0380
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMQS0390
C                                                                       VMQS0400
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMQS0410
C                                                                       VMQS0420
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMQS0430
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMQS0440
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMQS0450
C                                                                       VMQS0460
C-----------------------------------------------------------------------VMQS0470
C                                                                       VMQS0480
      SUBROUTINE VMULQS (A,N,NC,IA,B,C,IC)                              VMQS0490
C                                                                       VMQS0500
      REAL               A(IA,1),B(1),C(IC,N)                           VMQS0510
      DOUBLE PRECISION   SUM                                            VMQS0520
C                                  FIRST EXECUTABLE STATEMENT           VMQS0530
      NCP1 = NC + 1                                                     VMQS0540
      NCP2 = NC + 2                                                     VMQS0550
      IRB = 1                                                           VMQS0560
      DO 40 J = 1,N                                                     VMQS0570
         DO 35 I = 1,N                                                  VMQS0580
C                                  SET INDEX OF VECTOR B AND INDICES    VMQS0590
C                                  OF MATRIX A                          VMQS0600
            INDXB = IRB                                                 VMQS0610
            IRA = I                                                     VMQS0620
            JCA = MAX0(NCP2-I,1)                                        VMQS0630
            JPTA = 0                                                    VMQS0640
            KAD = 1                                                     VMQS0650
            LEDZRO = NCP2                                               VMQS0660
            SUM = 0.0D0                                                 VMQS0670
            DO 25 K = 1,N                                               VMQS0680
C                                  TEST FOR ENDING ZERO IN COLUMN OF    VMQS0690
C                                  MATRIX A, AND IF FOUND, STORE PRODUCTVMQS0700
               IF (JCA .EQ. 0) GO TO 30                                 VMQS0710
C                                  TEST FOR LEADING ZERO IN COLUMN OF   VMQS0720
C                                  MATRIX A, AND IF FOUND, INCREMENT    VMQS0730
C                                  INDEX OF VECTOR B                    VMQS0740
               IF (K .GE. J) KAD = K                                    VMQS0750
               IF (I .LT. LEDZRO) GO TO 5                               VMQS0760
               LEDZRO = LEDZRO + 1                                      VMQS0770
               GO TO 20                                                 VMQS0780
    5          SUM = SUM + DBLE(A(IRA,JCA)) * DBLE(B(INDXB))            VMQS0790
C                                  INCREMENT OR DECREMENT COLUMN INDEX  VMQS0800
C                                  OF MATRIX A AND INCREMENT ROW INDEX  VMQS0810
C                                  OF MATRIX A                          VMQS0820
               IF (JPTA .NE. 0) GO TO 10                                VMQS0830
               IF (JCA .LT.NCP1) GO TO 15                               VMQS0840
               JPTA = 1                                                 VMQS0850
   10          JCA = JCA - 1                                            VMQS0860
               IRA = IRA + 1                                            VMQS0870
               GO TO 20                                                 VMQS0880
   15          JCA = JCA + 1                                            VMQS0890
C                                  INCREMENT INDEX OF VECTOR B          VMQS0900
   20          INDXB = INDXB + KAD                                      VMQS0910
   25       CONTINUE                                                    VMQS0920
   30       C(I,J) = SUM                                                VMQS0930
   35    CONTINUE                                                       VMQS0940
C                                  SET INDEX OF VECTOR B TO FIRST       VMQS0950
C                                  ELEMENT OF NEW ROW                   VMQS0960
         IRB = IRB + J                                                  VMQS0970
   40 CONTINUE                                                          VMQS0980
      RETURN                                                            VMQS0990
      END                                                               VMQS1000
C   IMSL ROUTINE NAME   - VMULSB                                        VMSB0010
C                                                                       VMSB0020
C-----------------------------------------------------------------------VMSB0030
C                                                                       VMSB0040
C   COMPUTER            - VAX/SINGLE                                    VMSB0050
C                                                                       VMSB0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMSB0070
C                                                                       VMSB0080
C   PURPOSE             - MATRIX MULTIPLICATION (SYMMETRIC BY BAND      VMSB0090
C                            MATRICES)                                  VMSB0100
C                                                                       VMSB0110
C   USAGE               - CALL VMULSB (A,B,IB,N,C,IC)                   VMSB0120
C                                                                       VMSB0130
C   ARGUMENTS    A      - N(1) BY N(1) SYMMETRIC MATRIX . A IS STORED   VMSB0140
C                           IN SYMMETRIC STORAGE MODE AND THEREFORE IS  VMSB0150
C                           A VECTOR OF LENGTH N(1)*(N(1)+1)/2. (INPUT) VMSB0160
C                B      - N(1) BY N(1) BAND MATRIX WITH N(2) LOWER      VMSB0170
C                           CODIAGONALS AND N(3) UPPER CODIAGONALS. B   VMSB0180
C                           IS STORED IN BAND STORAGE MODE AND          VMSB0190
C                           THEREFORE HAS DIMENSION N(1) BY             VMSB0200
C                           (N(2)+N(3)+1). (INPUT)                      VMSB0210
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMSB0220
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSB0230
C                           CALLING PROGRAM. (INPUT)                    VMSB0240
C                N      - INPUT VECTOR OF LENGTH 3 CONTAINING THE       VMSB0250
C                           FOLLOWING INFORMATION:                      VMSB0260
C                         N(1) CONTAINS THE NUMBER OF ROWS AND COLUMNS  VMSB0270
C                           IN MATRICES A,B,AND C.                      VMSB0280
C                         N(2) CONTAINS THE NUMBER OF LOWER CODIAGONALS VMSB0290
C                           IN MATRIX B.                                VMSB0300
C                         N(3) CONTAINS THE NUMBER OF UPPER CODIAGONALS VMSB0310
C                           IN MATRIX B.                                VMSB0320
C                C      - N(1) BY N(1) MATRIX CONTAINING THE PRODUCT    VMSB0330
C                           C=A*B. C IS STORED IN FULL STORAGE MODE.    VMSB0340
C                           (OUTPUT)                                    VMSB0350
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMSB0360
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSB0370
C                           CALLING PROGRAM. (INPUT)                    VMSB0380
C                                                                       VMSB0390
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMSB0400
C                       - SINGLE/H36,H48,H60                            VMSB0410
C                                                                       VMSB0420
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMSB0430
C                                                                       VMSB0440
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMSB0450
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMSB0460
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMSB0470
C                                                                       VMSB0480
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMSB0490
C                                                                       VMSB0500
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMSB0510
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMSB0520
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMSB0530
C                                                                       VMSB0540
C-----------------------------------------------------------------------VMSB0550
C                                                                       VMSB0560
      SUBROUTINE VMULSB (A,B,IB,N,C,IC)                                 VMSB0570
C                                                                       VMSB0580
      INTEGER            N(3)                                           VMSB0590
      REAL               A(1),B(IB,1),C(IC,1)                           VMSB0600
      DOUBLE PRECISION   SUM                                            VMSB0610
C                                  FIRST EXECUTABLE STATEMENT           VMSB0620
      NN = N(1)                                                         VMSB0630
      N3P1 = N(3)+1                                                     VMSB0640
      NC = N3P1+N(2)                                                    VMSB0650
      NMN2 = NN-N(2)                                                    VMSB0660
      IBEG1 = 1                                                         VMSB0670
      DO 30 I = 1,NN                                                    VMSB0680
         KBEG = 1                                                       VMSB0690
         JZ = 1                                                         VMSB0700
         DO 25 J = 1,NN                                                 VMSB0710
            IF (J .EQ. 1) IBEGP = IBEG1                                 VMSB0720
            SUM = 0.0D0                                                 VMSB0730
            KEND = N(2)+J                                               VMSB0740
            IF (J .LE. N3P1) GO TO 15                                   VMSB0750
            IF (JZ .GE. I) GO TO 5                                      VMSB0760
            IBEGP = IBEGP+1                                             VMSB0770
            GO TO 10                                                    VMSB0780
    5       IBEGP = IBEGP+JZ                                            VMSB0790
   10       JZ = JZ+1                                                   VMSB0800
            KEND = NC                                                   VMSB0810
   15       IBEG = IBEGP                                                VMSB0820
            LPD = JZ                                                    VMSB0830
            JJ = 1                                                      VMSB0840
            KENDP1 = KEND+KBEG                                          VMSB0850
            ICNT = JZ-1                                                 VMSB0860
            DO 20 KK = KBEG,KEND                                        VMSB0870
               ICNT = ICNT+1                                            VMSB0880
               K = KENDP1-KK                                            VMSB0890
               SUM = SUM+DBLE(A(IBEG))*DBLE(B(LPD,K))                   VMSB0900
               LPD = LPD+1                                              VMSB0910
               IF (ICNT .GE. I) JJ = ICNT                               VMSB0920
               IBEG = IBEG+JJ                                           VMSB0930
   20       CONTINUE                                                    VMSB0940
            C(I,J) = SUM                                                VMSB0950
            IF (J .GE. NMN2) KBEG = KBEG+1                              VMSB0960
   25    CONTINUE                                                       VMSB0970
         IBEG1 = IBEG1+I                                                VMSB0980
   30 CONTINUE                                                          VMSB0990
      RETURN                                                            VMSB1000
      END                                                               VMSB1010
C   IMSL ROUTINE NAME   - VMULSF                                        VMSF0010
C                                                                       VMSF0020
C-----------------------------------------------------------------------VMSF0030
C                                                                       VMSF0040
C   COMPUTER            - VAX/SINGLE                                    VMSF0050
C                                                                       VMSF0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMSF0070
C                                                                       VMSF0080
C   PURPOSE             - MATRIX MULTIPLICATION (SYMMETRIC BY FULL      VMSF0090
C                           MATRICES)                                   VMSF0100
C                                                                       VMSF0110
C   USAGE               - CALL VMULSF (A,N,B,M,IB,C,IC)                 VMSF0120
C                                                                       VMSF0130
C   ARGUMENTS    A      - INPUT VECTOR OF LENGTH N(N+1)/2 CONTAINING AN VMSF0140
C                           N BY N SYMMETRIC MATRIX STORED IN SYMMETRIC VMSF0150
C                           STORAGE MODE.                               VMSF0160
C                N      - ORDER OF A AND NUMBER OF ROWS IN B.  (INPUT)  VMSF0170
C                B      - N BY M MATRIX IN FULL STORAGE MODE.  (INPUT)  VMSF0180
C                M      - NUMBER OF COLUMNS IN B AND NUMBER OF COLUMNS  VMSF0190
C                           IN OUTPUT MATRIX C.  (INPUT)                VMSF0200
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMSF0210
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSF0220
C                           CALLING PROGRAM. (INPUT)                    VMSF0230
C                C      - N BY M MATRIX IN FULL STORAGE MODE CONTAINING VMSF0240
C                           THE PRODUCT C = A*B.  (OUTPUT)              VMSF0250
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMSF0260
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSF0270
C                           CALLING PROGRAM. (INPUT)                    VMSF0280
C                                                                       VMSF0290
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMSF0300
C                       - SINGLE/H36,H48,H60                            VMSF0310
C                                                                       VMSF0320
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMSF0330
C                                                                       VMSF0340
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMSF0350
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMSF0360
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMSF0370
C                                                                       VMSF0380
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMSF0390
C                                                                       VMSF0400
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMSF0410
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMSF0420
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMSF0430
C                                                                       VMSF0440
C-----------------------------------------------------------------------VMSF0450
C                                                                       VMSF0460
      SUBROUTINE VMULSF (A,N,B,M,IB,C,IC)                               VMSF0470
C                                                                       VMSF0480
      REAL               A(1),B(IB,M),C(IC,M)                           VMSF0490
      DOUBLE PRECISION   SUM                                            VMSF0500
C                                  FIRST EXECUTABLE STATEMENT           VMSF0510
      DO 15 J = 1,M                                                     VMSF0520
         IBEG = 1                                                       VMSF0530
         DO 10 I = 1,N                                                  VMSF0540
            L = IBEG                                                    VMSF0550
            KK = 1                                                      VMSF0560
            SUM = 0.0D0                                                 VMSF0570
            DO 5 K = 1,N                                                VMSF0580
               SUM = SUM + DBLE(A(L))*DBLE(B(K,J))                      VMSF0590
               IF (K .GE. I) KK = K                                     VMSF0600
               L = L + KK                                               VMSF0610
    5       CONTINUE                                                    VMSF0620
            C(I,J) = SUM                                                VMSF0630
            IBEG = IBEG + I                                             VMSF0640
   10    CONTINUE                                                       VMSF0650
   15 CONTINUE                                                          VMSF0660
      RETURN                                                            VMSF0670
      END                                                               VMSF0680
C   IMSL ROUTINE NAME   - VMULSQ                                        VMSQ0010
C                                                                       VMSQ0020
C-----------------------------------------------------------------------VMSQ0030
C                                                                       VMSQ0040
C   COMPUTER            - VAX/SINGLE                                    VMSQ0050
C                                                                       VMSQ0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMSQ0070
C                                                                       VMSQ0080
C   PURPOSE             - MATRIX MULTIPLICATION (SYMMETRIC BY BAND      VMSQ0090
C                           SYMMETRIC MATRICES)                         VMSQ0100
C                                                                       VMSQ0110
C   USAGE               - CALL VMULSQ (A,N,B,NC,IB,C,IC)                VMSQ0120
C                                                                       VMSQ0130
C   ARGUMENTS    A      - INPUT VECTOR OF LENGTH N(N+1)/2 CONTAINING AN VMSQ0140
C                           N BY N SYMMETRIC MATRIX STORED IN SYMMETRIC VMSQ0150
C                           STORAGE MODE.                               VMSQ0160
C                N      - ORDER OF MATRICES A, B, AND C.  (INPUT)       VMSQ0170
C                B      - N BY N BAND SYMMETRIC MATRIX STORED IN BAND   VMSQ0180
C                           SYMMETRIC STORAGE MODE.  MATRIX B IS        VMSQ0190
C                           DIMENSIONED N BY (NC + 1).  (INPUT)         VMSQ0200
C                NC     - NUMBER OF UPPER OR LOWER CO-DIAGONALS IN      VMSQ0210
C                           MATRIX B.  (INPUT)                          VMSQ0220
C                IB     - ROW DIMENSION OF MATRIX B EXACTLY AS          VMSQ0230
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSQ0240
C                           CALLING PROGRAM. (INPUT)                    VMSQ0250
C                C      - N X N MATRIX CONTAINING THE PRODUCT C = A*B   VMSQ0260
C                           STORED IN FULL STORAGE MODE.  (OUTPUT)      VMSQ0270
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMSQ0280
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSQ0290
C                           CALLING PROGRAM. (INPUT)                    VMSQ0300
C                                                                       VMSQ0310
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMSQ0320
C                       - SINGLE/H36,H48,H60                            VMSQ0330
C                                                                       VMSQ0340
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMSQ0350
C                                                                       VMSQ0360
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMSQ0370
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMSQ0380
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMSQ0390
C                                                                       VMSQ0400
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMSQ0410
C                                                                       VMSQ0420
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMSQ0430
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMSQ0440
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMSQ0450
C                                                                       VMSQ0460
C-----------------------------------------------------------------------VMSQ0470
C                                                                       VMSQ0480
      SUBROUTINE VMULSQ (A,N,B,NC,IB,C,IC)                              VMSQ0490
C                                                                       VMSQ0500
      REAL               A(1),B(IB,1),C(IC,N)                           VMSQ0510
      DOUBLE PRECISION   SUM                                            VMSQ0520
C                                  FIRST EXECUTABLE STATEMENT           VMSQ0530
      NCP1 = NC + 1                                                     VMSQ0540
      NCP2 = NC + 2                                                     VMSQ0550
      IRA = 1                                                           VMSQ0560
      DO 40 J = 1,N                                                     VMSQ0570
         DO 35 I = 1,N                                                  VMSQ0580
C                                  SET INDEX OF VECTOR A AND INDICES    VMSQ0590
C                                  OF MATRIX B                          VMSQ0600
            INDXA = IRA                                                 VMSQ0610
            IRB = I                                                     VMSQ0620
            JCB = MAX0(NCP2-I,1)                                        VMSQ0630
            JPTB = 0                                                    VMSQ0640
            KAD = 1                                                     VMSQ0650
            LEDZRO = NCP2                                               VMSQ0660
            SUM = 0.0D0                                                 VMSQ0670
            DO 25 K = 1,N                                               VMSQ0680
C                                  TEST FOR ENDING ZERO IN COLUMN OF    VMSQ0690
C                                  MATRIX B, AND IF FOUND, STORE PRODUCTVMSQ0700
               IF (JCB .EQ. 0) GO TO 30                                 VMSQ0710
C                                  TEST FOR LEADING ZERO IN COLUMN OF   VMSQ0720
C                                  MATRIX B, AND IF FOUND, INCREMENT    VMSQ0730
C                                  INDEX OF VECTOR A                    VMSQ0740
               IF (K .GE. J) KAD = K                                    VMSQ0750
               IF (I .LT. LEDZRO) GO TO 5                               VMSQ0760
               LEDZRO = LEDZRO + 1                                      VMSQ0770
               GO TO 20                                                 VMSQ0780
    5          SUM = SUM + DBLE(A(INDXA)) * DBLE(B(IRB,JCB))            VMSQ0790
C                                  INCREMENT OR DECREMENT COLUMN INDEX  VMSQ0800
C                                  OF MATRIX B AND INCREMENT ROW INDEX  VMSQ0810
C                                  OF MATRIX B                          VMSQ0820
               IF (JPTB .NE. 0) GO TO 10                                VMSQ0830
               IF (JCB .LT. NCP1) GO TO 15                              VMSQ0840
               JPTB = 1                                                 VMSQ0850
   10          JCB = JCB - 1                                            VMSQ0860
               IRB = IRB + 1                                            VMSQ0870
               GO TO 20                                                 VMSQ0880
   15          JCB = JCB + 1                                            VMSQ0890
C                                  INCREMENT INDEX OF VECTOR A          VMSQ0900
   20          INDXA = INDXA + KAD                                      VMSQ0910
   25       CONTINUE                                                    VMSQ0920
   30       C(J,I) = SUM                                                VMSQ0930
   35    CONTINUE                                                       VMSQ0940
C                                  SET INDEX OF VECTOR A TO FIRST       VMSQ0950
C                                  ELEMENT OF NEW ROW                   VMSQ0960
         IRA = IRA + J                                                  VMSQ0970
   40 CONTINUE                                                          VMSQ0980
      RETURN                                                            VMSQ0990
      END                                                               VMSQ1000
C   IMSL ROUTINE NAME   - VMULSS                                        VMSS0010
C                                                                       VMSS0020
C-----------------------------------------------------------------------VMSS0030
C                                                                       VMSS0040
C   COMPUTER            - VAX/SINGLE                                    VMSS0050
C                                                                       VMSS0060
C   LATEST REVISION     - JANUARY 1, 1978                               VMSS0070
C                                                                       VMSS0080
C   PURPOSE             - MATRIX MULTIPLICATION (SYMMETRIC STORAGE      VMSS0090
C                           MODE)                                       VMSS0100
C                                                                       VMSS0110
C   USAGE               - CALL VMULSS (A,B,N,C,IC)                      VMSS0120
C                                                                       VMSS0130
C   ARGUMENTS    A      - INPUT VECTOR OF LENGTH N(N+1)/2 CONTAINING AN VMSS0140
C                           N BY N SYMMETRIC MATRIX STORED IN SYMMETRIC VMSS0150
C                           STORAGE MODE.                               VMSS0160
C                B      - INPUT VECTOR OF LENGTH N(N+1)/2 CONTAINING AN VMSS0170
C                           N BY N SYMMETRIC MATRIX STORED IN SYMMETRIC VMSS0180
C                           STORAGE MODE.                               VMSS0190
C                N      - ORDER OF MATRICES A, B, AND C. (INPUT)        VMSS0200
C                C      - N BY N MATRIX IN FULL STORAGE MODE CONTAINING VMSS0210
C                           THE PRODUCT C = A*B. (OUTPUT)               VMSS0220
C                IC     - ROW DIMENSION OF MATRIX C EXACTLY AS          VMSS0230
C                           SPECIFIED IN THE DIMENSION STATEMENT IN THE VMSS0240
C                           CALLING PROGRAM. (INPUT)                    VMSS0250
C                                                                       VMSS0260
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                         VMSS0270
C                       - SINGLE/H36,H48,H60                            VMSS0280
C                                                                       VMSS0290
C   REQD. IMSL ROUTINES - NONE REQUIRED                                 VMSS0300
C                                                                       VMSS0310
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND           VMSS0320
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL      VMSS0330
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP  VMSS0340
C                                                                       VMSS0350
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.       VMSS0360
C                                                                       VMSS0370
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN VMSS0380
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,    VMSS0390
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.        VMSS0400
C                                                                       VMSS0410
C-----------------------------------------------------------------------VMSS0420
C                                                                       VMSS0430
      SUBROUTINE VMULSS (A,B,N,C,IC)                                    VMSS0440
C                                                                       VMSS0450
      REAL               A(1),B(1),C(IC,N)                              VMSS0460
      DOUBLE PRECISION   TEMP                                           VMSS0470
C                                  FIRST EXECUTABLE STATEMENT           VMSS0480
      LI=1                                                              VMSS0490
         DO 30 IR=1,N                                                   VMSS0500
         LJ=1                                                           VMSS0510
            DO 25 JC = 1,N                                              VMSS0520
            I=LI                                                        VMSS0530
            J=LJ                                                        VMSS0540
            TEMP=0.0D0                                                  VMSS0550
               DO 20 K=1,N                                              VMSS0560
               TEMP=DBLE(A(I))*DBLE(B(J))+TEMP                          VMSS0570
               IF (K .LT. IR) GO TO 5                                   VMSS0580
               I=I+K                                                    VMSS0590
               GO TO 10                                                 VMSS0600
    5          I=I+1                                                    VMSS0610
   10          IF (K .LT. JC) GO TO 15                                  VMSS0620
               J=J+K                                                    VMSS0630
               GO TO 20                                                 VMSS0640
   15          J=J+1                                                    VMSS0650
   20          CONTINUE                                                 VMSS0660
            C(IR,JC) = TEMP                                             VMSS0670
            LJ = LJ+JC                                                  VMSS0680
   25       CONTINUE                                                    VMSS0690
         LI=LI+IR                                                       VMSS0700
   30    CONTINUE                                                       VMSS0710
      RETURN                                                            VMSS0720
      END                                                               VMSS0730
