CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  DISPINV.FOR                                                       C
C                                                                    C
C  by: Paul Segall and Laurie Erickson                               C
C  revised: February 9, 1988                                         C
C  revised: July 15, 1988; added studentized residuals               C
C  revised: Sept 1, 1988; changed to new gstalc                      C
C  revised: Jan 5, 1989; shrinks G to exclude stations with no data  C
C  revised: Dec 13, 1989; include triangulation data                 C
C  revised: March 1990; allows interactive checking of rank          C
C  revised: May 1990; added GPS and VLBI data                        C
C  revised: July 1990; added Azimuth data                            C
C                                                                    C
C        Program DISPINV.FOR solves for benchmark displacements      C
C  (velocities) given mark-to-mark baseline length changes (rates of C
C  change).  For general discussion of method see P.Segall and M.V.  C
C  Matthews, 1988; Displacement Calculations From Geodetic Data and  C
C  the Testing of Geophysical Deformation Models                     C
C        The general problem is:                                     C
C                                  d = G u + epsilon                 C
C                                                                    C
C  numst   = number of stations                                      C
C  numdat  = number of data                                          C
C  d       = data vector (length numdat, units = meters or meters/yr)C
C  llcsn   = array of pairs of baseline endpoint names               C
C            llcsn(2,numst)                                          C
C  G       = Green's function matrix (numdat x 2*numst)              C
C  epsilon = error vector; N(0, sigma)                               C
C            sigma is usually diagonal with variances = stdev**2     C
C  stdev   = vector of standard deviations                           C
C  td      = data weight matrix                                      C
C  tdinv   = inverse of data weight matrix                           C
C                                                                    C
C  SOLUTION TYPES:                                                   C
C  uinn    = inner coordinat solution                                C
C  uout    = outer coordinat solution                                C
C  umod    = model coordinat solution                                C
C            Note: displacement vectors are stored as:               C
C                  u(1) = first station, east component              C
C                  u(2) = first station, north component             C
C                  u(3) = second station, east component             C
C            Units = meters or meters/yr; length = 2*numst           C
C  sta     = char array of station names                             C
C  x,y     = x and y coordinates for all stations                    C
C  staloc  = array holding latitude and longitude of stations        C
C                                                                    C
C  VARIABLES:                                                        C
C  alpha   = angle in degrees between x and x-prime; measured        C
C            counter-clockwise  (used for outer coord solution)      C
C  uapr    = prior model for model coord solution (2*numst)          C
C                                                                    C
C  CONSTANTS:                                                        C
C  The following constants are used for dimensioning the arrays.     C
C  Their values must be specified in the Parameter statement.        C
C  ndim   =  the number of data observations                         C
C  mdim0  =  at least equal to largest expected numst
C  mdim   =  the number of model parameters = 2*numst                C
C                                                                    C
C  The following variables are assigned constant values in the main  C
C  program.                                                          C
C  uot     =  unit number for general output file                    C
C  ust     =  unit number for station coordinate file                C
C  udt     =  unit number for data input files                       C
C  upm     =  unit number for prior model file                       C
C  uplt(1) =  unit number for inner coord plot file (output file)    C
C  uplt(2) =  unit number for outer coord plot file (output file)    C
C  uplt(3) =  unit number for model coord plot file (output file)    C
C  ustin   =  unit number for standard input (terminal)              C
C  ustout  =  unit number for standard output (terminal)             C
C  tol     =  tolerance in deciding when a singular value is zero    C
C  solopt(1) = option for inner coordinate solution                  C
C  solopt(2) = option for outer coordinate solution                  C
C  solopt(3) = option for model coordinate solution                  C
C  plotnm    = array holding output filenames for the inner, outer,  C
C              and model coord solutions respectively                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  MAIN PROGRAM
C
C THE USER MUST SET THE ARRAY DIMENSIONS IN THE FOLLOWING
C PARAMETER STATEMENTS
C

      Integer mdim0, mdim, ndim, datdim, wkdim
      Parameter (mdim0=250,mdim=500,ndim=500,datdim=500,wkdim=1000)
C
      Integer i,j,n,m,p
      Integer uot, ust, udt, upm, ustin, ustout, uplt(5)
      Integer numst, nmstus, ndat(20), totdat
      Real  x(mdim), y(mdim), az(ndim), staloc(mdim,4)
      Real smaj(mdim),smin(mdim),psi(mdim)
      Real stdev(ndim), td(ndim,ndim), tdinv(ndim,ndim)
      Real res(ndim),gpr(ndim,mdim),dpr(ndim),dcalc(ndim)
      Real tol,alpha, rpr(ndim), srpr(ndim)
      Real g(ndim,mdim),ut(ndim,ndim),s(mdim),v(mdim,mdim)
      Real ginv(mdim,ndim),d(ndim),sgn(mdim),uinn(mdim)
      Real qinn(mdim,mdim),wkar(wkdim,wkdim),unull(mdim)
      Real v0(mdim,mdim),uout(mdim),qout(mdim,mdim)
      Real wk(wkdim),sm1(mdim),sm1upt(mdim,ndim)
      Real umod(mdim),qmod(mdim,mdim),uapr(mdim),qapr(mdim,mdim)
      real xorg, yorg, stdevt(datdim), dtemp(datdim), Gtemp(ndim,mdim)
      Character*8 sta(mdim), llcsn(2,ndim), angsn(3,datdim)
      Character*8 gpssn(2,datdim), vlbisn(datdim), azsn(2,datdim)
      character*20 lab(mdim)
      real apr_uinn(mdim),lat(mdim),lon(mdim),rss,nrms
	real*8 llh3(3),xyz0(3)
	real*8 xyz(3,mdim0),dxyz(3,mdim0)
	real*8 qinn3(3*mdim0,3*mdim0), qmod3(3*mdim0,3*mdim0)
	real*8 refxyz(3)

	character*8 refsta
      Character*80 plotnm(5)
      Character*1 solopt(5), used(mdim), dataop(10), newp
      Character*70 header
	character*3 crdsys
      uot     = 10
      ust     = 11
      udt     = 12
      upm     = 13
      uplt(1) = 14
      uplt(2) = 15
      uplt(3) = 16
      uplt(4) = 17
      ustin   = 5
      ustout  = 6
      tol = 0.0001
 

C set-up.....
      Call Ginput(uot,upm,uplt,mdim,dataop,uapr,alpha,solopt,plotnm)
      xorg = 0.0
      yorg = 0.0
      Call Gstalc(uot,ust,mdim,x,y,sta,numst,staloc,xorg,yorg)
      Call Getdata(uot,udt,datdim,ndim,ndat,dataop,llcsn,
     +               angsn,gpssn,vlbisn,azsn,stdev,d,stdevt,dtemp)

      if(2*numst.gt.mdim) then
        print *,'number of unknowns (m) exceeds array dimension mdim'
        print *,'increase mdim in paramter statement'
        stop
      endif

      Call makegtot(ustout,mdim,ndim,datdim,numst,ndat,used,
     +             sta,llcsn,angsn,gpssn,vlbisn,azsn,x,y,Gtemp,G)

C......Note shrinkg has not been updated to include multiple data types
      if( (ndat(2).eq.0).and.(ndat(3).eq.0).and.(ndat(4).eq.0) 
     +                                    .and.(ndat(5).eq.0) ) then
        Call shrinkg(mdim,ndim,numst,nmstus,ndat(1),sta,used,staloc,G)
        write(uot,*) ' '
        write(uot,*) ' '
        write(uot,*) numst-nmstus, '   STATIONS WITHOUT DATA EXCLUDED'
        do i=nmstus+1, numst
          write(uot,*) sta(i)
        end do
      else
        nmstus=numst
      endif

      m=2*nmstus
      n=ndat(1) + ndat(2) + ndat(3) + ndat(4) + ndat(5)

C.....weight data and kernel
      Call Datwgt(ndim,stdev,n,td,tdinv)
      Call wtgd(td,g,d,n,m,ndim,mdim,gpr,dpr)

C.....get singular value decomposition
      call svd(gpr,n,m,ndim,mdim,ut,s,v,wk)
      header ='Singular Values'
      Call Prtvec(uot,mdim,m,header,s)

C.....determine rank of matrix: p
      tol = tol*s(1)
      call getp(s,n,m,mdim,tol,p)
 110  Write(ustout,*) 'number of data                    = ',n
      Write(ustout,*) 'number of unknowns                = ',m
      Write(ustout,*) 'singular values '
      do i = p-2,p+2
        Write(ustout,*) i, s(i)
      end do
      Call getint('Number of non-zero singular values ',p,p)

      Write(uot,*) 'number of data                    = ',n
      Write(uot,*) 'number of unknowns                = ',m
      Write(uot,*) 'number of nonzero singular values = ',p

C get natural inverse and inner coord solution....
      call natinv(ut,s,v,n,m,ndim,mdim,p,ginv,sm1,sm1upt)
      call natest(ginv,s,dpr,n,m,p,ndim,mdim,sm1upt,sgn,uinn)
      call covari(s,v,m,mdim,p,qinn,wk,wkar)
      call vnull(v,m,mdim,p,v0)
C
c print general output for all solutions.....
      Call Calcdt(mdim,ndim,m,n,g,uinn,dcalc)
      Call Studres(ndim, n, ut, p, dpr, rpr, srpr)
      Call Prntoc(uot,ndim,datdim,ndat,p,llcsn,angsn,gpssn,vlbisn,
     +                      azsn,d,dcalc,stdev,res,rpr,srpr,
     +                      totdat,rss)
c 
C print null vectors ?
      If(solopt(4).eq.'y') then
      do 10 j=1,m-p
      do 20 i=1,m
        unull(i) = v0(i,j)
20    continue
        Call Vecplt(uplt(4),mdim,nmstus,sta,staloc,unull,
     +                 smaj,smin,psi)
        write(uplt(4),*) ' '
10    continue
      Endif
C
C print inner coordinate solution.....
      If(solopt(1).eq.'y') Then
          header ='Inner Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,uinn)
          Call Errlps(qinn,m,mdim,smaj,smin,psi)
          header ='variance of inner coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          Call Vecplt(uplt(1),mdim,nmstus,sta,staloc,uinn,
     +                 smaj,smin,psi)
          do i = 1, nmstus
             lab(i*2-1) = sta(i)//' VEL E M/YR '
             lab(i*2  ) = sta(i)//' VEL N M/YR '
             apr_uinn(i*2-1) = 0.0
             apr_uinn(i*2  ) = 0.0
             lat(i) =  (staloc(i,1)+staloc(i,2)/60.)
             lon(i) = -(staloc(i,3)+staloc(i,4)/60.)
          enddo
          nrms = sqrt(rss/(totdat-p))
		write(6,*)'starting conversion to XYZ format'
		write(6,*)'qinn=',qinn(1,1),qinn(1,2),qinn(1,3),qinn(1,4),
&	qinn(1,5),qinn(1,6),qinn(1,7),qinn(1,8),qinn(1,9),qinn(1,10)
		write(6,*)'qinn=',qinn(2,1),qinn(2,2),qinn(2,3),qinn(2,4),
&	qinn(2,5),qinn(2,6),qinn(2,7),qinn(2,8),qinn(2,9),qinn(2,10)
c	Now fill up vertical displacement and associated cov matrix
c	elements with zeros and convert from llh to XYZ coordinates.
	crdsys='XYZ'
c
	do i=1,nmstus
	llh3(1)=dfloat(lat(i))
	llh3(2)=dfloat(lon(i))
	llh3(3)=0.d0
	call llh2xyz(llh3,xyz0)
c	next 3 lines.  position vector after rotation to XYZ.
	xyz(1,i)=xyz0(1)
	xyz(2,i)=xyz0(2)
	xyz(3,i)=xyz0(3)
	enddo
c	Rotate displacement vector to XYZ.
	call rotden(uinn,mdim,dxyz,mdim0,nmstus,lat,lon)
c	Rotate covariance matrix to XYZ.
	call rotcov(qinn,mdim,qinn3,mdim0,nmstus,lat,lon)	
c
	refsta=' '
	refxyz(1)=0.d0
	refxyz(2)=0.d0
	refxyz(3)=0.d0
	open(2,file='result.vel')
		write(6,*)'dxyz(1,1),(2,1),(3,1),(1,2),(2,2),(3,2)='
	write(6,*) dxyz(1,1),dxyz(2,1),dxyz(3,1),dxyz(1,2),dxyz(2,2),
&	dxyz(3,2)
		write(6,*)'qinn3(1,1),qinn3(2,2),qinn3(3,3)='
		write(6,*) qinn3(1,1),qinn3(2,2),qinn3(3,3)
		write(6,*)'qinn3(4,4),qinn3(5,5),qinn3(6,6)='
		write(6,*) qinn3(4,4),qinn3(5,5),qinn3(6,6)
		write(6,*)'calling wrtvel'
	call wrtvel(2,mdim0,sta,xyz,dxyz,qinn3,nmstus,refsta,refxyz,crdsys)
	close(2)
	  call write_hfile(18,m,mdim,lab,lat,lon,
     +                     nrms,apr_uinn,uinn,qinn)
      EndIf
C
C get and print outer coordinate solution.....
      If(solopt(2).eq.'y') then
          call outercrd(v0,m,mdim,p,alpha,uinn,qinn,uout,qout)
          call errlps(qout,m,mdim,smaj,smin,psi)
          header ='Outer Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,uout)
          header ='variance of outer coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          call vecplt(uplt(2),mdim,nmstus,sta,staloc,uout,
     +                 smaj,smin,psi)
      Endif
C
C get and print model coordinate solution.....
      If(solopt(3).eq.'y') then
          call modelcrd(v0,m,mdim,p,uinn,qinn,uapr,qapr,umod,qmod)
          call errlps(qmod,m,mdim,smaj,smin,psi)
          header ='Model Coordinate Solution'
          Call Prtsol(uot,header,nmstus,sta,umod)
          header ='variance of model coordinate solution'
          Call Prtvar(uot,header,m,sta,smaj,smin,psi)
          call vecplt(uplt(3),mdim,nmstus,sta,staloc,umod,
     +                 smaj,smin,psi)
          do i = 1, nmstus
             lab(i*2-1) = sta(i)//' VEL E M/YR '
             lab(i*2  ) = sta(i)//' VEL N M/YR '
             apr_uinn(i*2-1) = 0.0
             apr_uinn(i*2  ) = 0.0
             lat(i) =  (staloc(i,1)+staloc(i,2)/60.)
             lon(i) = -(staloc(i,3)+staloc(i,4)/60.)
          enddo

		write(6,*)'starting conversion to XYZ format'
		write(6,*)'qmod=',qmod(1,1),qmod(1,2),qmod(1,3),qmod(1,4),
&	qmod(1,5),qmod(1,6),qmod(1,7),qmod(1,8),qmod(1,9),qmod(1,10)
		write(6,*)'qmod=',qmod(2,1),qmod(2,2),qmod(2,3),qmod(2,4),
&	qmod(2,5),qmod(2,6),qmod(2,7),qmod(2,8),qmod(2,9),qmod(2,10)
c	Now fill up vertical displacement and associated cov matrix
c	elements with zeros and convert from llh to XYZ coordinates.
	crdsys='XYZ'
c
	do i=1,nmstus
	llh3(1)=dfloat(lat(i))
	llh3(2)=dfloat(lon(i))
	llh3(3)=0.d0
	call llh2xyz(llh3,xyz0)
c	next 3 lines.  position vector after rotation to XYZ.
	xyz(1,i)=xyz0(1)
	xyz(2,i)=xyz0(2)
	xyz(3,i)=xyz0(3)
	enddo
c	Rotate displacement vector to XYZ.
	call rotden(umod,mdim,dxyz,mdim0,nmstus,lat,lon)
c	Rotate covariance matrix to XYZ.
	call rotcov(qmod,mdim,qmod3,mdim0,nmstus,lat,lon)	
c
	refsta=' '
	refxyz(1)=0.d0
	refxyz(2)=0.d0
	refxyz(3)=0.d0
	open(2,file='result.vel')
		write(6,*)'dxyz(1,1),(2,1),(3,1),(1,2),(2,2),(3,2)='
	write(6,*) dxyz(1,1),dxyz(2,1),dxyz(3,1),dxyz(1,2),dxyz(2,2),
&	dxyz(3,2)
		write(6,*)'qmod3(1,1),qmod3(2,2),qmod3(3,3)='
		write(6,*) qmod3(1,1),qmod3(2,2),qmod3(3,3)
		write(6,*)'qmod3(4,4),qmod3(5,5),qmod3(6,6)='
		write(6,*) qmod3(4,4),qmod3(5,5),qmod3(6,6)
		write(6,*)'calling wrtvel'
	call wrtvel(2,mdim0,sta,xyz,dxyz,qmod3,nmstus,refsta,refxyz,crdsys)
	close(2)
      Endif
C
      call getopt('try a different value of p?','y/n','n',newp)
      if(newp.eq.'y') goto 110
      STOP
      END
