c***********************************************************************
      subroutine gstalc(uot,ust,stadim,x,y,sta,numst,staloc,xorg,yorg)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  ALLOWS EITHER MENLO PARK OR HVO FORMAT                          C
C  Get the station input file name and then read the station names C
C  and coordinates.  Store coords in x and y. Store names in sta.  C
C  Also writes station filename and coordinates to output file.    C
C  Modified 8/88 so that:                                          C
C       x,y in kilometers                                          C
C       xorg, yorg are input, however they default to the coord.   C
C             of last station if the input values are 0.0,  0.0    C
C  Local variables/constants:                                      C
C  staloc contains the station locations:                          C
C           staloc(i,1) = latitude degrees of station(1)           C
C           staloc(i,2) = latitude decimal minutes of station(1)   C
C           staloc(i,3) = longitude degrees of station(1)          C
C           staloc(i,4) = longitude decimal minutes of station(1)  C
C  xorg = longitude of the origin of the global coordinate system  C
C  yorg = latitude of the origin of the global coordinate system   C 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer uot, ust, i, stadim, numst, wkdim
      Real x(stadim), y(stadim), staloc(stadim,4)
      real xorg, yorg, dx, dy
      Character*(8)  sta(stadim)
      Character*80 filenm
      Character*1 form
      parameter (wkdim = 1000)
      real xplx(wkdim),xply(wkdim)
C
C Get filename.....
C
      Call getfile2(ust,'Station coordinate file name','in',
     +              'formatted',filenm)
C
C Write filename to output file......
C
      Write(uot,*)' '
      Write(uot,50) filenm
   50 Format(10x,'Station Coordinate File: ',A)
C
      call getopt('Menlo Park or HVO format?','m/h','m',form)
      if(form.eq.'m')
     +    call redstm(uot,ust,stadim,sta,numst,staloc,xplx,xply)
      if(form.eq.'h')
     +    call redsth(uot,ust,stadim,sta,numst,staloc,xplx,xply)
c
      if(yorg.eq.0.0) yorg = xply(numst)
      if(xorg.eq.0.0) xorg = xplx(numst)

      Write(uot,52)
   52 Format(//10x,'station',9x,'xcoord(km)',6x,'ycoord(km)')
c
      do 300   i = 1,numst
      dx=xorg-xplx(i)
      dy=xply(i)
      call poly44(yorg,dy,dx,x(i),y(i))
c.....convert units from meter into kilometer
      x(i)=x(i)/1000.0
      y(i)=y(i)/1000.0            
      write(uot,266) sta(i),x(i),y(i)
  266 format (6x,a8,2x,2f15.4)
c
  300 continue
C
      if(numst.gt.stadim) then
        print *,'number of stations exceeds dimension of arrays'
        print *,'increase dimension of station arrays in parameter',
     +                    'statement'
        stop
      endif

      RETURN
      END
c*********************************************************************
      Subroutine redsth(uot,ust,stadim,sta,numst,staloc,xplx,xply)
CCCCCCCCC0CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Read the coordinate data and write to output file               C
C Count the number of stations = numst.......                     C
C HVO format                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer uot, ust, i, stadim, wkdim, numst
      real xlat, xlon
      Real staloc(stadim,4)
      Character*8  sta(stadim)
      Character*12 stanam
      parameter (wkdim = 1000)
      real xplx(wkdim),xply(wkdim)

      i=1
   60 read(ust,65,end=70) xlat,xlon,stanam
   65 format(20x,f7.4,2x,f8.4,a12)
      write(uot,66) i,stanam,xlat,xlon
   66 format (1x,i3,1x,a12,6x,f7.4,6x,f8.4)

c truncate full station name to 8 characters
      sta(i) = stanam(:8)
c change from decimal degrees to degrees and decimal minutes 
      staloc(i,1) = iint(xlat)     
      staloc(i,2) = 60.0*(xlat - staloc(i,1))
      staloc(i,3) = iint(xlon)
      staloc(i,4) = 60.0*(xlon - staloc(i,3))

c change from decimal degrees to decimal seconds
      xplx(i) = 3600.0*xlon
      xply(i) = 3600.0*xlat

      i = i+1
      Go To 60
   70 Continue
      numst = i-1
 
      Write(uot,*)' '
      Write(uot,*)'numst = ',numst
      Close(unit=ust)

      if(numst.gt.wkdim) then
         print *, 'dimension wkdim too small in gstalc'
         stop      
      endif
c
      return
      end
c
c*********************************************************************
c
      Subroutine redstm(uot,ust,stadim,sta,numst,staloc,xplx,xply)
CCCCCCCCC0CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Read the coordinate data and write to output file               C
C Count the number of stations = numst.......                     C
C Standard Menlo Park geodolit format                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer uot, ust, i, stadim, numst, wkdim, is
      integer ilatd,ilatm,ilond,ilonm
      Real staloc(stadim,4)
      real xlat(3),xlon(3)
      Character*8  sta(stadim)
      parameter (wkdim = 1000)
      real xplx(wkdim),xply(wkdim)
c
      i=1
   60 read(ust,65,end=70) sta(i),ilatd,ilatm,xlat(3),
     +                           ilond,ilonm,xlon(3),is
   65 format(6x,a8,5x,i2,1x,i2,1x,f8.5,1x,i3,1x,i2,1x,
     +       f8.5,1x,i1)
      write(uot,66) i,sta(i),ilatd,ilatm,xlat(3),
     +                       ilond,ilonm,xlon(3)
   66 format (1x,i3,1x,a8,3x,i2,1x,i2,1x,f8.5,4x,i3,1x,
     +       i2,1x,f8.5)
c
c  convert lat and long (dg-min-sec) to radians
c     and to decimal seconds.
c
      xlat(1)=float(ilatd)
      xlat(2)=float(ilatm)
      xlon(1)=float(ilond)
      xlon(2)=float(ilonm)
      call convrt(xlat,xlon,staloc(i,2),staloc(i,4),
     +            xplx(i),xply(i))
      staloc(i,1) = xlat(1)
      staloc(i,3) = xlon(1)
c
      i = i+1
      Go To 60
   70 Continue
      numst = i-1
c
      if(numst.gt.wkdim) then
         print *, 'dimension wkdim too small in gstalc'
         stop      
      endif
c
      Write(uot,*)' '
      Write(uot,*)'numst = ',numst
      Close(unit=ust)
c
      RETURN
      END
c 
c************************************************************
c
      subroutine convrt(xlat,xlon,alat,alon,plx,ply)
c
      integer i
      real sttlat,sttlon,alat,alon,torad,plx,ply
      real xlat(3),xlon(3),todeg(3),tosec(3),tomin(3)
c
c      data todeg/1.0,1.66666666e-2,2.777777777e-4/
c      data torad/.01745329252/
      data tosec/3600.0,60.0,1.0/
      data tomin/0.0,1.0,1.6666666e-2/
c
c  convert minutes and seconds to decimal minutes:
c
      sttlat = 0.0
      sttlon = 0.0
      do 1 i = 1,3
      sttlat = sttlat + xlat(i)*tomin(i)
      sttlon = sttlon + xlon(i)*tomin(i)
    1 continue
      alat = sttlat
      alon = sttlon
c
c  conversion to decimal seconds:
c
      plx = 0.0
      ply = 0.0
      do 2 i = 1,3
      plx = plx + xlon(i)*tosec(i)
      ply = ply + xlat(i)*tosec(i)
    2 continue
c
      return
      end
c
c********************************************************************
c
      subroutine poly44(p1,p2,il,x,y)
c  polyconic projection of point lat=p2, diff long=il from arbitrary
c     central meridian. lat of arbitrary origin is p1. x=dist from cm
c     along lat p2. y=dist from p1 to p2. x,y in meters.
c     p1,p2,and il in seconds.
      real il,la,ip,ipr
      data arcone,esq,la,a0,a2,a4,a6,a8/4.8481368e-6,6.7686580e-3,
     *     6378206.4,6367399.7,32433.888,34.4187,.0454,6.0e-5/
      ip=p2-p1
      sinp2=sin(p2*arcone)
      cosp2=cos(p2*arcone)
      theta=il*sinp2
      a=sqrt(1.0-(esq*(2.*sinp2)))/(la*arcone)
      cot=cosp2/sinp2
      x=(cot*sin(theta*arcone))/(a*arcone)
      ipr=ip*arcone
      pr=((p2+p1)/2.)*arcone
      y= a0*ipr-(a2*cos(2.*pr)*sin(ipr))+(a4*cos(4.*pr)*sin(2.*ipr))-
     * (a6*cos(6.*pr)*sin(3.*ipr))+a8*cos(8.*pr)*sin(4.*ipr)
      return
      end
c
c**********************************************************************
