CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    SUBROUTINE TO SET UP THE GREEN'S FUNCTION MATRIX             C
C      relating displacements at each station to different data   C
C      SUBROUTINES FOR:                                           C
C                      EDM DATA                                   C
C                      TRAINGULATION DATA                         C
C                      ASTRONOMIC AZIMUTHS                        C
C                      GPS DATA                                   C
C                      VLBI DATA                                  C
C    Last modified: July 5, 1990 Astro-Azimuths added             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      Subroutine makegtot(ustout,mdim,ndim,datdim,numst,ndat,used,
     +            sta,llcsn,angsn,gpssn,vlbisn,azsn,x,y,Gtemp,G)
      Integer i, j, ndim, mdim, datdim, ustout, numst, ndat(20)
      Integer nprev
      Real G(ndim,mdim), x(mdim), y(mdim), Gtemp(ndim,mdim)
      Character*(*) sta(mdim), llcsn(2,ndim), angsn(3,datdim),
     +              gpssn(2,datdim), vlbisn(datdim), azsn(2,datdim)
      Character*(1) used(ndim)

C INITIALIZE USE TO 'N'
      Do 10 i=1,numst
              used(i) = 'n'
  10  Continue

c.....For EDM DATA
         if (ndat(1).ne.0) then
            call Makeg(ustout,mdim,ndim,numst,ndat(1),used,
     +                                     sta,llcsn,x,y,G)
         endif

c....For TRIANGULATION DATA
         if (ndat(2).ne.0) then
            call makegagc(ustout,datdim,mdim,ndim,numst,ndat(2),used,
     +                                           sta,angsn,x,y,Gtemp)
            do 20 i=ndat(1) + 1, ndat(1) + ndat(2)
             do 30 j=1,numst*2
               G(i,j) = Gtemp(i-ndat(1),j)  
  30         continue 
  20    continue 
        endif

c....For GPS (RELATIVE VECTOR) DATA
         if (ndat(3).ne.0) then
            call makegps(ustout,datdim,mdim,ndim,numst,ndat(3),used,
     +                                            sta,gpssn,x,y,Gtemp)
            do 40 i=ndat(1) + ndat(2) + 1, ndat(1) + ndat(2) + ndat(3)
             do 50 j=1,numst*2
               G(i,j) = Gtemp(i-ndat(1)-ndat(2),j)  
  50         continue 
  40    continue 
        endif


c....For VLBI (ABSOLUTE VECTOR) DATA
         if (ndat(4).ne.0) then
            call makevlbi(ustout,datdim,mdim,ndim,numst,ndat(4),used,
     .                                            sta,vlbisn,x,y,Gtemp)
            do 60 i=ndat(1)+ndat(2)+ndat(3)+1, 
     .                    ndat(1)+ndat(2)+ndat(3)+ndat(4)
             do 70 j=1,numst*2
               G(i,j) = Gtemp(i-ndat(1)-ndat(2)-ndat(3),j)  
  70         continue 
  60    continue 
        endif

c....For ASTRONOMIC AZIMUTH DATA 
         if (ndat(5).ne.0) then
            call makegaz (ustout,datdim,mdim,ndim,numst,ndat(5),used,
     .                                            sta,azsn,x,y,Gtemp)
            nprev = ndat(1)+ndat(2)+ndat(3)+ndat(4)
            do 80 i= nprev + 1, nprev + ndat(5)
             do 90 j=1,numst*2
               G(i,j) = Gtemp(i - nprev, j)  
  90         continue 
  80    continue 
        endif

      return
      end   



      Subroutine Makeg(ustout,mdim,ndim,numst,numdat,used,
     +                                     sta,llcsn,x,y,G)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  SET UP THE GREEN'S FUNCTION MATRIX                       C
C  relating line changes to displacements at each station   C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer i, j, k, kk, n, nn, ndim, mdim, ustout
      Integer keep(2), SIndex, numdat, numst
      Real G(ndim,mdim), x(mdim), y(mdim), L
      Character*(*) sta(mdim), llcsn(2,ndim)
      Character*(1) used(ndim)

C INITIALIZE G TO ZERO
      Do 300 n=1,numdat
          Do 100 j=1,2*numst
              G(n,j) = 0.0
  100     Continue

C FIND THE MATCHING STATION NAMES
          Do 200 i=1,2
              keep(i) = SIndex(sta,llcsn(i,n),numst)
              used(keep(i)) = 'y'
  200     Continue
          k = keep(1)
          kk = keep(2)
C
C CHECK TO BE SURE THAT A MATCH WAS FOUND
          If ((k*kk).eq.0) Then
             Write(ustout,*)'error in subroutine Makeg'
             Write(ustout,*)'no matching station in sta list ',
     +                      'for nth data ,n= ',n
             Write(ustout,205)llcsn(1,n),llcsn(2,n)
  205        Format('llcsn(i,n) = ',A,2x,A)
             STOP
C
C COMPUTE G
          Else
              nn = n
C             G(nn,2*(k-1)+1) = -sin(az(n))
C             G(nn,2*(k-1)+2) = -cos(az(n))
C             G(nn,2*(kk-1)+1) = sin(az(n))
C             G(nn,2*(kk-1)+2) = cos(az(n))
              L = SQRT( (x(kk)-x(k))**2 + (y(kk)-y(k))**2 )
              G(nn,2*(k-1)+1) = -(x(kk)-x(k))/  L
              G(nn,2*(k-1)+2) = -(y(kk)-y(k)) / L
              G(nn,2*(kk-1)+1) = (x(kk)-x(k)) / L
              G(nn,2*(kk-1)+2) = (y(kk)-y(k)) / L
          EndIf
  300 Continue
      RETURN
      END
 
 


      subroutine makegagc(ustout,datdim,mdim,ndim,numst,
     +                  nac,used,sta,angsn,x,y,G)
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  this subroutine sets up the green's function matrix relating   c 
c  angle changes to displacements at each stations                c
c   nac  =  number of  data for angle change                      c
c   angsn(3,datdim)  = three stations which are related to angle  c
c       change.  angsn(2,datdim) is the station where the         c
c       measurements is made. So the angle is <123=<2             c 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      integer i,j,k,ka,ko,kb,n,ndim,mdim,ustout,datdim
      integer SIndex,numst,nac,keep(3)
      real G(ndim,mdim),x(mdim),y(mdim),lasqr,spr
      real la1,la2,lb1,lb2,lbsqr
      character*(*) sta(mdim),angsn(3,datdim)
      Character*(1) used(ndim)
c
c.....set up a constant for converting the angle change from
c     meters/kilometer to seconds

      spr=162.0/atan(1.0)
c
c.....initialize G to zeros
      do 10 n=1,nac
         do 10 j=1,numst*2
           G(n,j)=0.0
  10  continue
c
c.....do loop to form G
      do 1000 n=1,nac
c
c......find the matching station names
       do 200 i=1,3
           keep(i)=SIndex(sta,angsn(i,n),numst)
           used(keep(i)) = 'y'
  200  continue
       ka=keep(1)
       ko=keep(2)
       kb=keep(3)
c
c.......check to be sure that a matct was found
       if ((ka*ko*kb).eq.0) then
c.........do something
        write(ustout,*) 'error in subroutine makega...'
        write(ustout,*) 'no matching station in sta list',
     +                         'for nth data, n= ',n
        write(ustout,205) angsn(1,n),angsn(2,n),angsn(3,n)
  205   format('angsn(i,n)= ',A,2x,A,2x,A)
        stop           
c
c......compute G
       else
        la1 = x(ka) - x(ko)
        la2 = y(ka) - y(ko)
        lb1 = x(kb) - x(ko)
        lb2 = y(kb) - y(ko)
        lasqr=la1*la1+la2*la2
        lbsqr=lb1*lb1+lb2*lb2

        G(n,2*ka-1)= -(la2/lasqr)*spr
        G(n,2*ka)  = (la1/lasqr)*spr
        G(n,2*ko-1) = -((lb2/lbsqr)-(la2/lasqr))*spr
        G(n,2*ko)   = -((la1/lasqr)-(lb1/lbsqr))*spr
        G(n,2*kb-1)= (lb2/lbsqr)*spr
        G(n,2*kb)  = -(lb1/lbsqr)*spr
       endif
 1000 continue

      return
      end


      subroutine makegaz(ustout,datdim,mdim,ndim,numst,
     +                  naz,used,sta,azsn,x,y,G)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  this subroutine sets up the green's function matrix relating   c 
c  azimuth changes to displacements at each station               c
c   naz  =  number of  azimuth changes                            c
c   azsn(2,datdim)  = two stations related to azimuth change      c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      integer i,j,k,ka,ko,n,ndim,mdim,ustout,datdim
      integer SIndex,numst,naz,keep(2)
      real G(ndim,mdim),x(mdim),y(mdim)
      real la1,la2,lbsqr,lasqr,spr
      character*(*) sta(mdim),azsn(2,datdim)
      Character*(1) used(ndim)
c
c.....set up a constant for converting the angle change from
c     meters/kilometer to seconds

      spr=162.0/atan(1.0)
c
c.....initialize G to zeros
      do 10 n=1,naz
         do 10 j=1,numst*2
           G(n,j)=0.0
  10  continue
c
c.....do loop to form G
      do 1000 n=1,naz
c
c......find the matching station names
       do 200 i=1,2
           keep(i)=SIndex(sta,azsn(i,n),numst)
           used(keep(i)) = 'y'
  200  continue
       ka=keep(1)
       ko=keep(2)
c
c.......check to be sure that a matct was found
       if ((ka*ko).eq.0) then
c.........do something
        write(ustout,*) 'error in subroutine makegaz...'
        write(ustout,*) 'no matching station in sta list',
     +                         'for nth data, n= ',n
        write(ustout,205) azsn(1,n),azsn(2,n)
  205   format('azsn(i,n)= ',A,2x,A)
        stop           
c
c......compute G
       else
        la1 = x(ka) - x(ko)
        la2 = y(ka) - y(ko)
        lasqr=la1*la1+la2*la2
c
        G(n,2*ka-1)= -(la2/lasqr)*spr
        G(n,2*ka)  = (la1/lasqr)*spr
        G(n,2*ko-1) = (la2/lasqr)*spr
        G(n,2*ko)   = -(la1/lasqr)*spr
       endif
 1000 continue

      return
      end

      Subroutine Makegps(ustout,datdim,mdim,ndim,numst,numdat,
     +                              used,sta,gpssn,x,y,G)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  SET UP THE GREEN'S FUNCTION MATRIX FOR GPS OBSERVATIONS  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer i, j, k, kk, n, datdim, ndim, mdim, ustout
      Integer keep(2), SIndex, numdat, numst
      Real G(ndim,mdim), x(mdim), y(mdim)
      Character*(*) sta(mdim), gpssn(2,datdim)
      Character*(1) used(ndim)

C INITIALIZE G TO ZERO
      Do 300 n=1,numdat,2
          Do 100 j=1,2*numst
              G(n,  j) = 0.0
              G(n+1,j) = 0.0
  100     Continue

C FIND THE MATCHING STATION NAMES
          Do 200 i=1,2
              keep(i) = SIndex(sta,gpssn(i,n),numst)
              used(keep(i)) = 'y'
  200     Continue
          k = keep(1)
          kk = keep(2)
C
C CHECK TO BE SURE THAT A MATCH WAS FOUND
          If ((k*kk).eq.0) Then
             Write(ustout,*)'error in subroutine Makegps'
             Write(ustout,*)'no matching station in sta list ',
     +                      'for nth data ,n= ',n
             Write(ustout,205)gpssn(1,n),gpssn(2,n)
  205        Format('gpssn(i,n) = ',A,2x,A)
             STOP
C
C COMPUTE G
          Else
              G(n,2*(k -1)+1) = +1.0
              G(n,2*(kk-1)+1) = -1.0
c
              G(n+1,2*(k -1)+2) = +1.0
              G(n+1,2*(kk-1)+2) = -1.0
          EndIf
  300 Continue
      RETURN
      END

      Subroutine Makevlbi(ustout,datdim,mdim,ndim,numst,numdat,
     +                                  used,sta,vlbisn,x,y,G)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  SET UP THE GREEN'S FUNCTION MATRIX FOR VLBI OBSERVATIONS C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer i, j, k, n, datdim, ndim, mdim, ustout
      Integer SIndex, numdat, numst
      Real G(ndim,mdim), x(mdim), y(mdim)
      Character*(*) sta(mdim), vlbisn(datdim)
      Character*(1) used(ndim)

C INITIALIZE G TO ZERO
      Do 300 n=1,numdat,2
          Do 100 j=1,2*numst
              G(n,  j) = 0.0
              G(n+1,j) = 0.0
  100     Continue

C FIND THE MATCHING STATION NAME
           k = SIndex(sta,vlbisn(n),numst)
           used(k) = 'y'
C
C CHECK TO BE SURE THAT A MATCH WAS FOUND
          If (k.eq.0) Then
             Write(ustout,*)'error in subroutine MakeVLBI'
             Write(ustout,*)'no matching station in sta list ',
     +                      'for nth data ,n= ',n
             Write(ustout,205)vlbisn(n)
  205        Format('vlbisn(n) = ',A)
             STOP
C
C COMPUTE G
          Else
              G(n,  2*(k-1)+1) = +1.0
              G(n+1,2*(k-1)+2) = +1.0
          EndIf
  300 Continue
      RETURN
      END




      subroutine wtgd(td,g,d,n,m,ndim,mdim,gprime,dprime)
c
c.....subroutine wtgd weights the kernel g and the data
c     vector d by the 'data weight' matrix td.  For 
c     independent data this amounts to dividing the data
c     and the kernel by the standard deviation of the data
c
      integer i,j,m,n,mdim,ndim
      real td(ndim,ndim),g(ndim,mdim),d(ndim)
      real gprime(ndim,mdim),dprime(ndim)
c
c.....multiply g by td to form gprime
c         
      call vmulff (td,g,n,n,m,ndim,ndim,gprime,ndim,ier)
c
c.....multiply d by td to form dprime
c
      do 10 i=1,n
       dprime(i) = 0.0
       do 20 j=1,n
        dprime(i) = dprime(i) + td(i,j)*d(j)
 20    continue
 10   continue
c
       return
       end

      Integer Function SIndex(sta,sn,numst)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Searches through the vector of station names sta to find the  C
C one that matches the given station name sn.  Returns the      C
C index number of the matching component of the sta array.      C
C Variables:                                                    C
C sta = char array of station names that you are searching thru C
C sn  = char station name that you are trying to match          C
C numst = number of stations                                    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Character*(*) sta(*), sn
 
      Do 100 k=1,numst
          If (sn.eq.sta(k)) Then
              SIndex = k
              Go To 200
          EndIf
  100 Continue
          print *, 'station  ',sn,'  not found in station file'
          stop
  200 Continue
      RETURN
      END
      Subroutine shrinkg(mdim,ndim,numst,nmstus,numdat,sta,used,
     +                                                 staloc,G)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  SHRINK THE GREEN'S FUNCTION MATRIX                       C
C  to account for stations that have no shots to them       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer i, j, k, n, ndim, mdim, numdat, numst, nmstus
      Real G(ndim,mdim), staloc(mdim,4)
      Character*(*) sta(mdim)
      Character*(1) used(mdim)
      Character*(20) save

      nmstus = numst
      n=1

  300 If (used(n).eq.'y') Then
         n = n + 1

      Else
         nmstus = nmstus - 1
C        REORDER THE STATION LIST
         save = sta(n)
         do 100 j = n, numst-1
           sta(j) = sta(j+1)
           used(j) = used(j+1)
           do 100 k = 1, 4
              staloc(j,k) = staloc(j+1,k)
  100    Continue
           sta(numst) = save
           used(numst) = 'n'

C        REMOVE THE TWO "0" COLUMNS AND SHIFT THE OTHER 
C        COLUMNS TWO COLUMNS TO THE LEFT
         do 200 j = n, nmstus
           do 250 k = 1, numdat
            G(k,2*(j-1)+1) = G(k,2*(j-1)+3) 
            G(k,2*(j-1)+2) = G(k,2*(j-1)+4) 
  250      Continue
  200    Continue

      EndIf

      if(n.le.nmstus) go to 300

      RETURN
      END
