      subroutine studres(ndim, n, ut, p, dpr, rpr, srpr)
c     
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  this subroutine calculates Studentized residuals              c
c       stres = rpr/srpr                                         c
c  input:                                                        c
c     ndim  =  max dimension for data vectors                    c
c     n     =  number of data                                    c
c     ut    =  transpose of the matrix of data vectors from svd  c
c     p     =  the number of non-zero singular values            c
c     dpr   =  data vector (normalized)                          c
c  output:                                                       c
c     rpr   =  normalized residuals                              c
c     srpr  =  standard deviation of residuals                   c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     
      integer ndim, n, p, i, j, wdim, ier
      parameter(wdim=1000)
      real ut(ndim,ndim), dpr(ndim),  rpr(ndim), srpr(ndim)
      real hat(wdim,wdim), imhat(wdim,wdim)

      if(n.gt.wdim) then
        print *, 'dimension wdim is too small in subroutine studres'
      end if

c.....calculate hat = u*ut = ut^T * ut
c     call mxtxf(p, n, ut, ndim, n, hat, wdim)
      call vmulfm(ut, ut, p, n, n, ndim, ndim, hat, wdim, ier)

c.... calculate I - hat
      do 100 i=1,n
        do 100 j=1,n
           if(i.eq.j) then
                   imhat(i,j) = 1.0 - hat(i,j)
           else
                   imhat(i,j) =   - hat(i,j)
           end if
  100 continue        

c.... calculate (I - hat)*dpr = rpr

      Do 120 i=1,n
          rpr(i) = 0.0
          Do 110 j=1,n
              rpr(i) = rpr(i) + imhat(i,j) * dpr(j)
  110     Continue
  120 Continue

c.... calculate sqrt[ diag elements of (I - hat)]

      Do 130 i=1,n
          if(imhat(i,i).le.0.0) then
              srpr(i) = 0.0
          else
              srpr(i) = sqrt(imhat(i,i))
         endif
  130 Continue

      return
      end


      Subroutine Prntoc(uot,ndim,datdim,ndat,p,llcsn,angsn,gpssn,
     +                 vlbisn,azsn,d,dcalc,stdev,res,rpr,srpr,
     +                 totdat,rss)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Prints the observed data, calculated data, and the residuals   C
C for each line pair.                                            C
C Modified 7/88 to include studentized residuals                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer ndim, datdim, uot, ndat(20), p, i, j, k, totdat
      Real d(ndim), dcalc(ndim), res(ndim), stdev(ndim)
      real rpr(ndim), srpr(ndim)
      real nrmres, rss, stdres, eps
      character*(*)  llcsn(2,ndim), angsn(3,datdim), gpssn(2,datdim),
     .               vlbisn(datdim), azsn(2,datdim)
      character*8  star
      star = '  ***  '
 
      rss = 0.0
      Write(uot,*)' '
      Write(uot,*)'Observed and Calculated Data'
      Write(uot,*)' '

      totdat = ndat(1) + ndat(2) + ndat(3) + ndat(4) + ndat(5)
      Do 10 j=1,totdat
       res(j) = d(j) - dcalc(j)
       nrmres = res(j)/stdev(j)
       rss = rss + nrmres**2
   10  Continue

       if(ndat(1).ne.0) then
       Write(uot,*)'Line Length Changes'
       Write(uot,*)' '
       Write(uot,5)
  5    Format(3x,'sta1',5x,'sta2',4x,'observed',2x,'calculated',
     +  1x,'obs-calc',2x,'(o-c)/s',1x,'sigma res.',1x,'stud. res.')

        do 40 j=1,ndat(1)
        nrmres = res(j)/stdev(j)
c.......divide rpr/spr unless spr is essentially zero
        if((srpr(j).lt.1.0e-2).and.(res(j).lt.5.0e-2)) then
         Write(uot,21)(llcsn(i,j),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),star
        else
         stdres = rpr(j)/srpr(j)
         Write(uot,20)(llcsn(i,j),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),stdres
        endif

   40   Continue
        endif

   20   Format(1x,A8,1x,A8,2(1x,1PE9.2),4(1x,E8.1))
   21   Format(1x,A8,1x,A8,2(1x,1PE9.2),3(1x,E8.1),3x,A8)

      if(ndat(2).ne.0) then
      Write(uot,*)' '
      Write(uot,*)'Angle Changes'
      Write(uot,*)' '
      Write(uot,6)
  6   Format(3x,'sta1',5x,'sta2',5x,'sta3',4x,'observed',2x, 
     + 'calculated',1x,'obs-calc',2x,'(o-c)/s',1x,'stud. res.')

        do 45 k=1,ndat(2)
        j=k+ndat(1)
        nrmres = res(j)/stdev(j)
c.......divide rpr/spr unless spr is essentially zero
        if((srpr(j).lt.1.0e-2).and.(res(j).lt.5.0e-2)) then
        Write(uot,31) angsn(1,k),angsn(2,k),angsn(3,k),d(j),dcalc(j),
     +                         res(j),nrmres,star
        else
         stdres = rpr(j)/srpr(j)
         Write(uot,30) angsn(1,k),angsn(2,k),angsn(3,k),d(j),dcalc(j),
     +                        res(j),nrmres,stdres
        endif
   45   Continue
        endif

   30   Format(1x,A8,1x,A8,1x,A8,2(1x,1PE9.2),3(1x,E8.1))
   31   Format(1x,A8,1x,A8,1x,A8,2(1x,1PE9.2),2(1x,E8.1),3x,A8)

       if(ndat(3).ne.0) then
       Write(uot,*)' '
       Write(uot,*)'GPS Vectors'
       Write(uot,*)' '
       Write(uot,5)

       do 50 k=1,ndat(3)
        j=k+ndat(1)+ndat(2)
        nrmres = res(j)/stdev(j)
c.......divide rpr/spr unless spr is essentially zero
        if((srpr(j).lt.1.0e-2).and.(res(j).lt.5.0e-2)) then
         Write(uot,21)(gpssn(i,k),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),star
        else
         stdres = rpr(j)/srpr(j)
         Write(uot,20)(gpssn(i,k),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),stdres
        endif
   50   Continue
        endif

       if(ndat(4).ne.0) then
       Write(uot,*)' '
       Write(uot,*)'VLBI Vectors'
       Write(uot,*)' '
       Write(uot,7)
  7    Format(3x,'sta1',4x,'observed',2x,'calculated',
     +  1x,'obs-calc',2x,'(o-c)/s',1x,'sigma res.',1x,'stud. res.')

       do 55 k=1,ndat(4)
        j=k+ndat(1)+ndat(2)+ndat(3)
        nrmres = res(j)/stdev(j)
c.......divide rpr/spr unless spr is essentially zero
        if((srpr(j).lt.1.0e-2).and.(res(j).lt.5.0e-2)) then
         Write(uot,36) vlbisn(k),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),star
        else
         stdres = rpr(j)/srpr(j)
         Write(uot,35) vlbisn(k),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),stdres
        endif
   55   Continue
        endif
   35   Format(1x,A8,2(1x,1PE9.2),4(1x,E8.1))
   36   Format(1x,A8,2(1x,1PE9.2),3(1x,E8.1),3x,A8)

       if(ndat(5).ne.0) then
       Write(uot,*)' '
       Write(uot,*)'Azimuth Changes'
       Write(uot,*)' '
       Write(uot,5)

       do 65 k=1,ndat(5)
        j=k+ndat(1)+ndat(2)+ndat(3)+ndat(4)
        nrmres = res(j)/stdev(j)
c.......divide rpr/spr unless spr is essentially zero
        if((srpr(j).lt.1.0e-2).and.(res(j).lt.5.0e-2)) then
         Write(uot,21)(azsn(i,k),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),star
        else
         stdres = rpr(j)/srpr(j)
         Write(uot,20)(azsn(i,k),i=1,2),d(j),dcalc(j),res(j),
     +                                nrmres,srpr(j),stdres
        endif
   65   Continue
        endif

      write(uot,*) ' '
      write(uot,*) ' '
      write(uot,*) 'Note: *** indicates standard deviation of residual',
     +             'is numerically zero'

      write(uot,*) ' '
      write(uot,*) 'Residual Sum of Squares = sum[((o-c)/sigma)**2]'
      write(uot,*) 'Residual Sum of Squares = ',rss
      write(uot,*) ' '
      write(uot,*) 'Mean Square Pure Error  = sum[((o-c)/sigma)**2]
     +/(n-p)'
      if(totdat-p.ne.0) then
       write(uot,*) 'Mean Square Pure Error  = ',rss/(totdat-p)
       write(uot,*) ' '
      else
       write(uot,*) ' n-p = 0; can not divide '
      endif
      RETURN
      END


      Subroutine Calcdt(mdim,ndim,m,n,G,mest,dcalc)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Calculates the data vector given the green's function G and the C
C model estimate mest.                                            C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer mdim, ndim, m, n, i, j
      Real G(ndim,mdim), mest(mdim), dcalc(ndim)
 
      Do 20 i=1,n
          dcalc(i) = 0.0
          Do 10 j=1,m
              dcalc(i) = dcalc(i) + G(i,j) * mest(j)
   10     Continue
   20 Continue
      RETURN
      END



      Subroutine Prtsol(uot,header,numst,sta,mest)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Prints the solution to the output file.                        C
C Prints the station names, and the x and y calculated disp.     C
C Note + x-direction is East; + y-direction is North.            C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer  i, k,  uot, numst, lenh
      Real mest(*), xcomp, ycomp
      Character*(*) sta(*)
      Character*(*) header
c
      lenh = lentrue(header)
      Write(uot,*)' '
      Write(uot,5)header(1:lenh)
    5 Format(A)
      Write(uot,*)' '
c
      Write(uot,*)'Station displacements in same units as data'
      Write(uot,7)
    7 Format(5x,'num',3x,'station',6x,' east',8x,' north')
      Do 20 i=1,numst
          k = 2*(i-1)
          xcomp = mest(k+1)
          ycomp = mest(k+2)
          Write(uot,10)i,sta(i),xcomp,ycomp
   10     Format(5x,I3,3x,A8,3x,1pE10.3,3x,1pE10.3)
   20 Continue
      RETURN
      END



 
      Subroutine Prtvar(uot,header,m,sta,smaj,smin,psi)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Print the variance info to the output file.                 C
C Called once for each solution type.                         C
C Prints the string "header" at the top.                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer m,  uot, lenh, i
      Real smaj(*), smin(*), psi(*), smj, smn
      Character *(*) header, sta(*)
 
      lenh = lentrue(header)
      Write(uot,*)' '
      Write(uot,5)header(1:lenh)
    5 Format(A)
C
      write(uot,*) ' '
      Write(uot,*)'Smaj & Smin are axes of 95% confidence ellipse',
     +                                 ' in units of data'
      Write(uot,*) 'Psi is orientation of Smaj in degrees',
     +                         ' clockwise from North'
      Write(uot,7)
      write(uot,*) ' '
      write(uot,*) ' '
    7 Format(5x,'num',2x,'station',9x,'smaj',9x,'smin',9x,'psi')
      Do 20 i=1,m/2
          smj = smaj(i)
          smn = smin(i)
          Write(uot,10) i,sta(i),smj,smn,psi(i)
   10     Format(4x,i3,3x,A8,3(3x,1pe10.3))
   20 Continue
      RETURN
      END



      Subroutine Prtvec(uot,vecdim,numvec,header,vec)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Prints a vector of length numvec out to the output file.     C
C The string "header" is printed at the top.                   C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer uot,vecdim,numvec,lenh
      Real vec(vecdim)
      Character*(*) header
 
      lenh = lentrue(header)
      Write(uot,*)' '
      Write(uot,5)header(1:lenh)
    5 Format(A)
      Do 10 i=1,numvec
          Write(uot,*) i,vec(i)
   10 Continue
      RETURN
      END


      Subroutine vecplt(uplt,stadim,numst,sta,staloc,mest,a,b,az)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Prints the solution to an output file for plotting with QPLOT  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      Integer  i,k,uplt,numst,stadim
      real mest(stadim),a(stadim),b(stadim),az(stadim)
      Real staloc(stadim,4), east, north
      Character*(8) sta(stadim)


      Do 20 i=1,numst
          k = 2*(i-1)
          east  = mest(k+1)
          north = mest(k+2)
          Write(uplt,10) staloc(i,1),staloc(i,2),staloc(i,3),
     +          staloc(i,4),north,east,a(i),b(i),az(i),sta(i)
   10     Format(2(f5.0,f6.2),2(1pe10.2),2(1pe9.2),0pf6.1,1x,a8)
   20 Continue
      RETURN
      END

